//
// nt4ga_stat.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

#ifndef NT4GA_STAT_H_
#define NT4GA_STAT_H_

#include "nt_util.h"
#include "common_adapter_defs.h"

#define NT_MAX_COLOR_FLOW_STATS    0x400
//
// Structs
//

struct ColorCounters {
  uint64_t colorPackets;
  uint64_t colorBytes;
  uint8_t tcp_flags;
};

struct HostBufferCounters {
  uint64_t flushPackets;
  uint64_t dropPackets;
  uint64_t fwdPackets;
  uint64_t dbsDropPackets;
  uint64_t flushBytes;
  uint64_t dropBytes;
  uint64_t fwdBytes;
  uint64_t dbsDropBytes;
};


struct PortCounters_v2 {
  // Rx/Tx common port counters
  uint64_t drop_events;
  uint64_t pkts;
  // FPGA counters
  uint64_t octets;
  uint64_t broadcast_pkts;
  uint64_t multicast_pkts;
  uint64_t unicast_pkts;
  uint64_t pkts_alignment;
  uint64_t pkts_code_violation;
  uint64_t pkts_crc;
  uint64_t undersize_pkts;
  uint64_t oversize_pkts;
  uint64_t fragments;
  uint64_t jabbers_not_truncated;
  uint64_t jabbers_truncated;
  uint64_t pkts_64_octets;
  uint64_t pkts_65_to_127_octets;
  uint64_t pkts_128_to_255_octets;
  uint64_t pkts_256_to_511_octets;
  uint64_t pkts_512_to_1023_octets;
  uint64_t pkts_1024_to_1518_octets;
  uint64_t pkts_1519_to_2047_octets;
  uint64_t pkts_2048_to_4095_octets;
  uint64_t pkts_4096_to_8191_octets;
  uint64_t pkts_8192_to_max_octets;
  uint64_t mac_drop_events;
  uint64_t pkts_lr;
  // Rx only port counters
  uint64_t duplicate;
  uint64_t pkts_ip_chksum_error;
  uint64_t pkts_udp_chksum_error;
  uint64_t pkts_tcp_chksum_error;
  uint64_t pkts_giant_undersize;
  uint64_t pkts_baby_giant;
  uint64_t pkts_not_isl_vlan_mpls;
  uint64_t pkts_isl;
  uint64_t pkts_vlan;
  uint64_t pkts_isl_vlan;
  uint64_t pkts_mpls;
  uint64_t pkts_isl_mpls;
  uint64_t pkts_vlan_mpls;
  uint64_t pkts_isl_vlan_mpls;
  uint64_t pkts_no_filter;
  uint64_t pkts_dedup_drop;
  uint64_t pkts_filter_drop;
  uint64_t pkts_overflow;
  uint64_t pkts_dbs_drop;
  uint64_t octets_no_filter;
  uint64_t octets_dedup_drop;
  uint64_t octets_filter_drop;
  uint64_t octets_overflow;
  uint64_t octets_dbs_drop;
  uint64_t ipft_first_hit;
  uint64_t ipft_first_not_hit;
  uint64_t ipft_mid_hit;
  uint64_t ipft_mid_not_hit;
  uint64_t ipft_last_hit;
  uint64_t ipft_last_not_hit;
};

//
//
//
struct PortCounters_vswitch_v1 {
  // Rx/Tx common port counters
  uint64_t octets;
  uint64_t pkts;
  uint64_t drop_events;
  uint64_t qos_drop_octets;
  uint64_t qos_drop_pkts;
};


//
//
//
struct FlmCounters_v1 {
  uint64_t current;
  uint64_t learn_done;
  uint64_t learn_ignore;
  uint64_t learn_fail;
  uint64_t unlearn_done;
  uint64_t unlearn_ignore;
  uint64_t auto_unlearn_done;
  uint64_t auto_unlearn_ignore;
  uint64_t auto_unlearn_fail;
  uint64_t timeout_unlearn_done;
};


//
//
//
struct Nt4gaStat_s
{
  NthwStat_t* mpNthwStat;
  NthwRmc_t* mpNthwRmc;
  struct nt_dma_s* pStatDma;
  uint32_t* pStatDmaVirtual;
  uint32_t nStatSize;
  //
  uint64_t lastTimestamp;
  //
  //
  int mnRxHostBuffers;
  int mnTxHostBuffers;
  //
  int mnRxPorts;
  int mnTxPorts;
  //
//  struct ColorCounters* mpStatBufColorCounters;
//  struct HostBufferCounters* mpStatBufHbCounters;

  struct ColorCounters* mpStatStructsColor;
  struct ColorCounters aStatStructsColorBase[NT_MAX_COLOR_FLOW_STATS]; // For calculating increments between stats polls

  union {
    //Port counters for VSWITCH/inline
    struct {
      struct PortCounters_vswitch_v1* mpStatStructsPortRx;
      struct PortCounters_vswitch_v1* mpStatStructsPortTx;
    } virt;
    struct {
      struct PortCounters_v2* mpStatStructsPortRx;
      struct PortCounters_v2* mpStatStructsPortTx;
    } cap;
  };

  struct HostBufferCounters* mpStatStructsHb;

  struct FlmCounters_v1* mpStatStructsFlm;

  //
  // Rx/Tx totals:
  //
  uint64_t nTotalsResetTimestamp; // timestamp for last totals reset
  //
  uint64_t aPortRxOctetsTotal[NUM_ADAPTER_PORTS_MAX];
  uint64_t aPortRxOctetsBase[NUM_ADAPTER_PORTS_MAX]; // Base is for calculating increments between statistics reads
  //
  uint64_t aPortRxPacketsTotal[NUM_ADAPTER_PORTS_MAX];
  uint64_t aPortRxPacketsBase[NUM_ADAPTER_PORTS_MAX];
  //
  uint64_t aPortRxDropsTotal[NUM_ADAPTER_PORTS_MAX];
  uint64_t aPortRxDropsBase[NUM_ADAPTER_PORTS_MAX];
  //
  uint64_t aPortTxOctetsTotal[NUM_ADAPTER_PORTS_MAX];
  uint64_t aPortTxOctetsBase[NUM_ADAPTER_PORTS_MAX];
  //
  uint64_t aPortTxPacketsBase[NUM_ADAPTER_PORTS_MAX];
  uint64_t aPortTxPacketsTotal[NUM_ADAPTER_PORTS_MAX];
  //
  uint64_t aPortTxDropsBase[NUM_ADAPTER_PORTS_MAX];
  uint64_t aPortTxDropsTotal[NUM_ADAPTER_PORTS_MAX];
  //
};

typedef struct Nt4gaStat_s Nt4gaStat_t;


//
//
//
int Nt4gaStat_init(struct AdapterInfo_s* pAdapterInfo);
int Nt4gaStat_setup(struct AdapterInfo_s* pAdapterInfo);
int Nt4gaStat_stop(struct AdapterInfo_s* pAdapterInfo);
//
int Nt4gaStat_dump(struct AdapterInfo_s* pAdapterInfo, FILE* pfh);

int Nt4gaStat_collect(struct AdapterInfo_s* pAdapterInfo, Nt4gaStat_t* pNt4gaStat);

//
//
//
#endif // NT4GA_STAT_H_

//
// EOF
//
