//
// fpga_model.hpp
//

#ifndef FPGA_MODEL_HPP_
#define FPGA_MODEL_HPP_

//
//
//
#ifdef __cplusplus
#include <cstdint>
#else
#ifndef __KERNEL__
#include <unistd.h>
#include <stdint.h>
#include <inttypes.h>
#else
#include <linux/types.h>
#endif // __KERNEL__
#endif // __cplusplus

//
//
//
enum FpgaBusType {
  BUS_TYPE_UNKNOWN = 0, // Unknown/uninitialized - keep this as the first enum element
  BUS_TYPE_BAR,
  BUS_TYPE_PCI,
  BUS_TYPE_CCIP,
  BUS_TYPE_RAB0,
  BUS_TYPE_RAB1,
  BUS_TYPE_RAB2,
  BUS_TYPE_NMB,
  BUS_TYPE_NDM,
  BUS_TYPE_SPI0,
  BUS_TYPE_SPI = BUS_TYPE_SPI0,
};

typedef enum FpgaBusType FpgaBusType_t;

//
//
//
enum FpgaRegisterType {
  REGISTER_TYPE_UNKNOWN = 0, // Unknown/uninitialized - keep this as the first enum element
  REGISTER_TYPE_RW,
  REGISTER_TYPE_RO,
  REGISTER_TYPE_WO,
  REGISTER_TYPE_RC1,
  REGISTER_TYPE_MIXED,
};

typedef enum FpgaRegisterType FpgaRegisterType_t;

//
//
//
struct FpgaFieldInit {
  int id;
  //
  uint16_t bw;
  uint16_t low;
  uint64_t resetVal;
};

typedef struct FpgaFieldInit FpgaFieldInit_t;

//
//
//
struct FpgaRegisterInit {
  int id;
  //
  uint32_t addrRel;
  uint16_t bw;
  FpgaRegisterType_t type;
  uint64_t resetVal;
  //
  int nbFields;
  struct FpgaFieldInit* fields;
};

typedef struct FpgaRegisterInit FpgaRegisterInit_t;

//
//
//
struct FpgaModuleInit {
  int id;
  int instance;
  //
  int defId;
  int majorVersion;
  int minorVersion;
  //
  FpgaBusType_t busId;
  uint32_t addrBase;
  //
  int nbRegisters;
  struct FpgaRegisterInit* registers;
};

typedef struct FpgaModuleInit FpgaModuleInit_t;

//
//
//
struct FpgaProdParam {
  const int paramId;
  const int paramValue;
};

typedef struct FpgaProdParam FpgaProdParam_t;

//
//
//
struct FpgaProdInit {
  int fpgaItemId;
  int fpgaProductId;
  int fpgaVersion;
  int fpgaRevision;
  int fpgaPatchNo;
  int fpgaBuildNo;
  uint32_t fpgaBuildTime;
  //
  int nbProdParams;
  struct FpgaProdParam* productParams;
  //
  int nbModules;
  struct FpgaModuleInit* modules;
};

typedef struct FpgaProdInit FpgaProdInit_t;

//
//
//

#endif // FPGA_REGISTER_HPP_

//
// EOF
//
