//
// nthw_gmf.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */


#ifndef __NTHW_GMF_H__
#define __NTHW_GMF_H__

//
//
//
enum GMF_STATUS_MASK {
  GMF_STATUS_MASK_DATA_UNDERFLOWED = 1,
  GMF_STATUS_MASK_IFG_ADJUSTED
};

//
//
//
struct NthwGmf
{
  Fpga_t* mpFpga;
  Module_t* mpModGmf;
  int mnInstance;
  //

  Register_t* mpCtrl;
  Field_t* mpCtrlEnable;
  Field_t* mpCtrlIfgEnable;
  Field_t* mpCtrlIfgTxNowAlways;
  Field_t* mpCtrlIfgTxOnTsAlways;
  Field_t* mpCtrlIfgTxOnTsAdjustOnSetClock;
  Field_t* mpCtrlIfgAutoAdjustEnable;

  Register_t* mpSpeed;
  Field_t* mpSpeedIfgSpeed;

  Register_t* mpIfgClockDelta;
  Field_t* mpIfgClockDeltaDelta;

  Register_t* mpIfgClockDeltaAdjust;
  Field_t* mpIfgClockDeltaAdjustDelta;

  Register_t* mpIfgMaxAdjustSlack;
  Field_t* mpIfgMaxAdjustSlackSlack;

  Register_t* mpDebugLaneMarker;
  Field_t* mpDebugLaneMarkerCompensation;

  Register_t* mpStatSticky;
  Field_t* mpStatStickyDataUnderflowed;
  Field_t* mpStatStickyIfgAdjusted;

  Register_t* mpStatNextPkt;
  Field_t* mpStatNextPktNs;

  Register_t* mpStatMaxDelayedPkt;
  Field_t* mpStatMaxDelayedPktNs;

  int mnParamGmfIfgSpeedMul;
  int mnParamGmfIfgSpeedDiv;

  bool mAdministrativeBlock; // used to enforce license expiry
};

typedef struct NthwGmf NthwGmf_t;
typedef struct NthwGmf NthwGmf;

//
//
//
NthwGmf_t* NthwGmf_new(void);
void NthwGmf_delete(NthwGmf_t* p);
int NthwGmf_init(NthwGmf_t* p, Fpga_t* pFpga, int nInstance);
//
void NthwGmf_setEnable(NthwGmf_t* p, bool enable);
void NthwGmf_setIfgEnable(NthwGmf_t* p, bool enable);
//
void NthwGmf_setTxNowAlwaysEnable(NthwGmf_t* p, bool enable);
void NthwGmf_setTxOnTsAlwaysEnable(NthwGmf_t* p, bool enable);
void NthwGmf_setTxOnTsAdjustOnSetClock(NthwGmf_t* p, bool enable);
void NthwGmf_setIfgAutoAdjustEnable(NthwGmf_t* p, bool enable);
//
int NthwGmf_getIfgSpeedBitWidth(NthwGmf_t* p);
//
int NthwGmf_setIfgSpeedRaw(NthwGmf_t* p, uint64_t nSpeedVal);
int NthwGmf_setIfgSpeedBits(NthwGmf_t* p, const uint64_t nRateLimitBits, const uint64_t nLinkSpeed);
int NthwGmf_setIfgSpeedPercent(NthwGmf_t* p, const double fRateLimitPercent);
//
void NthwGmf_setDelta(NthwGmf_t* p, uint64_t delta);
void NthwGmf_setDeltaAdjust(NthwGmf_t* p, uint64_t deltaAdjust);
void NthwGmf_setSlack(NthwGmf_t* p, uint64_t slack);
void NthwGmf_setCompensation(NthwGmf_t* p, uint32_t compensation);
//
uint32_t NthwGmf_getStatusSticky(NthwGmf_t* p);
void NthwGmf_setStatusSticky(NthwGmf_t* p, uint32_t status);
//
uint64_t NthwGmf_getStatNextPktNs(NthwGmf_t* p);
uint64_t NthwGmf_getStatMaxPkDelayedtNs(NthwGmf_t* p);

void NthwGmf_administrativeBlock(NthwGmf_t* p); // used to enforce license expiry blocking

//
//
//
#endif // __NTHW_GMF_H__

//
// EOF
//
