//
// nthw_gpio_phy.h
//

#ifndef NTHW_GPIO_PHY_H_
#define NTHW_GPIO_PHY_H_

/*
 * %NT_SOFTWARE_LICENSE%
 */



#define GPIO_PHY_INTERFACES (2)

typedef struct {
  Field_t*    CfgFldLpMode;  // Cfg Low Power Mode
  Field_t*    CfgInt;        // Cfg Port Interrupt
  Field_t*    CfgReset;      // Cfg Reset
  Field_t*    CfgModPrs;     // Cfg Module Present
  Field_t*    CfgPLLInt;     // Cfg PLL Interrupt
  Field_t*    CfgPortRXLOS;  // Emulate Cfg Port RXLOS

  Field_t*    GpioFldLpMode; // Gpio Low Power Mode
  Field_t*    GpioInt;       // Gpio Port Interrupt
  Field_t*    GpioReset;     // Gpio Reset
  Field_t*    GpioModPrs;    // Gpio Module Present
  Field_t*    GpioPLLInt;    // Gpio PLL Interrupt
  Field_t*    GpioPortRXLOS; // Emulate Gpio Port RXLOS
} GpioPhyFields_t ;

struct NthwGpioPhy
{
  Fpga_t* mpFpga;
  //
  Module_t* mpModGpioPhy;
  //
  int mnInstance;

  // Registers
  Register_t* mpRegConfig;
  Register_t* mpRegGpio;

  // Fields
  GpioPhyFields_t mpaFields[GPIO_PHY_INTERFACES] ;

};

typedef struct NthwGpioPhy NthwGpioPhy_t;
typedef struct NthwGpioPhy NthwGpioPhy;

//
//
//
NthwGpioPhy_t* NthwGpioPhy_new(void);
void NthwGpioPhy_delete(NthwGpioPhy_t* p);
int NthwGpioPhy_init(NthwGpioPhy_t* p, Fpga_t* pFpga, int nInstance);
//

//
//
//
bool NthwGpioPhy_isLowPowerEnabled(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isInterruptSet(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isReset(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isModulePresent(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isPllInterruptSet(NthwGpioPhy_t* p, uint8_t ifNo);
bool NthwGpioPhy_isPortRXLOS(NthwGpioPhy_t* p, uint8_t ifNo);
void NthwGpioPhy_setLowPower(NthwGpioPhy_t* p, uint8_t ifNo, bool enable);
void NthwGpioPhy_setReset(NthwGpioPhy_t* p, uint8_t ifNo, bool enable);
void NthwGpioPhy_setPortRXLOS(NthwGpioPhy_t* p, uint8_t ifNo, bool enable);
void NthwGpioPhy_setCFGDefaultValues(NthwGpioPhy_t* p, uint8_t ifNo);


//
//
//
#endif // NTHW_GPIO_PHY_H_

//
// EOF
//
