/*
 * nthw_mac.h
 *
 */

/*
 * %NT_SOFTWARE_LICENSE%
 */

#ifndef __NTHW_MAC_H__
#define __NTHW_MAC_H__

enum NthwMAC_LEDMode {
      LED_AUTO=0x00,
      LED_ON=0x01,
      LED_OFF=0x02,
      LED_PORTID=0x03
};

struct NthwMac {
  //
  Fpga_t* mpFpga;
  //
  Module_t* mpModMac;
  //
  int mnInstance;

  // LED mode (NT_PORT_CONTROL)
  Field_t*    mpFldLEDMode;

  // INT
  Field_t*    mpFldIntEnable;
  Field_t*    mpFldIntMaxPace;

  // RX MAC Enable (RX_CONFIG)
  Field_t*    mpFldTsEop;
  Field_t*    mpFldMinRxFrame;
  Field_t*    mpFldRxEnable;
  Field_t*    mpFldClearHostCounters;
  Field_t*    mpFldClearMacCounters;

  // RXLOS MOD MASK (RX_CONFIG2)
  Field_t*    mpFldRxLosMaskLinkEnable;
  Field_t*    mpFldRxLosMaskIntEnable;
  Field_t*    mpFldRxLosMaskRstEnable;

  Field_t*    mpFldModAbsMaskLinkEnable;
  Field_t*    mpFldModAbsMaskIntEnable;
  Field_t*    mpFldModAbsMaskRstEnable;

  // TFG ADDR
  Field_t*    mpFldTfgAddrWriteRAMAdr;
  Field_t*    mpFldTfgAddrReadEnable;
  Field_t*    mpFldTfgAddrReadDone;

  // TFG DATA
  Field_t*    mpFldTfgDataLength;
  Field_t*    mpFldTfgDataGap;
  Field_t*    mpFldTfgDataId;

  // TFG CTRL
  Field_t*    mpFldTfgWrap;
  Field_t*    mpFldTfgRestart;
  Field_t*    mpFldTfgEnable;
  Field_t*    mpFldTfgTimeMode;
  Field_t*    mpFldTfgIdPos;
  Field_t*    mpFldTfgIdEna;
  Field_t*    mpFldTfgTxActive;

  // TFG REPETITION
  Field_t*    mpFldTfgRepetitionCount;

  // RAW ADDR
  Field_t*    mpFldRawAddrWriteRAMAdr;
  Field_t*    mpFldRawAddrReadEnable;
  Field_t*    mpFldRawAddrReadDone;

  // RAW DATA
  Field_t*    mpFldRawData;

  // RAW CTRL
  Field_t*    mpFldRawWrap;
  Field_t*    mpFldRawRestart;
  Field_t*    mpFldRawTgEnable;
  Field_t*    mpFldRawOverWrLaneMarkers;
  Field_t*    mpFldRawTgActive;

  // RAW REPETITION
  Field_t*    mpFldRawRepetitionCount;

  // TX_CONFIG
  Field_t*    mpFldTxHostEnable;
  Field_t*    mpFldTxMacLoopback;

  Field_t*    mpFldTxMacTxConfigPcsRemoteFaultDis;
  Field_t*    mpFldTxMacTxConfigPcsIdleDis;

  // RX MAC STAT
  Register_t* mpRegRxMacStatFrames;
  Register_t* mpRegRxMacStatBytes;
  Field_t*    mpFldRxMacStatFrames;
  Field_t*    mpFldRxMacStatBytes;
  Field_t*    mpFldRxMacStatCRC;
  Field_t*    mpFldRxMacStatCV;
  Field_t*    mpTrigger;

  // LINK SUMMARY
  Register_t* mpRegLinkSummary;
  Field_t*    mpFldLinkSummaryAbs;
  Field_t*    mpFldLinkSummaryNtPhyLinkState;
  Field_t*    mpFldLinkSummaryLHAbs;
  Field_t*    mpFldLinkSummaryLLPhyLinkState;
  Field_t*    mpFldLinkSummaryLinkDownCnt;
  Field_t*    mpFldLinkSummaryGBoxIntErr;
  Field_t*    mpFldLinkSummaryLHLocalFault;
  Field_t*    mpFldLinkSummaryLHRemoteFault;
  Field_t*    mpFldLinkSummaryLHRxLos;
  Field_t*    mpFldLinkSummaryLHGlbAlarmN;
  Field_t*    mpFldLinkSummaryLocalFault;
  Field_t*    mpFldLinkSummaryRemoteFault;
  Field_t*    mpFldLinkSummaryRxLos;
  Field_t*    mpFldLinkSummaryGlbAlarmN;
};

typedef struct NthwMac NthwMac_t;
typedef struct NthwMac NthwMac;

NthwMac_t* NthwMac_new(void);
void NthwMac_delete(NthwMac_t* p);
int NthwMac_init(NthwMac_t *p, Fpga_t *pFpga, int nInstance);

//int NthwMAC_getFpgaProductId(NthwMac_t* p);

void NthwMAC_setLEDMode(NthwMac_t* p, unsigned int mode);
void NthwMAC_rxEnable(NthwMac_t* p, bool enable);
void NthwMAC_clearHostCounters(NthwMac_t* p);
void NthwMAC_clearMacCounters(NthwMac_t* p);

void NthwMAC_setRxLosMaskLink(NthwMac_t* p, bool enable);
void NthwMAC_setRxLosMaskInt(NthwMac_t* p, bool enable);
void NthwMAC_setRxLosMaskRst(NthwMac_t* p, bool enable);

void NthwMAC_setModAbsMaskLink(NthwMac_t* p, bool enable);
void NthwMAC_setModAbsMaskInt(NthwMac_t* p, bool enable);
void NthwMAC_setModAbsMaskRst(NthwMac_t* p, bool enable);

void NthwMAC_TxHostEnable(NthwMac_t* p, bool enable);
void NthwMAC_TxMacLoopbackEnable(NthwMac_t* p, bool enable);
void NthwMAC_TxMacPcsRemoteFaultDis(NthwMac_t* p, bool disable);
void NthwMAC_TxMacPcsIdleDis(NthwMac_t* p, bool disable);

void NthwMAC_TfgTxStart(NthwMac_t* p,
                        uint32_t repetition,
                        uint32_t size,
                        uint32_t pkt_gap);
void NthwMAC_TfgTxStop(NthwMac_t* p);
bool NthwMAC_isTfgTransmitting(NthwMac_t* p);

void NthwMAC_RawTxStart(NthwMac_t* p,
                        uint32_t repetition,
                        uint32_t wrap);
void NthwMAC_RawTxStop(NthwMac_t* p);
bool NthwMAC_isRawTxTransmitting(NthwMac_t* p);
void NthwMAC_RawDataWrite(NthwMac_t* p,
                          uint32_t addr,
                          uint64_t val);

void NthwMAC_readMacCounters(NthwMac_t* p,
                             uint64_t *packets,
                             uint64_t *bytes,
                             uint32_t *cv_errors,
                             uint32_t *fcs_errors);

void NthwMAC_setTsEop(NthwMac_t* p, uint32_t bitEop);
void NthwMAC_setRxMinFrame(NthwMac_t* p, uint32_t min_rx_frame);

void NthwMAC_setIntEnable(NthwMac_t* p, bool enable);
void NthwMAC_setIntMaxPace(NthwMac_t* p, uint32_t max_pace);

void NthwMAC_getLinkSummary(NthwMac_t* p,
                            uint32_t* pModAbs,
                            uint32_t* pLatchHighModAbs,
                            uint32_t* pRxLos,
                            uint32_t* pLatchHighRxLos,
                            uint32_t* pLinkState,
                            uint32_t* pLatchLowLinkState,
                            uint32_t* pLinkDownCnt,
                            uint32_t* pLocalFault,
                            uint32_t* pLatchHighLocalFault,
                            uint32_t* pRemoteFault,
                            uint32_t* pLatchHighRemoteFault,
                            uint32_t* pGlobalAlarm,
                            uint32_t* pLatchHighGlobalAlarm);

#endif /* NTHW_MAC_H_ */

//
// EOF
//
