//
// nthw_mac_pcs_xxv.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

#ifndef NTHW_MAC_PCS_XXV_H_
#define NTHW_MAC_PCS_XXV_H_

#include <stdint.h>
#include <stdbool.h>
#include "nthw_fpga_model.h"
//
//
//


enum NthwMacPcsXxvLEDMode_e {
  NTHW_MAC_PCS_XXV_LED_AUTO = 0x00,
  NTHW_MAC_PCS_XXV_LED_ON = 0x01,
  NTHW_MAC_PCS_XXV_LED_OFF = 0x02,
  NTHW_MAC_PCS_XXV_LED_PORTID = 0x03,
};

enum NthwMacPcsXxvDACMode_e {
  NTHW_MAC_PCS_XXV_DAC_OFF = 0x00,
  NTHW_MAC_PCS_XXV_DAC_CA_25G_N = 0x01,
  NTHW_MAC_PCS_XXV_DAC_CA_25G_S = 0x02,
  NTHW_MAC_PCS_XXV_DAC_CA_25G_L = 0x03,
};

//
//
//
struct NthwMacPcsXxv {

  Fpga_t* mpFpga;
  //
  Module_t* mpModMacPcsXxv;
  //
  int mnInstance;
  //
  uint8_t mPortNo;
  //

#define NTHW_MAC_PCS_XXV_NUM_ELEMS 4
  struct NthwMacPcsXxv_RegistersFields {
    // CORE_CONF
    Register_t* mpRegCoreConf;
    Field_t*    mpFldCoreConfRxEnable;
    Field_t*    mpFldCoreConfRxForceResync;
    Field_t*    mpFldCoreConfTxEnable;
    Field_t*    mpFldCoreConfTxInsFcs;
    Field_t*    mpFldCoreConfTxIgnFcs;
    Field_t*    mpFldCoreConfTxSendLfi;
    Field_t*    mpFldCoreConfTxSendRfi;
    Field_t*    mpFldCoreConfTxSendIdle;
    Field_t*    mpFldCoreConfInlineMode;
    Field_t*    mpFldCoreConfLineLoopback;
    Field_t*    mpFldCoreConfTsAtEop;

    // ANEG_CONFIG
    Register_t* mpRegAnegConfig;
    Field_t*    mpFldAnegConfigEnable;
    Field_t*    mpFldAnegConfigBypass;
    Field_t*    mpFldAnegConfigRestart;
    Field_t*    mpFldAnegConfigPseudo;
    Field_t*    mpFldAnegConfigNonceSeed;
    Field_t*    mpFldAnegConfigRemoteFault;
    Field_t*    mpFldAnegConfigPause;
    Field_t*    mpFldAnegConfigAsmdir;
    Field_t*    mpFldAnegConfigFec74Request10G;
    Field_t*    mpFldAnegConfigHideFec74;
    Field_t*    mpFldAnegConfigFec74Request;
    Field_t*    mpFldAnegConfigFec91Request;
    Field_t*    mpFldAnegConfigFec91Ability;
    Field_t*    mpFldAnegConfigRsFecRequest;
    Field_t*    mpFldAnegConfigSwFecOverwrite;
    Field_t*    mpFldAnegConfigSwSpeedOverwrite;

    // ANEG_ABILITY
    Register_t* mpRegAnegAbility;
    Field_t*    mpFldAnegAbility25GBaseCR;
    Field_t*    mpFldAnegAbility25GBaseCRS;
    Field_t*    mpFldAnegAbility25GBaseCR1;

    // LT_CONF
    Register_t* mpRegLtConf;
    Field_t*    mpFldLtConfEnable;
    Field_t*    mpFldLtConfRestart;
    Field_t*    mpFldLtConfSeed;

    // SUB_RST
    Register_t* mpRegSubRst;
    Field_t*    mpFldSubRstRxMacPcs;
    Field_t*    mpFldSubRstTxMacPcs;
    Field_t*    mpFldSubRstRxGtData;
    Field_t*    mpFldSubRstTxGtData;
    Field_t*    mpFldSubRstRxBuf;
    Field_t*    mpFldSubRstRxPma;
    Field_t*    mpFldSubRstTxPma;
    Field_t*    mpFldSubRstRxPcs;
    Field_t*    mpFldSubRstTxPcs;
    Field_t*    mpFldSubRstAnLt;
    Field_t*    mpFldSubRstSpeedCtrl;

    // SUB_RST_STATUS
    Register_t* mpRegSubRstStatus;
    Field_t*    mpFldSubRstStatusUserRxRst;
    Field_t*    mpFldSubRstStatusUserTxRst;
    Field_t*    mpFldSubRstStatusQpllLock;

    // LINK_SUMMARY
    Register_t* mpRegLinkSummary;
    Field_t*    mpFldLinkSummaryNtPhyLinkState;
    Field_t*    mpFldLinkSummaryLlNtPhyLinkState;
    Field_t*    mpFldLinkSummaryAbs;
    Field_t*    mpFldLinkSummaryLhAbs;
    Field_t*    mpFldLinkSummaryLinkDownCnt;
    // Begin 2 x 10/25 Gbps only fields:
    Field_t*    mpFldLinkSummaryLlRxFec74Lock;
    Field_t*    mpFldLinkSummaryLhRxRsfecHiSer;
    Field_t*    mpFldLinkSummaryLlRxRsfecLaneAlignment;
    Field_t*    mpFldLinkSummaryLlTxRsfecLaneAlignment;
    Field_t*    mpFldLinkSummaryLhRxPcsValidCtrlCode;
    // End 2 x 10/25 Gbps only fields.
    Field_t*    mpFldLinkSummaryLlRxBlockLock;
    Field_t*    mpFldLinkSummaryLhRxHighBitErrorRate;
    Field_t*    mpFldLinkSummaryLhInternalLocalFault;
    Field_t*    mpFldLinkSummaryLhReceivedLocalFault;
    Field_t*    mpFldLinkSummaryLhLocalFault;
    Field_t*    mpFldLinkSummaryLhRemoteFault;
    Field_t*    mpFldLinkSummaryLhTxLocalFault;
    Field_t*    mpFldLinkSummaryNimInterr;

    // GTY_LOOP
    Register_t* mpRegGtyLoop;
    Field_t*    mpFldGtyLoopGtLoop;

    // GTY_CTL_RX
    Register_t* mpRegGtyCtlRx;
    Field_t*    mpFldGtyCtlRxPolarity;
    Field_t*    mpFldGtyCtlRxLpmEn;
    Field_t*    mpFldGtyCtlRxEquaRst;

    // GTY_CTL_TX
    Register_t* mpRegGtyCtlTx;
    Field_t*    mpFldGtyCtlTxPolarity;
    Field_t*    mpFldGtyCtlTxInhibit;

    // LINK_SPEED
    Register_t* mpRegLinkSpeed;
    Field_t*    mpFldLinkSpeed10G;
    Field_t*    mpFldLinkSpeedToggle;

    // RS_FEC_CONF
    Register_t* mpRegRsFecConf;
    Field_t*    mpFldRsFecConfRsFecEnable;

    // DEBOUNCE_CTRL
    Register_t* mpRegDebounceCtrl;
    Field_t*    mpFieldDebounceCtrlNtPortCtrl;

    // FEC_CCW_CNT
    Register_t* mpRegRsFecCcw;
    Field_t*    mpFieldRegRsFecCcwRegRsFecCcwCnt;

    // FEC_UCW_CNT
    Register_t* mpRegRsFecUcw;
    Field_t*    mpFieldRegRsFecUcwRegRsFecUcwCnt;

    // TIMESTAMP_COMP
    Register_t* mpRegTimestampComp;
    Field_t* mpFieldTimestampCompRxDly;
    Field_t* mpFieldTimestampCompTxDly;

    // GTY_PRE_CURSOR
    Register_t* mpRegGtyPreCursor;
    Field_t* mpFieldGtyPreCursorTxPreCsr;

    // GTY_DIFF_CTL
    Register_t* mpRegGtyDiffCtl;
    Field_t* mpFieldGtyGtyDiffCtlTxDiffCtl;

    // GTY_POST_CURSOR
    Register_t* mpRegGtyPostCursor;
    Field_t* mpFieldGtyPostCursorTxPostCsr;
  } regs[NTHW_MAC_PCS_XXV_NUM_ELEMS];
};

typedef struct NthwMacPcsXxv NthwMacPcsXxv_t;
typedef struct NthwMacPcsXxv NthwMacPcsXxv;


//
//
//
NthwMacPcsXxv_t* NthwMacPcsXxv_new(void);
void NthwMacPcsXxv_delete(NthwMacPcsXxv_t* p);
int NthwMacPcsXxv_init(NthwMacPcsXxv_t* p, Fpga_t* pFpga, int nInstance, int nChannels);
//
void NthwMacPcsXxv_getLinkSummary(NthwMacPcsXxv_t* p,
                                uint32_t* pAbs,
                                uint32_t* pNtPhyLinkState,
                                uint32_t* pLhAbs,
                                uint32_t* pLlNtPhyLinkState,
                                uint32_t* pLinkDownCnt,
                                uint32_t* pNimInterr,
                                uint32_t* pLhLocalFault,
                                uint32_t* pLhRemoteFault,
                                uint32_t* pLhInternalLocalFault,
                                uint32_t* pLhReceivedLocalFault,
                                uint8_t index
                              );

uint8_t NthwMacPcsXxv_getPortNo(const NthwMacPcsXxv_t *p, uint8_t index);

//
void NthwMacPcsXxv_setPortNo(NthwMacPcsXxv_t* p, uint8_t portNo);

void NthwMacPcsXxv_setTxEnable(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setRxEnable(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_rxForceResync(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_resetRxGtData(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_resetTxGtData(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_resetAnLt(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_resetSpeedCtrl(NthwMacPcsXxv_t *p, bool enable, uint8_t index);

void NthwMacPcsXxv_setTxSendRfi(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxSendLfi(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxSendLfiTxSendRfi(NthwMacPcsXxv_t* p, bool enableLfi, bool enableRfi, uint8_t index);

bool NthwMacPcsXxv_isDfeEnabled(NthwMacPcsXxv_t* p, uint8_t index);
void NthwMacPcsXxv_setDfe(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setRxGtyPolarity(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxGtyPolarity(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setTxGtyInhibit(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setHostLoopback(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setLineLoopback(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

bool NthwMacPcsXxv_isUserRxRst(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_isUserTxRst(NthwMacPcsXxv_t* p, uint8_t index);

bool NthwMacPcsXxv_isQpllLock(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_isSubRstReady(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_isAnegEnabled(NthwMacPcsXxv_t* p, uint8_t index);

void NthwMacPcsXxv_setTxSendIdle(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxInsFcs(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

bool NthwMacPcsXxv_getLinkSpeed10G(NthwMacPcsXxv_t* p, uint8_t index);
void NthwMacPcsXxv_setLinkSpeed10G(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setLinkSpeedToggle(NthwMacPcsXxv_t* p, uint8_t index);

void NthwMacPcsXxv_setRsFecConfRsFecEnable(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setLEDMode(NthwMacPcsXxv_t* p, uint8_t mode, uint8_t index);

void NthwMacPcsXxv_setRxMacPcsRst(NthwMacPcsXxv_t* p, bool enable, uint8_t index);
void NthwMacPcsXxv_setTxMacPcsRst(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_resetFecCounters(NthwMacPcsXxv_t* p, uint8_t index);

void NthwMacPcsXxv_setGtyDiff(NthwMacPcsXxv_t* p, uint8_t value, uint8_t index);
void NthwMacPcsXxv_setGtyPre(NthwMacPcsXxv_t* p, uint8_t value, uint8_t index);
void NthwMacPcsXxv_setGtyPost(NthwMacPcsXxv_t* p, uint8_t value, uint8_t index);

void NthwMacPcsXxv_setTsAtEop(NthwMacPcsXxv_t* p, bool enable, uint8_t index);

void NthwMacPcsXxv_setAnegConfigEnable(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setAnegConfigBypass(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setLtConfEnable(NthwMacPcsXxv_t *p, bool enable, uint8_t index);

void NthwMacPcsXxv_setDACMode(NthwMacPcsXxv_t *p, uint8_t DACMode, uint8_t index);

void NthwMacPcsXxv_setTimestampCompRx(NthwMacPcsXxv_t* p, uint16_t rxDly, uint8_t index);
void NthwMacPcsXxv_setTimestampCompTx(NthwMacPcsXxv_t* p, uint16_t txDly, uint8_t index);

void NthwMacPcsXxv_setAnegConfigFec91Request(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setAnegConfigRsFecRequest(NthwMacPcsXxv_t *p, bool enable, uint8_t index);
void NthwMacPcsXxv_setAnegConfigFec74Request(NthwMacPcsXxv_t *p, bool enable, uint8_t index);

bool NthwMacPcsXxv_getLlRxFec74Lock(NthwMacPcsXxv_t* p, uint8_t index);
bool NthwMacPcsXxv_getLlRxRsfecLaneAlignment(NthwMacPcsXxv_t* p, uint8_t index);

//
//
//
#endif // NTHW_MAC_PCS_XXV_H_

//
// EOF
//
