//
// nthw_qm.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */


//
//
//
#ifndef NTHW_QM_HPP_
#define NTHW_QM_HPP_

//
//
//

enum NthwQm_QueuePriority {
    NTHW_QM_QUEUE_PRIORITY_MAX=0x00,
    NTHW_QM_QUEUE_PRIORITY_HIGH=0x01,
    NTHW_QM_QUEUE_PRIORITY_LOW=0x02,
    NTHW_QM_QUEUE_PRIORITY_MIN=0x03,
};

//
//
//
struct NthwQm
{
  Fpga_t* mpFpga;
  //
  Module_t* mpModQm;
  //
  int mnInstance;
  //

  uint32_t mnQueues;
  uint32_t mnBlocks;
  uint32_t mnCells;
  uint32_t mnCellSize;

  // 1: QM_CTRL
  Register_t* mpRegCtrl;
  Field_t*    mpFldCtrlEnable;
        // 0: Priorities are controlled by the priorities set in the Priority_Levels table.
        // 1: Each Queues priority level depends on the fill level (of Queue or Group Queue belongs to) for that Queue.
        // 2: Each Queues priority level depends on the how long time since the Cells were written to SDRAM.
  Field_t*    mpFldCtrlPriorityScheme; // 1: this is currently the only scheme supported
  Field_t*    mpFldCtrlActiveQueues;

  // 2: QM_DEBUG_CRC
  Register_t* mpRegDebugCrc;
  Field_t*    mpFldDebugCrcForceError;

  // 3: QM_DEBUG_SDRAM_SIZE: default 4 MCells each 2 KByte: 8 GByte SDRAM
  Register_t* mpRegDebugSdramSize;
  Field_t*    mpFldDebugSdramSizeMask;

  // 3: QM_DEBUG_BLOCK_CELLS: default 4 MCells each 2 KByte: 8 GByte SDRAM
  Register_t* mpRegDebugBlockSize;
  Field_t*    mpFldDebugBlockSizeCells;

  // 4: QM_STATUS_SDRAM_BLOCK_USAGE
  Register_t* mpRegStatusSdramBlockUsage;
  Field_t*    mpFldStatusSdramBlockUsageCount;    // SDRAM Block Usage, shows the current number of Blocks used in SDRAM. MAX = 2048 Blocks
                                                      // 1 Block = 2048 Cells; 1 Cell = 2 KByte; 1 Block = 2048 * 2048 = 4 MByte

  // 5: QM_STATUS_SDRAM_BLOCK_MAX_USAGE
  Register_t* mpRegStatusSdramBlockMaxUsage;
  Field_t*    mpFldStatusSdramBlockMaxUsageCount; // SDRAM Block Max Usage, shows the maximum number of Blocks used in SDRAM.

  // 6: QM_STATUS_SDRAM_CELL_USAGE
  Register_t* mpRegStatusSdramCellUsage;
  Field_t*    mpFldStatusSdramCellUsageCount;     // SDRAM Cell Usage, shows the number of Cells used in SDRAM.

  // 7: QM_STATUS_SDRAM_CELL_MAX_USAGE
  Register_t* mpRegStatusSdramCellMaxUsage;
  Field_t*    mpFldStatusSdramCellMaxUsageCount;  // SDRAM Cell Usage, shows the maximum number of Cells used in SDRAM.

  // 8: QM_STATUS_CRC_ERROR
  Register_t* mpRegStatusCrcError;
  Field_t*    mpFldStatusCrcErrorCount;           // SDRAM Cells with CRC error

  // 9: QM_STATUS_FLUSH_DROP
  Register_t* mpRegStatusFlushDrop;
  Field_t*    mpFldStatusFlushDropCount;          // SDRAM Cells which are dropped due to Flushes

  // 10: QM_STATUS_BLOCKED
  Register_t* mpRegStatusBlocked;
  Field_t*    mpFldStatusBlockedBitStatus;        // One bit for each Queue indicates whether that queue blocked by next module in RX pipe (QHM).

  // 11: QM_STATUS_BLOCKING
  Register_t* mpRegStatusBlocking;
  Field_t*    mpFldStatusBlockingBitStatus;       // One bit for each Queue indicates whether that queue is blocking previous module in RX pipe (CB).

  // 12: QM_STATUS_EMPTY
  Register_t* mpRegStatusEmpty;
  Field_t*    mpFldStatusEmptyBitStatus;          // One bit for each Queue indicates whether that queue is empty.

  // ==>> 0: Priorities are controlled by the priorities set in the Priority_Levels table.
  // 13: QM_PRIO_LEVELS_MEM_CTRL
  Register_t* mpRegPrioLevelsMemCtrl;
  Field_t*    mpFldPrioLevelsMemCtrlAdr;
  Field_t*    mpFldPrioLevelsMemCtrlCnt;

  // 14: QM_PRIO_LEVELS_MEM_DATA
  Register_t* mpRegPrioLevelsMemData;
  Field_t*    mpFldPrioLevelsMemDataPriority;

  // ==>>1: Group mapping
  // 15: QM_GROUP_MAPPING_MEM_CTRL
  Register_t* mpRegGroupMappingMemCtrl;
  Field_t*    mpFldGroupMappingMemCtrlAdr;
  Field_t*    mpFldGroupMappingMemCtrlCnt;

  // 16: QM_GROUP_MAPPING_MEM_DATA
  Register_t* mpRegGroupMappingMemData;
  Field_t*    mpFldGroupMappingMemDataGroup0;
  Field_t*    mpFldGroupMappingMemDataGroup1;
  Field_t*    mpFldGroupMappingMemDataGroup2;

  // ==>> 2: Queue limit
  // 17: QM_QUEUE_LIMIT_MEM_CTRL
  Register_t* mpRegQueueLimitMemCtrl;
  Field_t*    mpFldQueueLimitMemCtrlAdr;
  Field_t*    mpFldQueueLimitMemCtrlCnt;

  // 18: QM_QUEUE_LIMIT_MEM_DATA
  Register_t* mpRegQueueLimitMemData;
  Field_t*    mpFldQueueLimitMemDataLimit;

  // ==>>3: Group limit
  // 19: QM_GROUP_LIMIT_MEM_CTRL
  Register_t* mpRegGroupLimitMemCtrl;
  Field_t*    mpFldGroupLimitMemCtrlAdr;
  Field_t*    mpFldGroupLimitMemCtrlCnt;

  // 20: QM_GROUP_LIMIT_MEM_DATA
  Register_t* mpRegGroupLimitMemData;
  Field_t*    mpFldGroupLimitMemDataLimit;


};

typedef struct NthwQm NthwQm_t;
typedef struct NthwQm NthwQm;

//
//
//

//
//
//
NthwQm_t* NthwQm_new(void);
void NthwQm_delete(NthwQm_t* p);
int NthwQm_init(NthwQm_t* p, Fpga_t* pFpga, int nInstance);
//
void NthwQm_enable(NthwQm_t* p);
void NthwQm_disable(NthwQm_t* p);
uint32_t NthwQm_getEnable(NthwQm_t* p);
void NthwQm_setEnable(NthwQm_t* p, uint32_t value);
void NthwQm_setActiveQueues(NthwQm_t* p, int value);

uint32_t NthwQm_getStatusCrcErrorCount(NthwQm_t* p);
void NthwQm_setStatusCrcErrorCount(NthwQm_t* p, uint32_t value);

uint32_t NthwQm_getStatusSdramBlockUsageCount(NthwQm_t* p);
uint32_t NthwQm_getStatusSdramBlockMaxUsageCount(NthwQm_t* p);
uint32_t NthwQm_getStatusSdramCellUsageCount(NthwQm_t* p);
uint32_t NthwQm_getStatusSdramCellMaxUsageCount(NthwQm_t* p);
uint32_t NthwQm_getDebugSdramSizeMask(NthwQm_t* p);
uint32_t NthwQm_getDebugBlockSizeCells(NthwQm_t* p);

void NthwQm_setQueuePriority(NthwQm_t* p, uint16_t queue, uint8_t priority);
void NthwQm_setGroupLimit(NthwQm_t* p, uint16_t group, uint16_t limit);
void NthwQm_setQueueLimit(NthwQm_t* p, uint16_t queue, uint16_t limit);
void NthwQm_setQueueGroupMapping(NthwQm_t* p, uint16_t queue, uint16_t group0, uint16_t group1, uint16_t group2 );
void NthwQm_setPriorityScheme(NthwQm_t* p, uint8_t priority_scheme);
void NthwQm_setConfiguration(NthwQm_t* p, uint8_t variant);

uint32_t NthwQm_getParamQueues(NthwQm_t* p);
uint32_t NthwQm_getParamSdramCells(NthwQm_t* p);
uint32_t NthwQm_getParamSdramCellSize(NthwQm_t* p);
uint32_t NthwQm_getParamSdramBlocks(NthwQm_t* p);

// set QM config variant 1
void NthwQm_setConfigurationVariant1(NthwQm_t* p);


//
//
//
#endif // NTHW_QM_H_

//
// EOF
//
