//
// nthw_smm.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

#ifndef NTHW_SMM_H_
#define NTHW_SMM_H_

//
//
//

#define NTHW_SMM_RX_INSTANCE (0)
#define NTHW_SMM_TX_INSTANCE (1)

//
//
//
typedef struct {
  uint32_t startSeg;
  uint64_t hbSegTablePhyAddr;
  uint32_t hbSegCount;
} hbSegmentSetup_t;


//
//
//
struct NthwSmm
{
  Fpga_t* mpFpga;
  //
  Module_t* mpModSmm;
  //
  int mnInstance;
  //
  int mnInstanceType;
  //
  int mnModuleMajorVersion;
  int mnModuleMinorVersion;
  //
  uint64_t mnModuleVersionPacked64;

  //
  uint32_t nbHb;

  Register_t* mpRegCtrl;
  Field_t* mpFldCtrlEnable;

  Register_t* mpRegReadyStatus;
  Field_t* mpFldReadyStatusD;

  Register_t* mpRegInvalidStickyStatus;
  Field_t* mpFldInvalidStickyStatusD;

  Register_t* mpRegSegMemCtrl;
  Field_t* mpFldSegMemCtrlA;
  Field_t* mpFldSegMemCtrlCnt;

  Register_t* mpRegSegMemData;
  Field_t* mpFldSegMemDataPhyAddr;
  Field_t* mpFldSegMemDataSize;

  Register_t* mpRegStartSegMemCtrl;
  Field_t* mpFldStartSegMemCtrlAdr;
  Field_t* mpFldStartSegMemCtrlCnt;

  Register_t* mpRegStartSegMemData;
  Field_t* mpFldStartSegMemDataSeg;

  //
  //
  //
  uint32_t maHbEnable[4];
  //
  hbSegmentSetup_t maSegmentTableSetup[128];
  //
};


typedef struct NthwSmm NthwSmm_t;
typedef struct Nthwsmm NthwSmm;

//
//
//
NthwSmm_t* NthwSmm_new(void);
void NthwSmm_delete(NthwSmm_t* p);
int NthwSmm_init(NthwSmm_t *p, Fpga_t *pFpga, int nInstance);
//
void NthwSmm_clear(NthwSmm_t* p);
void NthwSmm_enable(NthwSmm_t* p);
void NthwSmm_disable(NthwSmm_t* p);
//
int NthwSmm_getSegmentCount(NthwSmm_t* p, uint32_t nHbFeed);
hbSegmentSetup_t* NthwSmm_getSegmentSetup(NthwSmm_t* p, const uint32_t hbNum); // used in inline scenarios to exchange the HB segment setup between RX and TX SMM
void NthwSmm_setStartSegment(NthwSmm_t* p, uint32_t hbNum, uint32_t startSeg);
void NthwSmm_setSegmentTable(NthwSmm_t* p, uint32_t hbNum, uint64_t hbSegTablePhyAddr, uint32_t hbSegCount);
bool NthwSmm_isHbPhyAdrInvalid(NthwSmm_t* p, uint32_t* invalidHbMask);
//
// NOTE: Max. number of host buffers are currently 128 for RX and TX.
//       If this constant is no longer valid, consider to use dynamic allocation.
void NthwSmm_disableHb(NthwSmm_t* p, uint32_t hbNum);
void NthwSmm_enableHb(NthwSmm_t* p, uint32_t hbNum);
bool NthwSmm_isHbReady(NthwSmm_t* p, uint32_t hbNum);

//
//
//
#endif // NTHW_SMM_H_

//
// EOF
//
