#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include "flow_api_backend.h"

#define _MOD_  "SLC"
#define _VER_  be->slc.ver


bool hw_mod_slc_present(struct flow_api_backend_s *be)
{
  return be->iface->get_slc_present(be->be_dev);
}

int hw_mod_slc_alloc(struct flow_api_backend_s *be)
{
  _VER_ = be->iface->get_slc_version(be->be_dev);
  NT_LOG(DBG, FILTER, "SLC MODULE VERSION  %i.%i\n", VER_MAJOR(_VER_), VER_MINOR(_VER_));

  switch (_VER_) {
    case 1:
      if (!callocate_mod(CAST_COMMON(&be->slc), 1,
          &be->slc.v1.rcp,  be->max_categories,  sizeof(struct slc_v1_rcp_s)))
        return -1;
      break;
    // end case 1
    default:
      RETURN_UNSUP_VER;
  }

  return 0;
}

void hw_mod_slc_free(struct flow_api_backend_s *be)
{
  if (be->slc.base) {
    free(be->slc.base);
    be->slc.base = NULL;
  }
}

int hw_mod_slc_reset(struct flow_api_backend_s *be)
{
  // Zero entire cache area
  ZERO_MOD_CACHE(slc);

  NT_LOG(DBG, FILTER, "INIT SLC RCP\n");
  return hw_mod_slc_rcp_flush(be, 0, be->max_categories);
}


int hw_mod_slc_rcp_flush(struct flow_api_backend_s *be, int start_idx, int count)
{
  if (count == ALL_ENTRIES) count = be->max_categories;
  if ((unsigned)(start_idx + count) > be->max_categories) {
    RETURN_INDEX_TOO_LARGE;
  }
  return be->iface->slc_rcp_flush(be->be_dev, &be->slc, start_idx, count);
}


static int hw_mod_slc_rcp_mod(struct flow_api_backend_s *be, enum hw_slc_e field, uint32_t index,
    uint32_t *value, int get)
{
  if (index >= be->max_categories) {
    RETURN_INDEX_TOO_LARGE;
  }
  switch (_VER_) {
    case 1:
      switch (field) {
      case HW_SLC_RCP_PRESET_ALL:
        if (get) RETURN_UNSUP_FIELD;
        memset(&be->slc.v1.rcp[index], (uint8_t)*value, sizeof(struct hw_mod_slc_v1_s));
        break;
      case HW_SLC_RCP_FIND:
        FIND_EQUAL_INDEX(slc.v1.rcp, struct hw_mod_slc_v1_s, index, *value, be->max_categories);
        break;
      case HW_SLC_RCP_COMPARE:
        DO_COMPARE_INDEXS(slc.v1.rcp, struct hw_mod_slc_v1_s, index, *value, be->max_categories);
        break;
      case HW_SLC_RCP_SLC_EN:
        GET_SET(be->slc.v1.rcp[index].tail_slc_en, value);
        break;
      case HW_SLC_RCP_DYN:
        GET_SET(be->slc.v1.rcp[index].tail_dyn, value);
        break;
      case HW_SLC_RCP_OFS:
        GET_SET_SIGNED(be->slc.v1.rcp[index].tail_ofs, value);
        break;
      case HW_SLC_RCP_PCAP:
        GET_SET(be->slc.v1.rcp[index].pcap, value);
        break;

      default:
        RETURN_UNSUP_FIELD;
      }
      break;
    // end case 1
    default:
      RETURN_UNSUP_VER;
  }

  return 0;
}

int hw_mod_slc_rcp_set(struct flow_api_backend_s *be, enum hw_slc_e field, uint32_t index, uint32_t value)
{
  return hw_mod_slc_rcp_mod(be, field, index, &value, 0);
}

int hw_mod_slc_rcp_get(struct flow_api_backend_s *be, enum hw_slc_e field, uint32_t index, uint32_t *value)
{
  return hw_mod_slc_rcp_mod(be, field, index, value, 1);
}
