/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "flow_nthw_cat.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

struct CatNthw *CatNthw_new(void)
{
  struct CatNthw *p = malloc(sizeof(struct CatNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void CatNthw_delete(struct CatNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

void catNthw_setDebugMode(struct CatNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mCat, nDebugMode);
}

int CatNthw_init(struct CatNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_CAT, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Cat %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mCat = pMod;

#if defined(DEBUG) && (0)
  Module_setDebugMode(p->mCat, 0xFF); // DEBUG:
#endif

  p->mKmIfCnt = Fpga_getProductParam(p->mpFpga, NT_CAT_KM_IF_CNT, -1);

  // CFN
  p->mpCfnCtrl = Module_getRegister(p->mCat, CAT_CFN_CTRL);
  p->mpCfnAddr = Register_getField(p->mpCfnCtrl, CAT_CFN_CTRL_ADR);
  p->mpCfnCnt = Register_getField(p->mpCfnCtrl, CAT_CFN_CTRL_CNT);
  p->mpCfnData = Module_getRegister(p->mCat, CAT_CFN_DATA);
  p->mpCfnDataEnable = Register_getField(p->mpCfnData, CAT_CFN_DATA_ENABLE);
  p->mpCfnDataInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_INV);
  p->mpCfnDataPtcInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_INV);
  p->mpCfnDataPtcIsl = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_ISL);
  p->mpCfnDataPtcMac = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_MAC);
  p->mpCfnDataPtcL2 = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_L2);
  p->mpCfnDataPtcVnTag =     Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_VNTAG);
  p->mpCfnDataPtcVlan = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_VLAN);
  p->mpCfnDataPtcMpls = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_MPLS);
  p->mpCfnDataPtcL3 = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_L3);
  p->mpCfnDataPtcFrag = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_FRAG);
  p->mpCfnDataPtcIpProt =     Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_IP_PROT);
  p->mpCfnDataPtcL4 = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_L4);
  p->mpCfnDataPtcTunnel = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TUNNEL);
  p->mpCfnDataPtcTnlL2 = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_L2);
  p->mpCfnDataPtcTnlVlan = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_VLAN);
  p->mpCfnDataPtcTnlMpls = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_MPLS);
  p->mpCfnDataPtcTnlL3 = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_L3);
  p->mpCfnDataPtcTnlFrag = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_FRAG);
  p->mpCfnDataPtcTnlIpProt = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_IP_PROT);
  p->mpCfnDataPtcTnlL4 = Register_getField(p->mpCfnData, CAT_CFN_DATA_PTC_TNL_L4);
  p->mpCfnDataErrInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_ERR_INV);
  p->mpCfnDataErrCv = Register_getField(p->mpCfnData, CAT_CFN_DATA_ERR_CV);
  p->mpCfnDataErrFcs = Register_getField(p->mpCfnData, CAT_CFN_DATA_ERR_FCS);
  p->mpCfnDataErrTrunc = Register_getField(p->mpCfnData, CAT_CFN_DATA_ERR_TRUNC);
  p->mpCfnDataMacPort = Register_getField(p->mpCfnData, CAT_CFN_DATA_MAC_PORT);
  p->mpCfnDataPmCmp = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_CMP);
  p->mpCfnDataPmDct = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_DCT);
  p->mpCfnDataPmExtInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_EXT_INV);
  p->mpCfnDataPmCmb = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_CMB);
  p->mpCfnDataPmAndInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_AND_INV);
  p->mpCfnDataPmOrInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_OR_INV);
  p->mpCfnDataPmInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_PM_INV);
  p->mpCfnDataLc = Register_getField(p->mpCfnData, CAT_CFN_DATA_LC);
  p->mpCfnDataLcInv = Register_getField(p->mpCfnData, CAT_CFN_DATA_LC_INV);

  if (p->mKmIfCnt == -1) {
    p->mpCfnDataKm0Or = Register_getField(p->mpCfnData, CAT_CFN_DATA_KM_OR);
  } else {
    p->mpCfnDataKm0Or = Register_getField(p->mpCfnData, CAT_CFN_DATA_KM0_OR);
    p->mpCfnDataKm1Or = Register_queryField(p->mpCfnData, CAT_CFN_DATA_KM1_OR);
  }

  if (p->mKmIfCnt < 0) {
    // KCE
    p->mpKceCtrl[0] = Module_getRegister(p->mCat, CAT_KCE_CTRL);
    p->mpKceAddr[0] = Register_getField(p->mpKceCtrl[0], CAT_KCE_CTRL_ADR);
    p->mpKceCnt[0] = Register_getField(p->mpKceCtrl[0], CAT_KCE_CTRL_CNT);
    p->mpKceData[0] = Module_getRegister(p->mCat, CAT_KCE_DATA);
    p->mpKceDataEnable[0] = Register_getField(p->mpKceData[0], CAT_KCE_DATA_ENABLE);
    // KCS
    p->mpKcsCtrl[0] = Module_getRegister(p->mCat, CAT_KCS_CTRL);
    p->mpKcsAddr[0] = Register_getField(p->mpKcsCtrl[0], CAT_KCS_CTRL_ADR);
    p->mpKcsCnt[0] = Register_getField(p->mpKcsCtrl[0], CAT_KCS_CTRL_CNT);
    p->mpKcsData[0] = Module_getRegister(p->mCat, CAT_KCS_DATA);
    p->mpKcsDataCategory[0] = Register_getField(p->mpKcsData[0], CAT_KCS_DATA_CATEGORY);
    // FTE
    p->mpFteCtrl[0] = Module_getRegister(p->mCat, CAT_FTE_CTRL);
    p->mpFteAddr[0] = Register_getField(p->mpFteCtrl[0], CAT_FTE_CTRL_ADR);
    p->mpFteCnt[0] = Register_getField(p->mpFteCtrl[0], CAT_FTE_CTRL_CNT);
    p->mpFteData[0] = Module_getRegister(p->mCat, CAT_FTE_DATA);
    p->mpFteDataEnable[0] = Register_getField(p->mpFteData[0], CAT_FTE_DATA_ENABLE);
  } else {
    // KCE 0
    p->mpKceCtrl[0] = Module_getRegister(p->mCat, CAT_KCE0_CTRL);
    p->mpKceAddr[0] = Register_getField(p->mpKceCtrl[0], CAT_KCE0_CTRL_ADR);
    p->mpKceCnt[0] = Register_getField(p->mpKceCtrl[0], CAT_KCE0_CTRL_CNT);
    p->mpKceData[0] = Module_getRegister(p->mCat, CAT_KCE0_DATA);
    p->mpKceDataEnable[0] = Register_getField(p->mpKceData[0], CAT_KCE0_DATA_ENABLE);
    // KCS 0
    p->mpKcsCtrl[0] = Module_getRegister(p->mCat, CAT_KCS0_CTRL);
    p->mpKcsAddr[0] = Register_getField(p->mpKcsCtrl[0], CAT_KCS0_CTRL_ADR);
    p->mpKcsCnt[0] = Register_getField(p->mpKcsCtrl[0], CAT_KCS0_CTRL_CNT);
    p->mpKcsData[0] = Module_getRegister(p->mCat, CAT_KCS0_DATA);
    p->mpKcsDataCategory[0] = Register_getField(p->mpKcsData[0], CAT_KCS0_DATA_CATEGORY);
    // FTE 0
    p->mpFteCtrl[0] = Module_getRegister(p->mCat, CAT_FTE0_CTRL);
    p->mpFteAddr[0] = Register_getField(p->mpFteCtrl[0], CAT_FTE0_CTRL_ADR);
    p->mpFteCnt[0] = Register_getField(p->mpFteCtrl[0], CAT_FTE0_CTRL_CNT);
    p->mpFteData[0] = Module_getRegister(p->mCat, CAT_FTE0_DATA);
    p->mpFteDataEnable[0] = Register_getField(p->mpFteData[0], CAT_FTE0_DATA_ENABLE);
    // KCE 1
    p->mpKceCtrl[1] = Module_getRegister(p->mCat, CAT_KCE1_CTRL);
    p->mpKceAddr[1] = Register_getField(p->mpKceCtrl[1], CAT_KCE1_CTRL_ADR);
    p->mpKceCnt[1] = Register_getField(p->mpKceCtrl[1], CAT_KCE1_CTRL_CNT);
    p->mpKceData[1] = Module_getRegister(p->mCat, CAT_KCE1_DATA);
    p->mpKceDataEnable[1] = Register_getField(p->mpKceData[1], CAT_KCE1_DATA_ENABLE);
    // KCS 1
    p->mpKcsCtrl[1] = Module_getRegister(p->mCat, CAT_KCS1_CTRL);
    p->mpKcsAddr[1] = Register_getField(p->mpKcsCtrl[1], CAT_KCS1_CTRL_ADR);
    p->mpKcsCnt[1] = Register_getField(p->mpKcsCtrl[1], CAT_KCS1_CTRL_CNT);
    p->mpKcsData[1] = Module_getRegister(p->mCat, CAT_KCS1_DATA);
    p->mpKcsDataCategory[1] = Register_getField(p->mpKcsData[1], CAT_KCS1_DATA_CATEGORY);
    // FTE 1
    p->mpFteCtrl[1] = Module_getRegister(p->mCat, CAT_FTE1_CTRL);
    p->mpFteAddr[1] = Register_getField(p->mpFteCtrl[1], CAT_FTE1_CTRL_ADR);
    p->mpFteCnt[1] = Register_getField(p->mpFteCtrl[1], CAT_FTE1_CTRL_CNT);
    p->mpFteData[1] = Module_getRegister(p->mCat, CAT_FTE1_DATA);
    p->mpFteDataEnable[1] = Register_getField(p->mpFteData[1], CAT_FTE1_DATA_ENABLE);
  }

  // CTE
  p->mpCteCtrl = Module_getRegister(p->mCat, CAT_CTE_CTRL);
  p->mpCteAddr = Register_getField(p->mpCteCtrl, CAT_CTE_CTRL_ADR);
  p->mpCteCnt = Register_getField(p->mpCteCtrl, CAT_CTE_CTRL_CNT);
  p->mpCteData = Module_getRegister(p->mCat, CAT_CTE_DATA);
  p->mpCteDataCol = Register_getField(p->mpCteData, CAT_CTE_DATA_COL_ENABLE);
  p->mpCteDataCor = Register_getField(p->mpCteData, CAT_CTE_DATA_COR_ENABLE);
  p->mpCteDataHsh = Register_getField(p->mpCteData, CAT_CTE_DATA_HSH_ENABLE);
  p->mpCteDataQsl = Register_getField(p->mpCteData, CAT_CTE_DATA_QSL_ENABLE);
  p->mpCteDataIpf = Register_getField(p->mpCteData, CAT_CTE_DATA_IPF_ENABLE);
  p->mpCteDataSlc = Register_getField(p->mpCteData, CAT_CTE_DATA_SLC_ENABLE);
  p->mpCteDataPdb = Register_getField(p->mpCteData, CAT_CTE_DATA_PDB_ENABLE);
  p->mpCteDataMsk = Register_queryField(p->mpCteData, CAT_CTE_DATA_MSK_ENABLE);
  p->mpCteDataHst = Register_queryField(p->mpCteData, CAT_CTE_DATA_HST_ENABLE);
  p->mpCteDataEpp = Register_queryField(p->mpCteData, CAT_CTE_DATA_EPP_ENABLE);
  p->mpCteDataTpe = Register_queryField(p->mpCteData, CAT_CTE_DATA_TPE_ENABLE);
  p->mpCteDataRrb = Register_queryField(p->mpCteData, CAT_CTE_DATA_RRB_ENABLE);
  // CTS
  p->mpCtsCtrl = Module_getRegister(p->mCat, CAT_CTS_CTRL);
  p->mpCtsAddr = Register_getField(p->mpCtsCtrl, CAT_CTS_CTRL_ADR);
  p->mpCtsCnt = Register_getField(p->mpCtsCtrl, CAT_CTS_CTRL_CNT);
  p->mpCtsData = Module_getRegister(p->mCat, CAT_CTS_DATA);
  p->mpCtsDataCatA = Register_getField(p->mpCtsData, CAT_CTS_DATA_CAT_A);
  p->mpCtsDataCatB = Register_getField(p->mpCtsData, CAT_CTS_DATA_CAT_B);
  // COT
  p->mpCotCtrl = Module_getRegister(p->mCat, CAT_COT_CTRL);
  p->mpCotAddr = Register_getField(p->mpCotCtrl, CAT_COT_CTRL_ADR);
  p->mpCotCnt = Register_getField(p->mpCotCtrl, CAT_COT_CTRL_CNT);
  p->mpCotData = Module_getRegister(p->mCat, CAT_COT_DATA);
  p->mpCotDataColor = Register_getField(p->mpCotData, CAT_COT_DATA_COLOR);
  p->mpCotDataKm = Register_getField(p->mpCotData, CAT_COT_DATA_KM);
  p->mpCotDataNfvSb = Register_queryField(p->mpCotData, CAT_COT_DATA_NFV_SB);
  // CCT
  p->mpCctCtrl = Module_getRegister(p->mCat, CAT_CCT_CTRL);
  p->mpCctAddr = Register_getField(p->mpCctCtrl, CAT_CCT_CTRL_ADR);
  p->mpCctCnt = Register_getField(p->mpCctCtrl, CAT_CCT_CTRL_CNT);
  p->mpCctData = Module_getRegister(p->mCat, CAT_CCT_DATA);
  p->mpCctDataColor = Register_getField(p->mpCctData, CAT_CCT_DATA_COLOR);
  p->mpCctDataKm = Register_getField(p->mpCctData, CAT_CCT_DATA_KM);
  // EXO
  p->mpExoCtrl = Module_getRegister(p->mCat, CAT_EXO_CTRL);
  p->mpExoAddr = Register_getField(p->mpExoCtrl, CAT_EXO_CTRL_ADR);
  p->mpExoCnt = Register_getField(p->mpExoCtrl, CAT_EXO_CTRL_CNT);
  p->mpExoData = Module_getRegister(p->mCat, CAT_EXO_DATA);
  p->mpExoDataDyn = Register_getField(p->mpExoData, CAT_EXO_DATA_DYN);
  p->mpExoDataOfs = Register_getField(p->mpExoData, CAT_EXO_DATA_OFS);
  // RCK
  p->mpRckCtrl = Module_getRegister(p->mCat, CAT_RCK_CTRL);
  p->mpRckAddr = Register_getField(p->mpRckCtrl, CAT_RCK_CTRL_ADR);
  p->mpRckCnt = Register_getField(p->mpRckCtrl, CAT_RCK_CTRL_CNT);
  p->mpRckData = Module_getRegister(p->mCat, CAT_RCK_DATA);
  // LEN
  p->mpLenCtrl = Module_getRegister(p->mCat, CAT_LEN_CTRL);
  p->mpLenAddr = Register_getField(p->mpLenCtrl, CAT_LEN_CTRL_ADR);
  p->mpLenCnt = Register_getField(p->mpLenCtrl, CAT_LEN_CTRL_CNT);
  p->mpLenData = Module_getRegister(p->mCat, CAT_LEN_DATA);
  p->mpLenDataLower = Register_getField(p->mpLenData, CAT_LEN_DATA_LOWER);
  p->mpLenDataUpper = Register_getField(p->mpLenData, CAT_LEN_DATA_UPPER);
  p->mpLenDataDyn1 = Register_getField(p->mpLenData, CAT_LEN_DATA_DYN1);
  p->mpLenDataDyn2 = Register_getField(p->mpLenData, CAT_LEN_DATA_DYN2);
  p->mpLenDataInv = Register_getField(p->mpLenData, CAT_LEN_DATA_INV);

  p->mpCfnDataPtcCfp = Register_queryField(p->mpCfnData, CAT_CFN_DATA_PTC_CFP);
  p->mpCfnDataErrL3Cs = Register_queryField(p->mpCfnData, CAT_CFN_DATA_ERR_L3_CS);
  p->mpCfnDataErrL4Cs = Register_queryField(p->mpCfnData, CAT_CFN_DATA_ERR_L4_CS);
  p->mpCfnDataErrTnlL3Cs = Register_queryField(p->mpCfnData, CAT_CFN_DATA_ERR_TNL_L3_CS);
  p->mpCfnDataErrTnlL4Cs = Register_queryField(p->mpCfnData, CAT_CFN_DATA_ERR_TNL_L4_CS);
  p->mpCfnDataErrTtlExp = Register_queryField(p->mpCfnData, CAT_CFN_DATA_ERR_TTL_EXP);
  p->mpCfnDataErrTnlTtlExp = Register_queryField(p->mpCfnData, CAT_CFN_DATA_ERR_TNL_TTL_EXP);

  p->mpKccCtrl = Module_queryRegister(p->mCat, CAT_KCC_CTRL);
  if (p->mpKccCtrl != NULL) {
    p->mpKccAddr = Register_queryField(p->mpKccCtrl, CAT_KCC_CTRL_ADR);
    p->mpKccCnt = Register_queryField(p->mpKccCtrl, CAT_KCC_CTRL_CNT);
  }
  p->mpKccData = Module_queryRegister(p->mCat, CAT_KCC_DATA);
  if (p->mpKccData != NULL) {
    p->mpKccDataKey = Register_queryField(p->mpKccData, CAT_KCC_DATA_KEY);
    p->mpKccDataCategory = Register_queryField(p->mpKccData, CAT_KCC_DATA_CATEGORY);
    p->mpKccDataId = Register_queryField(p->mpKccData, CAT_KCC_DATA_ID);
  }


  p->mpCceCtrl = Module_queryRegister(p->mCat, CAT_CCE_CTRL);
  if (p->mpCceCtrl != NULL) {
    p->mpCceAddr = Register_queryField(p->mpCceCtrl, CAT_CCE_CTRL_ADR);
    p->mpCceCnt = Register_queryField(p->mpCceCtrl, CAT_CCE_CTRL_CNT);
  }
  p->mpCceData = Module_queryRegister(p->mCat, CAT_CCE_DATA);
  if (p->mpCceData != NULL) {
    p->mpCceDataImm = Register_queryField(p->mpCceData, CAT_CCE_DATA_IMM);
    p->mpCceDataInd = Register_queryField(p->mpCceData, CAT_CCE_DATA_IND);
  }

  p->mpCcsCtrl = Module_queryRegister(p->mCat, CAT_CCS_CTRL);
  if (p->mpCcsCtrl != NULL) {
    p->mpCcsAddr = Register_queryField(p->mpCcsCtrl, CAT_CCS_CTRL_ADR);
    p->mpCcsCnt  = Register_queryField(p->mpCcsCtrl, CAT_CCS_CTRL_CNT);
  }
  p->mpCcsData = Module_queryRegister(p->mCat, CAT_CCS_DATA);
  if (p->mpCcsData != NULL) {
    p->mpCcsDataCorEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_COR_EN);
    p->mpCcsDataCor     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_COR);
    p->mpCcsDataHshEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_HSH_EN);
    p->mpCcsDataHsh     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_HSH);
    p->mpCcsDataQslEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_QSL_EN);
    p->mpCcsDataQsl     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_QSL);
    p->mpCcsDataIpfEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_IPF_EN);
    p->mpCcsDataIpf     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_IPF);
    p->mpCcsDataSlcEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SLC_EN);
    p->mpCcsDataSlc     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SLC);
    p->mpCcsDataPdbEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_PDB_EN);
    p->mpCcsDataPdb     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_PDB);
    p->mpCcsDataMskEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_MSK_EN);
    p->mpCcsDataMsk     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_MSK);
    p->mpCcsDataHstEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_HST_EN);
    p->mpCcsDataHst     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_HST);
    p->mpCcsDataEppEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_EPP_EN);
    p->mpCcsDataEpp     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_EPP);
    p->mpCcsDataTpeEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_TPE_EN);
    p->mpCcsDataTpe     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_TPE);
    p->mpCcsDataRrbEn   = Register_queryField(p->mpCcsData, CAT_CCS_DATA_RRB_EN);
    p->mpCcsDataRrb     = Register_queryField(p->mpCcsData, CAT_CCS_DATA_RRB);
    p->mpCcsDataSb0Type = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SB0_TYPE);
    p->mpCcsDataSb0Data = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SB0_DATA);
    p->mpCcsDataSb1Type = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SB1_TYPE);
    p->mpCcsDataSb1Data = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SB1_DATA);
    p->mpCcsDataSb2Type = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SB2_TYPE);
    p->mpCcsDataSb2Data = Register_queryField(p->mpCcsData, CAT_CCS_DATA_SB2_DATA);
  }

  return 0;
}

// CFN
void CatNthw_CfnSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnAddr, val);
}

void CatNthw_CfnCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnCnt, val);
}

void CatNthw_CfnEnable(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataEnable, val);
}

void CatNthw_CfnInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataInv, val);
}

void CatNthw_CfnPtcInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcInv, val);
}

void CatNthw_CfnPtcIsl(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcIsl, val);
}

void CatNthw_CfnPtcMac(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcMac, val);
}

void CatNthw_CfnPtcL2(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcL2, val);
}

void CatNthw_CfnPtcVnTag(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcVnTag, val);
}

void CatNthw_CfnPtcVlan(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcVlan, val);
}

void CatNthw_CfnPtcMpls(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcMpls, val);
}

void CatNthw_CfnPtcL3(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcL3, val);
}

void CatNthw_CfnPtcFrag(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcFrag, val);
}

void CatNthw_CfnPtcIpProt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcIpProt, val);
}

void CatNthw_CfnPtcL4(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcL4, val);
}

void CatNthw_CfnPtcTunnel(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTunnel, val);
}

void CatNthw_CfnPtcTnlL2(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlL2, val);
}

void CatNthw_CfnPtcTnlVlan(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlVlan, val);
}

void CatNthw_CfnPtcTnlMpls(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlMpls, val);
}

void CatNthw_CfnPtcTnlL3(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlL3, val);
}

void CatNthw_CfnPtcTnlFrag(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlFrag, val);
}

void CatNthw_CfnPtcTnlIpProt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlIpProt, val);
}

void CatNthw_CfnPtcTnlL4(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPtcTnlL4, val);
}

void CatNthw_CfnPtcCfp(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataPtcCfp);
  Field_setVal32(p->mpCfnDataPtcCfp, val);
}

void CatNthw_CfnErrL3Cs(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataErrL3Cs);
  Field_setVal32(p->mpCfnDataErrL3Cs, val);
}

void CatNthw_CfnErrL4Cs(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataErrL4Cs);
  Field_setVal32(p->mpCfnDataErrL4Cs, val);
}

void CatNthw_CfnErrTnlL3Cs(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataErrTnlL3Cs);
  Field_setVal32(p->mpCfnDataErrTnlL3Cs, val);
}

void CatNthw_CfnErrTnlL4Cs(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataErrTnlL4Cs);
  Field_setVal32(p->mpCfnDataErrTnlL4Cs, val);
}

void CatNthw_CfnErrTtlExp(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataErrTtlExp);
  Field_setVal32(p->mpCfnDataErrTtlExp, val);
}

void CatNthw_CfnErrTnlTtlExp(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataErrTnlTtlExp);
  Field_setVal32(p->mpCfnDataErrTnlTtlExp, val);
}

void CatNthw_CfnErrInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataErrInv, val);
}

void CatNthw_CfnErrCv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataErrCv, val);
}

void CatNthw_CfnErrFcs(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataErrFcs, val);
}

void CatNthw_CfnErrTrunc(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataErrTrunc, val);
}

void CatNthw_CfnMacPort(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataMacPort, val);
}

void CatNthw_CfnPmCmp(const struct CatNthw *p, const uint32_t *val)
{
  Field_setVal(p->mpCfnDataPmCmp, val, p->mpCfnDataPmCmp->mnWords);
}

void CatNthw_CfnPmDct(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPmDct, val);
}

void CatNthw_CfnPmExtInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPmExtInv, val);
}

void CatNthw_CfnPmCmb(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPmCmb, val);
}

void CatNthw_CfnPmAndInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPmAndInv, val);
}

void CatNthw_CfnPmOrInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPmOrInv, val);
}

void CatNthw_CfnPmInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataPmInv, val);
}

void CatNthw_CfnLc(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataLc, val);
}

void CatNthw_CfnLcInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataLcInv, val);
}


void CatNthw_CfnKm0Or(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCfnDataKm0Or, val);
}

void CatNthw_CfnKm1Or(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCfnDataKm1Or);
  Field_setVal32(p->mpCfnDataKm1Or, val);
}

void CatNthw_CfnFlush(const struct CatNthw *p)
{
  Register_flush(p->mpCfnCtrl, 1);
  Register_flush(p->mpCfnData, 1);
}



void CatNthw_KceSelect(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpKceAddr[index], val);
}

void CatNthw_KceCnt(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpKceCnt[index], val);
}

void CatNthw_KceEnable(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpKceDataEnable[index], val);
}

void CatNthw_KceFlush(const struct CatNthw *p, int index)
{
  Register_flush(p->mpKceCtrl[index], 1);
  Register_flush(p->mpKceData[index], 1);
}

void CatNthw_KcsSelect(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpKcsAddr[index], val);
}

void CatNthw_KcsCnt(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpKcsCnt[index], val);
}

void CatNthw_KcsCategory(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpKcsDataCategory[index], val);
}

void CatNthw_KcsFlush(const struct CatNthw *p, int index)
{
  Register_flush(p->mpKcsCtrl[index], 1);
  Register_flush(p->mpKcsData[index], 1);
}

void CatNthw_FteSelect(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpFteAddr[index], val);
}

void CatNthw_FteCnt(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpFteCnt[index], val);
}

void CatNthw_FteEnable(const struct CatNthw *p, int index, uint32_t val)
{
  Field_setVal32(p->mpFteDataEnable[index], val);
}

void CatNthw_FteFlush(const struct CatNthw *p, int index)
{
  Register_flush(p->mpFteCtrl[index], 1);
  Register_flush(p->mpFteData[index], 1);
}


void CatNthw_CteSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteAddr, val);
}

void CatNthw_CteCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteCnt, val);
}

void CatNthw_CteEnableCol(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataCol, val);
}

void CatNthw_CteEnableCor(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataCor, val);
}

void CatNthw_CteEnableHsh(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataHsh, val);
}

void CatNthw_CteEnableQsl(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataQsl, val);
}

void CatNthw_CteEnableIpf(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataIpf, val);
}

void CatNthw_CteEnableSlc(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataSlc, val);
}

void CatNthw_CteEnablePdb(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCteDataPdb, val);
}

void CatNthw_CteEnableMsk(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCteDataMsk);
  Field_setVal32(p->mpCteDataMsk, val);
}

void CatNthw_CteEnableHst(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCteDataHst);
  Field_setVal32(p->mpCteDataHst, val);
}

void CatNthw_CteEnableEpp(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCteDataEpp);
  Field_setVal32(p->mpCteDataEpp, val);
}

void CatNthw_CteEnableTpe(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCteDataTpe);
  Field_setVal32(p->mpCteDataTpe, val);
}

void CatNthw_CteEnableRrb(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCteDataRrb);
  Field_setVal32(p->mpCteDataRrb, val);
}


void CatNthw_CteFlush(const struct CatNthw *p)
{
  Register_flush(p->mpCteCtrl, 1);
  Register_flush(p->mpCteData, 1);
}

void CatNthw_CtsSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtsAddr, val);
}

void CatNthw_CtsCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtsCnt, val);
}

void CatNthw_CtsCatA(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtsDataCatA, val);
}

void CatNthw_CtsCatB(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtsDataCatB, val);
}

void CatNthw_CtsFlush(const struct CatNthw *p)
{
  Register_flush(p->mpCtsCtrl, 1);
  Register_flush(p->mpCtsData, 1);
}

void CatNthw_CotSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCotAddr, val);
}

void CatNthw_CotCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCotCnt, val);
}

void CatNthw_CotColor(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCotDataColor, val);
}

void CatNthw_CotKm(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCotDataKm, val);
}

void CatNthw_CotNfvSb(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCotDataNfvSb);
  Field_setVal32(p->mpCotDataNfvSb, val);
}
void CatNthw_CotFlush(const struct CatNthw *p)
{
  Register_flush(p->mpCotCtrl, 1);
  Register_flush(p->mpCotData, 1);
}

void CatNthw_CctSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCctAddr, val);
}

void CatNthw_CctCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCctCnt, val);
}

void CatNthw_CctColor(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCctDataColor, val);
}

void CatNthw_CctKm(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCctDataKm, val);
}

void CatNthw_CctFlush(const struct CatNthw *p)
{
  Register_flush(p->mpCctCtrl, 1);
  Register_flush(p->mpCctData, 1);
}

void CatNthw_ExoSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpExoAddr, val);
}

void CatNthw_ExoCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpExoCnt, val);
}

void CatNthw_ExoDyn(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpExoDataDyn, val);
}

void CatNthw_ExoOfs(const struct CatNthw *p, int32_t val)
{
  Field_setVal32(p->mpExoDataOfs, val);
}

void CatNthw_ExoFlush(const struct CatNthw *p)
{
  Register_flush(p->mpExoCtrl, 1);
  Register_flush(p->mpExoData, 1);
}

void CatNthw_RckSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRckAddr, val);
}

void CatNthw_RckCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRckCnt, val);
}

void CatNthw_RckData(const struct CatNthw *p, uint32_t val)
{
  Register_setVal(p->mpRckData, &val, 1);
  Register_makeDirty(p->mpRckData);
}

void CatNthw_RckFlush(const struct CatNthw *p)
{
  Register_flush(p->mpRckCtrl, 1);
  Register_flush(p->mpRckData, 1);
}

void CatNthw_LenSelect(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenAddr, val);
}

void CatNthw_LenCnt(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenCnt, val);
}

void CatNthw_LenLower(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenDataLower, val);
}

void CatNthw_LenUpper(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenDataUpper, val);
}

void CatNthw_LenDyn1(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenDataDyn1, val);
}

void CatNthw_LenDyn2(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenDataDyn2, val);
}

void CatNthw_LenInv(const struct CatNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLenDataInv, val);
}

void CatNthw_LenFlush(const struct CatNthw *p)
{
  Register_flush(p->mpLenCtrl, 1);
  Register_flush(p->mpLenData, 1);
}

void CatNthw_KccSelect(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpKccAddr);
  Field_setVal32(p->mpKccAddr, val);
}

void CatNthw_KccCnt(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpKccCnt);
  Field_setVal32(p->mpKccCnt, val);
}

void CatNthw_KccKey(const struct CatNthw *p, uint32_t *val)
{
  assert(p->mpKccDataKey);
  Field_setVal(p->mpKccDataKey, val, 2);
}

void CatNthw_KccCategory(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpKccDataCategory);
  Field_setVal32(p->mpKccDataCategory, val);
}

void CatNthw_KccId(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpKccDataId);
  Field_setVal32(p->mpKccDataId, val);
}

void CatNthw_KccFlush(const struct CatNthw *p)
{
  assert(p->mpKccCtrl);
  assert(p->mpKccData);
  Register_flush(p->mpKccCtrl, 1);
  Register_flush(p->mpKccData, 1);
}




void CatNthw_CceSelect(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCceAddr);
  Field_setVal32(p->mpCceAddr, val);
}

void CatNthw_CceCnt(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCceCnt);
  Field_setVal32(p->mpCceCnt, val);
}

void CatNthw_CceDataImm(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCceDataImm);
  Field_setVal32(p->mpCceDataImm, val);
}

void CatNthw_CceDataInd(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCceDataInd);
  Field_setVal32(p->mpCceDataInd, val);
}

void CatNthw_CceFlush(const struct CatNthw *p)
{
  assert(p->mpCceCtrl);
  assert(p->mpCceData);
  Register_flush(p->mpCceCtrl, 1);
  Register_flush(p->mpCceData, 1);
}


void CatNthw_CcsSelect(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCcsAddr);
  Field_setVal32(p->mpCcsAddr, val);
}

void CatNthw_CcsCnt(const struct CatNthw *p, uint32_t val)
{
  assert(p->mpCcsCnt);
  Field_setVal32(p->mpCcsCnt, val);
}

#define CATNTHW_CCS_SET(name) \
void CatNthw_CcsData##name(const struct CatNthw *p, uint32_t val) \
{ assert(p->mpCcsData##name);\
  Field_setVal32(p->mpCcsData##name, val);}

CATNTHW_CCS_SET(CorEn);
CATNTHW_CCS_SET(Cor);
CATNTHW_CCS_SET(HshEn);
CATNTHW_CCS_SET(Hsh);
CATNTHW_CCS_SET(QslEn);
CATNTHW_CCS_SET(Qsl);
CATNTHW_CCS_SET(IpfEn);
CATNTHW_CCS_SET(Ipf);
CATNTHW_CCS_SET(SlcEn);
CATNTHW_CCS_SET(Slc);
CATNTHW_CCS_SET(PdbEn);
CATNTHW_CCS_SET(Pdb);
CATNTHW_CCS_SET(MskEn);
CATNTHW_CCS_SET(Msk);
CATNTHW_CCS_SET(HstEn);
CATNTHW_CCS_SET(Hst);
CATNTHW_CCS_SET(EppEn);
CATNTHW_CCS_SET(Epp);
CATNTHW_CCS_SET(TpeEn);
CATNTHW_CCS_SET(Tpe);
CATNTHW_CCS_SET(RrbEn);
CATNTHW_CCS_SET(Rrb);
CATNTHW_CCS_SET(Sb0Type);
CATNTHW_CCS_SET(Sb0Data);
CATNTHW_CCS_SET(Sb1Type);
CATNTHW_CCS_SET(Sb1Data);
CATNTHW_CCS_SET(Sb2Type);
CATNTHW_CCS_SET(Sb2Data);


void CatNthw_CcsFlush(const struct CatNthw *p)
{
  assert(p->mpCcsCtrl);
  assert(p->mpCcsData);
  Register_flush(p->mpCcsCtrl, 1);
  Register_flush(p->mpCcsData, 1);
}
