/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"
#include "nthw_rac.h"

//
//
//
#include "flow_nthw_flm.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

struct FlmNthw *FlmNthw_new(void)
{
  struct FlmNthw *p = malloc(sizeof(struct FlmNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void FlmNthw_delete(struct FlmNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

void flmNthw_setDebugMode(struct FlmNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mFlm, nDebugMode);
}

int FlmNthw_init(struct FlmNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_FLM, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Flm %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpRac = pFpga->pFpgaInfo->mpNthwRac;

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mFlm = pMod;

#if defined(DEBUG) && (0)
  Module_setDebugMode(p->mFlm, 0xFF); // DEBUG:
#endif

  p->mpControl = Module_getRegister(p->mFlm, FLM_CONTROL);
  p->mpControlEnable = Register_getField(p->mpControl, FLM_CONTROL_ENABLE);
  p->mpControlInit = Register_getField(p->mpControl, FLM_CONTROL_INIT);
  p->mpControlLds = Register_getField(p->mpControl, FLM_CONTROL_LDS);
  p->mpControlLfs = Register_getField(p->mpControl, FLM_CONTROL_LFS);
  p->mpControlLis = Register_getField(p->mpControl, FLM_CONTROL_LIS);
  p->mpControlUds = Register_getField(p->mpControl, FLM_CONTROL_UDS);
  p->mpControlUis = Register_getField(p->mpControl, FLM_CONTROL_UIS);
  p->mpControlRds = Register_getField(p->mpControl, FLM_CONTROL_RDS);
  p->mpControlRis = Register_getField(p->mpControl, FLM_CONTROL_RIS);
  p->mpControlPds = Register_queryField(p->mpControl, FLM_CONTROL_PDS);
  p->mpControlPis = Register_queryField(p->mpControl, FLM_CONTROL_PIS);
  p->mpControlCrcwr = Register_getField(p->mpControl, FLM_CONTROL_CRCWR);
  p->mpControlCrcrd = Register_getField(p->mpControl, FLM_CONTROL_CRCRD);
  p->mpControlRbl = Register_getField(p->mpControl, FLM_CONTROL_RBL);
  p->mpControlEab = Register_getField(p->mpControl, FLM_CONTROL_EAB);
  p->mpControlSplitSdramUsage = Register_getField(p->mpControl, FLM_CONTROL_SPLIT_SDRAM_USAGE);

  p->mpStatus = Module_getRegister(p->mFlm, FLM_STATUS);
  p->mpStatusCalibdone = Register_getField(p->mpStatus, FLM_STATUS_CALIBDONE);
  p->mpStatusInitdone = Register_getField(p->mpStatus, FLM_STATUS_INITDONE);
  p->mpStatusIdle = Register_getField(p->mpStatus, FLM_STATUS_IDLE);
  p->mpStatusCritical = Register_getField(p->mpStatus, FLM_STATUS_CRITICAL);
  p->mpStatusPanic = Register_getField(p->mpStatus, FLM_STATUS_PANIC);
  p->mpStatusCrcerr = Register_getField(p->mpStatus, FLM_STATUS_CRCERR);
  p->mpStatusEftBp = Register_getField(p->mpStatus, FLM_STATUS_EFT_BP);

  p->mpTimeout = Module_getRegister(p->mFlm, FLM_TIMEOUT);
  p->mpTimeoutT = Register_getField(p->mpTimeout, FLM_TIMEOUT_T);

  p->mpScrub = Module_getRegister(p->mFlm, FLM_SCRUB);
  p->mpScrubI = Register_getField(p->mpScrub, FLM_SCRUB_I);

  p->mpLoadBin = Module_getRegister(p->mFlm, FLM_LOAD_BIN);
  p->mpLoadBinBin = Register_getField(p->mpLoadBin, FLM_LOAD_BIN_BIN);

  p->mpLoadPps = Module_getRegister(p->mFlm, FLM_LOAD_PPS);
  p->mpLoadPpsPps = Register_getField(p->mpLoadPps, FLM_LOAD_PPS_PPS);

  p->mpLoadLps = Module_getRegister(p->mFlm, FLM_LOAD_LPS);
  p->mpLoadLpsLps = Register_getField(p->mpLoadLps, FLM_LOAD_LPS_LPS);

  p->mpLoadAps = Module_getRegister(p->mFlm, FLM_LOAD_APS);
  p->mpLoadApsAps = Register_getField(p->mpLoadAps, FLM_LOAD_APS_APS);

  p->mpPrio = Module_getRegister(p->mFlm, FLM_PRIO);
  p->mpPrioLimit0 = Register_getField(p->mpPrio, FLM_PRIO_LIMIT0);
  p->mpPrioFt0 = Register_getField(p->mpPrio, FLM_PRIO_FT0);
  p->mpPrioLimit1 = Register_getField(p->mpPrio, FLM_PRIO_LIMIT1);
  p->mpPrioFt1 = Register_getField(p->mpPrio, FLM_PRIO_FT1);
  p->mpPrioLimit2 = Register_getField(p->mpPrio, FLM_PRIO_LIMIT2);
  p->mpPrioFt2 = Register_getField(p->mpPrio, FLM_PRIO_FT2);
  p->mpPrioLimit3 = Register_getField(p->mpPrio, FLM_PRIO_LIMIT3);
  p->mpPrioFt3 = Register_getField(p->mpPrio, FLM_PRIO_FT3);

  p->mpPstCtrl = Module_getRegister(p->mFlm, FLM_PST_CTRL);
  p->mpPstCtrlAdr = Register_getField(p->mpPstCtrl, FLM_PST_CTRL_ADR);
  p->mpPstCtrlCnt = Register_getField(p->mpPstCtrl, FLM_PST_CTRL_CNT);
  p->mpPstData = Module_getRegister(p->mFlm, FLM_PST_DATA);
  p->mpPstDataBp = Register_getField(p->mpPstData, FLM_PST_DATA_BP);
  p->mpPstDataPp = Register_getField(p->mpPstData, FLM_PST_DATA_PP);
  p->mpPstDataTp = Register_getField(p->mpPstData, FLM_PST_DATA_TP);

  p->mpRcpCtrl = Module_getRegister(p->mFlm, FLM_RCP_CTRL);
  p->mpRcpCtrlAdr = Register_getField(p->mpRcpCtrl, FLM_RCP_CTRL_ADR);
  p->mpRcpCtrlCnt = Register_getField(p->mpRcpCtrl, FLM_RCP_CTRL_CNT);
  p->mpRcpData = Module_getRegister(p->mFlm, FLM_RCP_DATA);
  p->mpRcpDataLookup = Register_getField(p->mpRcpData, FLM_RCP_DATA_LOOKUP);
  p->mpRcpDataQw0Dyn = Register_getField(p->mpRcpData, FLM_RCP_DATA_QW0_DYN);
  p->mpRcpDataQw0Ofs = Register_getField(p->mpRcpData, FLM_RCP_DATA_QW0_OFS);
  p->mpRcpDataQw0Sel = Register_getField(p->mpRcpData, FLM_RCP_DATA_QW0_SEL);
  p->mpRcpDataQw4Dyn = Register_getField(p->mpRcpData, FLM_RCP_DATA_QW4_DYN);
  p->mpRcpDataQw4Ofs = Register_getField(p->mpRcpData, FLM_RCP_DATA_QW4_OFS);
  p->mpRcpDataSw8Dyn = Register_getField(p->mpRcpData, FLM_RCP_DATA_SW8_DYN);
  p->mpRcpDataSw8Ofs = Register_getField(p->mpRcpData, FLM_RCP_DATA_SW8_OFS);
  p->mpRcpDataSw8Sel = Register_getField(p->mpRcpData, FLM_RCP_DATA_SW8_SEL);
  p->mpRcpDataSw9Dyn = Register_getField(p->mpRcpData, FLM_RCP_DATA_SW9_DYN);
  p->mpRcpDataSw9Ofs = Register_getField(p->mpRcpData, FLM_RCP_DATA_SW9_OFS);
  p->mpRcpDataMask = Register_getField(p->mpRcpData, FLM_RCP_DATA_MASK);
  p->mpRcpDataKid = Register_getField(p->mpRcpData, FLM_RCP_DATA_KID);
  p->mpRcpDataOpn = Register_getField(p->mpRcpData, FLM_RCP_DATA_OPN);
  p->mpRcpDataIpn = Register_getField(p->mpRcpData, FLM_RCP_DATA_IPN);
  p->mpRcpDataBytDyn = Register_getField(p->mpRcpData, FLM_RCP_DATA_BYT_DYN);
  p->mpRcpDataBytOfs = Register_getField(p->mpRcpData, FLM_RCP_DATA_BYT_OFS);
  p->mpRcpDataTxplm = Register_getField(p->mpRcpData, FLM_RCP_DATA_TXPLM);
  p->mpRcpDataAutoIpv4Mask = Register_getField(p->mpRcpData, FLM_RCP_DATA_AUTO_IPV4_MASK);

  p->mpBufCtrl = Module_getRegister(p->mFlm, FLM_BUF_CTRL);

  p->mpLrnData = Module_getRegister(p->mFlm, FLM_LRN_DATA);
  p->mpInfData = Module_getRegister(p->mFlm, FLM_INF_DATA);
  p->mpStaData = Module_getRegister(p->mFlm, FLM_STA_DATA);

  p->mpStatLrnDone = Module_getRegister(p->mFlm, FLM_STAT_LRN_DONE);
  p->mpStatLrnDoneCnt = Register_getField(p->mpStatLrnDone, FLM_STAT_LRN_DONE_CNT);

  p->mpStatLrnIgnore = Module_getRegister(p->mFlm, FLM_STAT_LRN_IGNORE);
  p->mpStatLrnIgnoreCnt = Register_getField(p->mpStatLrnIgnore, FLM_STAT_LRN_IGNORE_CNT);

  p->mpStatLrnFail = Module_getRegister(p->mFlm, FLM_STAT_LRN_FAIL);
  p->mpStatLrnFailCnt = Register_getField(p->mpStatLrnFail, FLM_STAT_LRN_FAIL_CNT);

  p->mpStatUnlDone = Module_getRegister(p->mFlm, FLM_STAT_UNL_DONE);
  p->mpStatUnlDoneCnt = Register_getField(p->mpStatUnlDone, FLM_STAT_UNL_DONE_CNT);

  p->mpStatUnlIgnore = Module_getRegister(p->mFlm, FLM_STAT_UNL_IGNORE);
  p->mpStatUnlIgnoreCnt = Register_getField(p->mpStatUnlIgnore, FLM_STAT_UNL_IGNORE_CNT);

  p->mpStatPrbDone = Module_queryRegister(p->mFlm, FLM_STAT_PRB_DONE);
  p->mpStatPrbDoneCnt = Register_queryField(p->mpStatPrbDone, FLM_STAT_PRB_DONE_CNT);

  p->mpStatPrbIgnore = Module_queryRegister(p->mFlm, FLM_STAT_PRB_IGNORE);
  p->mpStatPrbIgnoreCnt = Register_queryField(p->mpStatPrbIgnore, FLM_STAT_PRB_IGNORE_CNT);

  p->mpStatRelDone = Module_getRegister(p->mFlm, FLM_STAT_REL_DONE);
  p->mpStatRelDoneCnt = Register_getField(p->mpStatRelDone, FLM_STAT_REL_DONE_CNT);

  p->mpStatRelIgnore = Module_getRegister(p->mFlm, FLM_STAT_REL_IGNORE);
  p->mpStatRelIgnoreCnt = Register_getField(p->mpStatRelIgnore, FLM_STAT_REL_IGNORE_CNT);

  p->mpStatAulDone = Module_getRegister(p->mFlm, FLM_STAT_AUL_DONE);
  p->mpStatAulDoneCnt = Register_getField(p->mpStatAulDone, FLM_STAT_AUL_DONE_CNT);

  p->mpStatAulIgnore = Module_getRegister(p->mFlm, FLM_STAT_AUL_IGNORE);
  p->mpStatAulIgnoreCnt = Register_getField(p->mpStatAulIgnore, FLM_STAT_AUL_IGNORE_CNT);

  p->mpStatAulFail = Module_getRegister(p->mFlm, FLM_STAT_AUL_FAIL);
  p->mpStatAulFailCnt = Register_getField(p->mpStatAulFail, FLM_STAT_AUL_FAIL_CNT);

  p->mpStatTulDone = Module_getRegister(p->mFlm, FLM_STAT_TUL_DONE);
  p->mpStatTulDoneCnt = Register_getField(p->mpStatTulDone, FLM_STAT_TUL_DONE_CNT);

  p->mpStatFlows = Module_getRegister(p->mFlm, FLM_STAT_FLOWS);
  p->mpStatFlowsCnt = Register_getField(p->mpStatFlows, FLM_STAT_FLOWS_CNT);

  return 0;
}

void FlmNthw_ControlEnable(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlEnable, val);
}

void FlmNthw_ControlInit(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlInit, val);
}

void FlmNthw_ControlLds(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlLds, val);
}

void FlmNthw_ControlLfs(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlLfs, val);
}

void FlmNthw_ControlLis(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlLis, val);
}

void FlmNthw_ControlUds(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlUds, val);
}

void FlmNthw_ControlUis(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlUis, val);
}

void FlmNthw_ControlRds(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlRds, val);
}

void FlmNthw_ControlRis(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlRis, val);
}

void FlmNthw_ControlPds(const struct FlmNthw *p, uint32_t val)
{
  assert(p->mpControlPds);
  Field_setVal32(p->mpControlPds, val);
}

void FlmNthw_ControlPis(const struct FlmNthw *p, uint32_t val)
{
  assert(p->mpControlPis);
  Field_setVal32(p->mpControlPis, val);
}

void FlmNthw_ControlCrcwr(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlCrcwr, val);
}

void FlmNthw_ControlCrcrd(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlCrcrd, val);
}

void FlmNthw_ControlRbl(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlRbl, val);
}

void FlmNthw_ControlEab(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlEab, val);
}

void FlmNthw_ControlSplitSdramUsage(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpControlSplitSdramUsage, val);
}

void FlmNthw_ControlFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpControl, 1);
}

void FlmNthw_StatusCalibdone(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusCalibdone);
  }
}

void FlmNthw_StatusInitdone(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusInitdone);
  }
}

void FlmNthw_StatusIdle(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusIdle);
  }
}

void FlmNthw_StatusCritical(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusCritical);
  }
  else {
    Field_setVal32(p->mpStatusCritical, *val);
  }
}

void FlmNthw_StatusPanic(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusPanic);
  }
  else {
    Field_setVal32(p->mpStatusPanic, *val);
  }
}

void FlmNthw_StatusCrcerr(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusCrcerr);
  }
  else {
    Field_setVal32(p->mpStatusCrcerr, *val);
  }
}

void FlmNthw_StatusEftBp(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatusEftBp);
  }
}

void FlmNthw_StatusFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpStatus, 1);
}

void FlmNthw_StatusUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatus);
}

void FlmNthw_TimeoutT(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpTimeoutT, val);
}

void FlmNthw_TimeoutFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpTimeout, 1);
}

void FlmNthw_ScrubI(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpScrubI, val);
}

void FlmNthw_ScrubFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpScrub, 1);
}

void FlmNthw_LoadBin(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLoadBinBin, val);
}

void FlmNthw_LoadBinFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpLoadBin, 1);
}

void FlmNthw_LoadPps(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLoadPpsPps, val);
}

void FlmNthw_LoadPpsFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpLoadPps, 1);
}

void FlmNthw_LoadLps(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLoadLpsLps, val);
}

void FlmNthw_LoadLpsFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpLoadLps, 1);
}

void FlmNthw_LoadAps(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpLoadApsAps, val);
}

void FlmNthw_LoadApsFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpLoadAps, 1);
}

void FlmNthw_PrioLimit0(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioLimit0, val);
}

void FlmNthw_PrioFt0(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioFt0, val);
}

void FlmNthw_PrioLimit1(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioLimit1, val);
}

void FlmNthw_PrioFt1(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioFt1, val);
}

void FlmNthw_PrioLimit2(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioLimit2, val);
}

void FlmNthw_PrioFt2(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioFt2, val);
}

void FlmNthw_PrioLimit3(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioLimit3, val);
}

void FlmNthw_PrioFt3(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPrioFt3, val);
}

void FlmNthw_PrioFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpPrio, 1);
}

void FlmNthw_PstSelect(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPstCtrlAdr, val);
}

void FlmNthw_PstCnt(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPstCtrlCnt, val);
}

void FlmNthw_PstBp(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPstDataBp, val);
}

void FlmNthw_PstPp(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPstDataPp, val);
}

void FlmNthw_PstTp(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpPstDataTp, val);
}

void FlmNthw_PstFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpPstCtrl, 1);
  Register_flush(p->mpPstData, 1);
}

void FlmNthw_RcpSelect(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCtrlAdr, val);
}

void FlmNthw_RcpCnt(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCtrlCnt, val);
}

void FlmNthw_RcpLookup(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLookup, val);
}

void FlmNthw_RcpQw0Dyn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw0Dyn, val);
}

void FlmNthw_RcpQw0Ofs(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw0Ofs, val);
}

void FlmNthw_RcpQw0Sel(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw0Sel, val);
}

void FlmNthw_RcpQw4Dyn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw4Dyn, val);
}

void FlmNthw_RcpQw4Ofs(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw4Ofs, val);
}

void FlmNthw_RcpSw8Dyn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSw8Dyn, val);
}

void FlmNthw_RcpSw8Ofs(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSw8Ofs, val);
}

void FlmNthw_RcpSw8Sel(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSw8Sel, val);
}

void FlmNthw_RcpSw9Dyn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSw9Dyn, val);
}

void FlmNthw_RcpSw9Ofs(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSw9Ofs, val);
}

void FlmNthw_RcpMask(const struct FlmNthw *p, const uint32_t *val)
{
  Field_setVal(p->mpRcpDataMask, val, 10);
}

void FlmNthw_RcpKid(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataKid, val);
}

void FlmNthw_RcpOpn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataOpn, val);
}

void FlmNthw_RcpIpn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataIpn, val);
}

void FlmNthw_RcpBytDyn(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataBytDyn, val);
}

void FlmNthw_RcpBytOfs(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataBytOfs, val);
}

void FlmNthw_RcpTxplm(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTxplm, val);
}

void FlmNthw_RcpAutoIpv4Mask(const struct FlmNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataAutoIpv4Mask, val);
}

void FlmNthw_RcpFlush(const struct FlmNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
}

int FlmNthw_BufCtrlUpdate(const struct FlmNthw *p, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail)
{
  int ret = -1;

  struct NthwRac* rac = (struct NthwRac*)p->mpRac;
  uint32_t address_bufctrl = Register_getAddress(p->mpBufCtrl);
  RabBusId_t busId = 1;
  struct DmaBufPtr bc_buf;
  if ((ret = NthwRac_rab_dma_begin(rac)) == 0) {
    NthwRac_rab_read32_dma(rac, address_bufctrl, busId, 2, &bc_buf);
    if ((ret = NthwRac_rab_dma_commit(rac)) != 0) {
      return ret;
    }

    uint32_t bc_mask = bc_buf.size - 1;
    uint32_t bc_index = bc_buf.index;
    *lrn_free = bc_buf.base[bc_index & bc_mask] & 0xffff;
    *inf_avail = (bc_buf.base[bc_index & bc_mask] >> 16) & 0xffff;
    *sta_avail = bc_buf.base[(bc_index + 1) & bc_mask] & 0xffff;
  }

  return ret;
}

int FlmNthw_LrnDataFlush(const struct FlmNthw *p, const uint32_t *data, uint32_t wordCount, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail)
{
  int ret = -1;

  struct NthwRac* rac = (struct NthwRac*)p->mpRac;
  uint32_t address = Register_getAddress(p->mpLrnData);
  uint32_t address_bufctrl = Register_getAddress(p->mpBufCtrl);
  RabBusId_t busId = 1;
  struct DmaBufPtr bc_buf;

  if (NthwRac_rab_dma_begin(rac) == 0) {
    // Announce the number of words to write to LRN_DATA
    uint32_t bufctrl_data[2];
    bufctrl_data[0] = wordCount;
    bufctrl_data[1]= 0;
    NthwRac_rab_write32_dma(rac, address_bufctrl, busId, 2, bufctrl_data);
    NthwRac_rab_write32_dma(rac, address, busId, wordCount, data);
    NthwRac_rab_read32_dma(rac, address_bufctrl, busId, 2, &bc_buf);
    if ((ret = NthwRac_rab_dma_commit(rac)) != 0) {
      return ret;
    }

    uint32_t bc_mask = bc_buf.size - 1;
    uint32_t bc_index = bc_buf.index;
    *lrn_free = bc_buf.base[bc_index & bc_mask] & 0xffff;
    *inf_avail = (bc_buf.base[bc_index & bc_mask] >> 16) & 0xffff;
    *sta_avail = bc_buf.base[(bc_index + 1) & bc_mask] & 0xffff;
  }

  return ret;
}

int FlmNthw_InfDataUpdate(const struct FlmNthw *p, uint32_t *data, uint32_t wordCount, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail)
{
  int ret = -1;

  struct NthwRac* rac = (struct NthwRac*)p->mpRac;
  uint32_t address_infdata = Register_getAddress(p->mpInfData);
  uint32_t address_bufctrl = Register_getAddress(p->mpBufCtrl);
  RabBusId_t busId = 1;
  struct DmaBufPtr buf;
  struct DmaBufPtr bc_buf;
  if ((ret = NthwRac_rab_dma_begin(rac)) == 0) {
    // Announce the number of words to read from INF_DATA
    uint32_t bufctrl_data[2];
    bufctrl_data[0] = wordCount << 16;
    bufctrl_data[1]= 0;
    NthwRac_rab_write32_dma(rac, address_bufctrl, busId, 2, bufctrl_data);
    NthwRac_rab_read32_dma(rac, address_infdata, busId, wordCount, &buf);
    NthwRac_rab_read32_dma(rac, address_bufctrl, busId, 2, &bc_buf);
    if ((ret = NthwRac_rab_dma_commit(rac)) != 0) {
      return ret;
    }

    uint32_t mask = buf.size - 1;
    uint32_t index = buf.index;
    for (uint32_t i = 0; i < wordCount; ++index, ++i) {
      data[i] = buf.base[index & mask];
    }

    uint32_t bc_mask = bc_buf.size - 1;
    uint32_t bc_index = bc_buf.index;
    *lrn_free = bc_buf.base[bc_index & bc_mask] & 0xffff;
    *inf_avail = (bc_buf.base[bc_index & bc_mask] >> 16) & 0xffff;
    *sta_avail = bc_buf.base[(bc_index + 1) & bc_mask] & 0xffff;
  }

  return ret;
}

int FlmNthw_StaDataUpdate(const struct FlmNthw *p, uint32_t *data, uint32_t wordCount, uint32_t* lrn_free, uint32_t* inf_avail, uint32_t* sta_avail)
{
  int ret = -1;

  struct NthwRac* rac = (struct NthwRac*)p->mpRac;
  uint32_t address_stadata = Register_getAddress(p->mpStaData);
  uint32_t address_bufctrl = Register_getAddress(p->mpBufCtrl);
  RabBusId_t busId = 1;
  struct DmaBufPtr buf;
  struct DmaBufPtr bc_buf;
  if ((ret = NthwRac_rab_dma_begin(rac)) == 0) {
    // Announce the number of words to read from STA_DATA
    uint32_t bufctrl_data[2];
    bufctrl_data[0] = 0;
    bufctrl_data[1]= wordCount;
    NthwRac_rab_write32_dma(rac, address_bufctrl, busId, 2, bufctrl_data);
    NthwRac_rab_read32_dma(rac, address_stadata, busId, wordCount, &buf);
    NthwRac_rab_read32_dma(rac, address_bufctrl, busId, 2, &bc_buf);
    if ((ret = NthwRac_rab_dma_commit(rac)) != 0) {
      return ret;
    }

    uint32_t mask = buf.size - 1;
    uint32_t index = buf.index;
    for (uint32_t i = 0; i < wordCount; ++index, ++i) {
      data[i] = buf.base[index & mask];
    }

    uint32_t bc_mask = bc_buf.size - 1;
    uint32_t bc_index = bc_buf.index;
    *lrn_free = bc_buf.base[bc_index & bc_mask] & 0xffff;
    *inf_avail = (bc_buf.base[bc_index & bc_mask] >> 16) & 0xffff;
    *sta_avail = bc_buf.base[(bc_index + 1) & bc_mask] & 0xffff;
  }

  return ret;
}

void FlmNthw_StatLrnDoneCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatLrnDoneCnt);
  }
}

void FlmNthw_StatLrnDoneUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatLrnDone);
}

void FlmNthw_StatLrnIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatLrnIgnoreCnt);
  }
}

void FlmNthw_StatLrnIgnoreUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatLrnIgnore);
}

void FlmNthw_StatLrnFailCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatLrnFailCnt);
  }
}

void FlmNthw_StatLrnFailUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatLrnFail);
}

void FlmNthw_StatUnlDoneCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatUnlDoneCnt);
  }
}

void FlmNthw_StatUnlDoneUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatUnlDone);
}

void FlmNthw_StatUnlIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatUnlIgnoreCnt);
  }
}

void FlmNthw_StatUnlIgnoreUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatUnlIgnore);
}

void FlmNthw_StatPrbDoneCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  assert(p->mpStatPrbDoneCnt);
  if (get) {
    *val = Field_getVal32(p->mpStatPrbDoneCnt);
  }
}

void FlmNthw_StatPrbDoneUpdate(const struct FlmNthw *p)
{
  assert(p->mpStatPrbDone);
  Register_update(p->mpStatPrbDone);
}

void FlmNthw_StatPrbIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  assert(p->mpStatPrbIgnoreCnt);
  if (get) {
    *val = Field_getVal32(p->mpStatPrbIgnoreCnt);
  }
}

void FlmNthw_StatPrbIgnoreUpdate(const struct FlmNthw *p)
{
  assert(p->mpStatPrbIgnore);
  Register_update(p->mpStatPrbIgnore);
}

void FlmNthw_StatRelDoneCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatRelDoneCnt);
  }
}

void FlmNthw_StatRelDoneUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatRelDone);
}

void FlmNthw_StatRelIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatRelIgnoreCnt);
  }
}

void FlmNthw_StatRelIgnoreUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatRelIgnore);
}

void FlmNthw_StatAulDoneCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatAulDoneCnt);
  }
}

void FlmNthw_StatAulDoneUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatAulDone);
}

void FlmNthw_StatAulIgnoreCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatAulIgnoreCnt);
  }
}

void FlmNthw_StatAulIgnoreUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatAulIgnore);
}

void FlmNthw_StatAulFailCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatAulFailCnt);
  }
}

void FlmNthw_StatAulFailUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatAulFail);
}

void FlmNthw_StatTulDoneCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatTulDoneCnt);
  }
}

void FlmNthw_StatTulDoneUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatTulDone);
}

void FlmNthw_StatFlowsCnt(const struct FlmNthw *p, uint32_t *val, int get)
{
  if (get) {
    *val = Field_getVal32(p->mpStatFlowsCnt);
  }
}

void FlmNthw_StatFlowsUpdate(const struct FlmNthw *p)
{
  Register_update(p->mpStatFlows);
}
