/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
#include "nthw_drv.h"
#include "nthw_register.h"

#include "flow_nthw_hfu.h"

#include <stdlib.h>
#include <string.h>

void hfuNthw_setDebugMode(struct HfuNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mHfu, nDebugMode);
}

struct HfuNthw *HfuNthw_new(void)
{
  struct HfuNthw *p = malloc(sizeof(struct HfuNthw));
  if (p) {
    (void)memset(p, 0, sizeof(*p));
  }
  return p;
}

void HfuNthw_delete(struct HfuNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int HfuNthw_init(struct HfuNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_HFU, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Hfu %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mHfu = Fpga_queryModule(pFpga, MOD_HFU, nInstance);

  p->mpRcpCtrl = Module_getRegister(p->mHfu, HFU_RCP_CTRL);
  p->mpRcpAddr = Register_getField(p->mpRcpCtrl, HFU_RCP_CTRL_ADR);
  p->mpRcpCnt = Register_getField(p->mpRcpCtrl, HFU_RCP_CTRL_CNT);

  p->mpRcpData = Module_getRegister(p->mHfu, HFU_RCP_DATA);
  p->mpRcpDataLenAWr = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_WR);
  p->mpRcpDataLenAOl4len = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_OL4LEN);
  p->mpRcpDataLenAPosDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_POS_DYN);
  p->mpRcpDataLenAPosOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_POS_OFS);
  p->mpRcpDataLenAAddDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_ADD_DYN);
  p->mpRcpDataLenAAddOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_ADD_OFS);
  p->mpRcpDataLenASubDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_A_SUB_DYN);
  p->mpRcpDataLenBWr = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_B_WR);
  p->mpRcpDataLenBPosDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_B_POS_DYN);
  p->mpRcpDataLenBPosOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_B_POS_OFS);
  p->mpRcpDataLenBAddDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_B_ADD_DYN);
  p->mpRcpDataLenBAddOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_B_ADD_OFS);
  p->mpRcpDataLenBSubDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_B_SUB_DYN);
  p->mpRcpDataLenCWr = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_C_WR);
  p->mpRcpDataLenCPosDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_C_POS_DYN);
  p->mpRcpDataLenCPosOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_C_POS_OFS);
  p->mpRcpDataLenCAddDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_C_ADD_DYN);
  p->mpRcpDataLenCAddOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_C_ADD_OFS);
  p->mpRcpDataLenCSubDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_LEN_C_SUB_DYN);
  p->mpRcpDataTtlWr = Register_getField(p->mpRcpData, HFU_RCP_DATA_TTL_WR);
  p->mpRcpDataTtlPosDyn = Register_getField(p->mpRcpData, HFU_RCP_DATA_TTL_POS_DYN);
  p->mpRcpDataTtlPosOfs = Register_getField(p->mpRcpData, HFU_RCP_DATA_TTL_POS_OFS);
  p->mpRcpDataCsinf = Register_getField(p->mpRcpData, HFU_RCP_DATA_CSINF);
  p->mpRcpDataL3prt = Register_getField(p->mpRcpData, HFU_RCP_DATA_L3PRT);
  p->mpRcpDataL3frag = Register_getField(p->mpRcpData, HFU_RCP_DATA_L3FRAG);
  p->mpRcpDataTunnel = Register_getField(p->mpRcpData, HFU_RCP_DATA_TUNNEL);
  p->mpRcpDataL4prt = Register_getField(p->mpRcpData, HFU_RCP_DATA_L4PRT);
  p->mpRcpDataOl3ofs = Register_getField(p->mpRcpData, HFU_RCP_DATA_OL3OFS);
  p->mpRcpDataOl4ofs = Register_getField(p->mpRcpData, HFU_RCP_DATA_OL4OFS);
  p->mpRcpDataIl3ofs = Register_getField(p->mpRcpData, HFU_RCP_DATA_IL3OFS);
  p->mpRcpDataIl4ofs = Register_getField(p->mpRcpData, HFU_RCP_DATA_IL4OFS);

  return 0;
}

void HfuNthw_RcpSelect(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpAddr, val);
}

void HfuNthw_RcpCnt(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCnt, val);
}

void HfuNthw_RcpLenAWr(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenAWr, val);
}

void HfuNthw_RcpLenAOl4len(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenAOl4len, val);
}

void HfuNthw_RcpLenAPosDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenAPosDyn, val);
}

void HfuNthw_RcpLenAPosOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenAPosOfs, val);
}

void HfuNthw_RcpLenAAddDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenAAddDyn, val);
}

void HfuNthw_RcpLenAAddOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenAAddOfs, val);
}

void HfuNthw_RcpLenASubDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenASubDyn, val);
}

void HfuNthw_RcpLenBWr(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenBWr, val);
}

void HfuNthw_RcpLenBPosDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenBPosDyn, val);
}

void HfuNthw_RcpLenBPosOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenBPosOfs, val);
}

void HfuNthw_RcpLenBAddDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenBAddDyn, val);
}

void HfuNthw_RcpLenBAddOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenBAddOfs, val);
}

void HfuNthw_RcpLenBSubDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenBSubDyn, val);
}

void HfuNthw_RcpLenCWr(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenCWr, val);
}

void HfuNthw_RcpLenCPosDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenCPosDyn, val);
}

void HfuNthw_RcpLenCPosOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenCPosOfs, val);
}

void HfuNthw_RcpLenCAddDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenCAddDyn, val);
}

void HfuNthw_RcpLenCAddOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenCAddOfs, val);
}

void HfuNthw_RcpLenCSubDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLenCSubDyn, val);
}

void HfuNthw_RcpTtlWr(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTtlWr, val);
}

void HfuNthw_RcpTtlPosDyn(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTtlPosDyn, val);
}

void HfuNthw_RcpTtlPosOfs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTtlPosOfs, val);
}

void HfuNthw_RcpCsinf(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataCsinf, val);
}

void HfuNthw_RcpL3prt(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataL3prt, val);
}

void HfuNthw_RcpL3frag(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataL3frag, val);
}

void HfuNthw_RcpTunnel(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTunnel, val);
}

void HfuNthw_RcpL4prt(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataL4prt, val);
}

void HfuNthw_RcpOl3ofs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataOl3ofs, val);
}

void HfuNthw_RcpOl4ofs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataOl4ofs, val);
}

void HfuNthw_RcpIl3ofs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataIl3ofs, val);
}

void HfuNthw_RcpIl4ofs(const struct HfuNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataIl4ofs, val);
}

void HfuNthw_RcpFlush(const struct HfuNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
}
