/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "flow_nthw_hsh.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

void hshNthw_setDebugMode(struct HshNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mHsh, nDebugMode);
}

struct HshNthw *HshNthw_new(void)
{
  struct HshNthw *p = malloc(sizeof(struct HshNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void HshNthw_delete(struct HshNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int HshNthw_init(struct HshNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_HSH, nInstance);
  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Hsh %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mHsh = pMod;

#if defined(DEBUG) && (0)
  Module_setDebugMode(p->mHsh, 0xFF); // DEBUG:
#endif

  // RCP
  p->mpRcpCtrl = Module_getRegister(p->mHsh, HSH_RCP_CTRL);
  p->mpRcpAddr = Register_getField(p->mpRcpCtrl, HSH_RCP_CTRL_ADR);
  p->mpRcpCnt = Register_getField(p->mpRcpCtrl, HSH_RCP_CTRL_CNT);
  p->mpRcpData = Module_getRegister(p->mHsh, HSH_RCP_DATA);
  p->mpRcpDataLoadDistType =     Register_getField(p->mpRcpData, HSH_RCP_DATA_LOAD_DIST_TYPE);
  p->mpRcpDataMacPortMask =     Register_getField(p->mpRcpData, HSH_RCP_DATA_MAC_PORT_MASK);
  p->mpRcpDataSort = Register_getField(p->mpRcpData, HSH_RCP_DATA_SORT);
  p->mpRcpDataQw0Pe = Register_getField(p->mpRcpData, HSH_RCP_DATA_QW0_PE);
  p->mpRcpDataQw0Ofs = Register_getField(p->mpRcpData, HSH_RCP_DATA_QW0_OFS);
  p->mpRcpDataQw4Pe = Register_getField(p->mpRcpData, HSH_RCP_DATA_QW4_PE);
  p->mpRcpDataQw4Ofs = Register_getField(p->mpRcpData, HSH_RCP_DATA_QW4_OFS);
  p->mpRcpDataW8Pe = Register_getField(p->mpRcpData, HSH_RCP_DATA_W8_PE);
  p->mpRcpDataW8Ofs = Register_getField(p->mpRcpData, HSH_RCP_DATA_W8_OFS);
  p->mpRcpDataW8Sort = Register_getField(p->mpRcpData, HSH_RCP_DATA_W8_SORT);
  p->mpRcpDataW9Pe = Register_getField(p->mpRcpData, HSH_RCP_DATA_W9_PE);
  p->mpRcpDataW9Ofs = Register_getField(p->mpRcpData, HSH_RCP_DATA_W9_OFS);
  p->mpRcpDataW9Sort = Register_getField(p->mpRcpData, HSH_RCP_DATA_W9_SORT);
  p->mpRcpDataW9P = Register_getField(p->mpRcpData, HSH_RCP_DATA_W9_P);
  p->mpRcpDataPMask = Register_getField(p->mpRcpData, HSH_RCP_DATA_P_MASK);
  p->mpRcpDataWordMask =     Register_getField(p->mpRcpData, HSH_RCP_DATA_WORD_MASK);
  p->mpRcpDataSeed = Register_getField(p->mpRcpData, HSH_RCP_DATA_SEED);
  p->mpRcpDataTnlP = Register_getField(p->mpRcpData, HSH_RCP_DATA_TNL_P);
  p->mpRcpDataHshValid =     Register_getField(p->mpRcpData, HSH_RCP_DATA_HSH_VALID);
  p->mpRcpDataHshType = Register_getField(p->mpRcpData, HSH_RCP_DATA_HSH_TYPE);
  p->mpRcpDataAutoIpv4Mask = Register_queryField(p->mpRcpData, HSH_RCP_DATA_AUTO_IPV4_MASK);

  // Init
  uint32_t val[10] = {0,0,0,0,0,0,0,0,0,0};
  Field_setVal32(p->mpRcpAddr, 0);
  Field_setVal32(p->mpRcpCnt, 1);

  Field_setVal32(p->mpRcpDataLoadDistType, 0);
  Field_setVal(p->mpRcpDataMacPortMask, val, p->mpRcpDataMacPortMask->mnWords);
  Field_setVal32(p->mpRcpDataSort, 0);
  Field_setVal32(p->mpRcpDataQw0Pe, 0);
  Field_setVal32(p->mpRcpDataQw0Ofs, 0);
  Field_setVal32(p->mpRcpDataQw4Pe, 0);
  Field_setVal32(p->mpRcpDataQw4Ofs, 0);
  Field_setVal32(p->mpRcpDataW8Pe, 0);
  Field_setVal32(p->mpRcpDataW8Ofs, 0);
  Field_setVal32(p->mpRcpDataW8Sort, 0);
  Field_setVal32(p->mpRcpDataW9Pe, 0);
  Field_setVal32(p->mpRcpDataW9Ofs, 0);
  Field_setVal32(p->mpRcpDataW9Sort, 0);
  Field_setVal32(p->mpRcpDataW9P, 0);
  Field_setVal(p->mpRcpDataWordMask, val, 10);
  Field_setVal32(p->mpRcpDataSeed, 0);
  Field_setVal32(p->mpRcpDataTnlP, 0);
  Field_setVal32(p->mpRcpDataHshValid, 0);
  Field_setVal32(p->mpRcpDataHshType, 31);

  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);

  return 0;
}

void HshNthw_RcpSelect(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpAddr, val);
}

void HshNthw_RcpCnt(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCnt, val);
}

void HshNthw_RcpLoadDistType(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLoadDistType, val);
}

void HshNthw_RcpMacPortMask(const struct HshNthw *p, uint32_t *val)
{
  Field_setVal(p->mpRcpDataMacPortMask, val, p->mpRcpDataMacPortMask->mnWords);
}

void HshNthw_RcpSort(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSort, val);
}

void HshNthw_RcpQw0Pe(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw0Pe, val);
}

void HshNthw_RcpQw0Ofs(const struct HshNthw *p, int32_t val)
{
  Field_setVal32(p->mpRcpDataQw0Ofs, val);
}

void HshNthw_RcpQw4Pe(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataQw4Pe, val);
}

void HshNthw_RcpQw4Ofs(const struct HshNthw *p, int32_t val)
{
  Field_setVal32(p->mpRcpDataQw4Ofs, val);
}

void HshNthw_RcpW8Pe(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataW8Pe, val);
}

void HshNthw_RcpW8Ofs(const struct HshNthw *p, int32_t val)
{
  Field_setVal32(p->mpRcpDataW8Ofs, val);
}

void HshNthw_RcpW8Sort(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataW8Sort, val);
}

void HshNthw_RcpW9Pe(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataW9Pe, val);
}

void HshNthw_RcpW9Ofs(const struct HshNthw *p, int32_t val)
{
  Field_setVal32(p->mpRcpDataW9Ofs, val);
}

void HshNthw_RcpW9Sort(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataW9Sort, val);
}

void HshNthw_RcpW9P(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataW9P, val);
}

void HshNthw_RcpPMask(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataPMask, val);
}

void HshNthw_RcpWordMask(const struct HshNthw *p, uint32_t *val)
{
  Field_setVal(p->mpRcpDataWordMask, val, 10);
}

void HshNthw_RcpSeed(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataSeed, val);
}

void HshNthw_RcpTnlP(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTnlP, val);
}

void HshNthw_RcpHshValid(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataHshValid, val);
}

void HshNthw_RcpHshType(const struct HshNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataHshType, val);
}

void HshNthw_RcpAutoIpv4Mask(const struct HshNthw *p, uint32_t val)
{
  if (p->mpRcpDataAutoIpv4Mask) {
    Field_setVal32(p->mpRcpDataAutoIpv4Mask, val);
  }
}

void HshNthw_RcpFlush(const struct HshNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
}
