/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "flow_nthw_hst.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

void hstNthw_setDebugMode(struct HstNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mHst, nDebugMode);
}

struct HstNthw *HstNthw_new(void)
{
  struct HstNthw *p = malloc(sizeof(struct HstNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void HstNthw_delete(struct HstNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int HstNthw_init(struct HstNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_HST, nInstance);
  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Hst %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mHst = pMod;

#if defined(DEBUG) && (0)
  Module_setDebugMode(p->mHst, 0xFF); // DEBUG:
#endif

  // RCP
  p->mpRcpCtrl = Module_getRegister(p->mHst, HST_RCP_CTRL);
  p->mpRcpAddr = Register_getField(p->mpRcpCtrl, HST_RCP_CTRL_ADR);
  p->mpRcpCnt = Register_getField(p->mpRcpCtrl, HST_RCP_CTRL_CNT);

  p->mpRcpData = Module_getRegister(p->mHst, HST_RCP_DATA);
  p->mpRcpDataStripMode = Register_getField(p->mpRcpData, HST_RCP_DATA_STRIP_MODE);
  p->mpRcpDataStartDyn = Register_getField(p->mpRcpData, HST_RCP_DATA_START_DYN);
  p->mpRcpDataStartOfs = Register_getField(p->mpRcpData, HST_RCP_DATA_START_OFS);
  p->mpRcpDataEndDyn = Register_getField(p->mpRcpData, HST_RCP_DATA_END_DYN);
  p->mpRcpDataEndOfs = Register_getField(p->mpRcpData, HST_RCP_DATA_END_OFS);
  p->mpRcpDataModif0Cmd = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF0_CMD);
  p->mpRcpDataModif0Dyn = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF0_DYN);
  p->mpRcpDataModif0Ofs = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF0_OFS);
  p->mpRcpDataModif0Value = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF0_VALUE);
  p->mpRcpDataModif1Cmd = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF1_CMD);
  p->mpRcpDataModif1Dyn = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF1_DYN);
  p->mpRcpDataModif1Ofs = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF1_OFS);
  p->mpRcpDataModif1Value = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF1_VALUE);
  p->mpRcpDataModif2Cmd = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF2_CMD);
  p->mpRcpDataModif2Dyn = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF2_DYN);
  p->mpRcpDataModif2Ofs = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF2_OFS);
  p->mpRcpDataModif2Value = Register_getField(p->mpRcpData, HST_RCP_DATA_MODIF2_VALUE);

  return 0;
}

// RCP
void HstNthw_RcpSelect(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpAddr, val);
}

void HstNthw_RcpCnt(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCnt, val);
}

void HstNthw_RcpStripMode(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataStripMode, val);
}

void HstNthw_RcpStartDyn(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataStartDyn, val);
}

void HstNthw_RcpStartOfs(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataStartOfs, val);
}

void HstNthw_RcpEndDyn(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataEndDyn, val);
}

void HstNthw_RcpEndOfs(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataEndOfs, val);
}

void HstNthw_RcpModif0Cmd(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif0Cmd, val);
}

void HstNthw_RcpModif0Dyn(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif0Dyn, val);
}

void HstNthw_RcpModif0Ofs(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif0Ofs, val);
}

void HstNthw_RcpModif0Value(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif0Value, val);
}

void HstNthw_RcpModif1Cmd(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif1Cmd, val);
}

void HstNthw_RcpModif1Dyn(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif1Dyn, val);
}

void HstNthw_RcpModif1Ofs(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif1Ofs, val);
}

void HstNthw_RcpModif1Value(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif1Value, val);
}

void HstNthw_RcpModif2Cmd(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif2Cmd, val);
}

void HstNthw_RcpModif2Dyn(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif2Dyn, val);
}

void HstNthw_RcpModif2Ofs(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif2Ofs, val);
}

void HstNthw_RcpModif2Value(const struct HstNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataModif2Value, val);
}

void HstNthw_RcpFlush(const struct HstNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
}
