#ifndef __FLOW_NTHW_IOA_H__
#define __FLOW_NTHW_IOA_H__

#include "nthw_fpga_model.h"

#include <stdint.h> // uint32_t

struct IoaNthw {
  uint8_t mPhysicalAdapterNo;
  Fpga_t *mpFpga;

  Module_t *mIoa;

  Register_t *mpRcpCtrl;
  Field_t *mpRcpAddr;
  Field_t *mpRcpCnt;
  Register_t *mpRcpData;

  Field_t *mpRcpDataTunnelPop;
  Field_t *mpRcpDataVlanPop;
  Field_t *mpRcpDataVlanPush;
  Field_t *mpRcpDataVlanVid;
  Field_t *mpRcpDataVlanDei;
  Field_t *mpRcpDataVlanPcp;
  Field_t *mpRcpDataVlanTpidSel;
  Field_t *mpRcpDataQueueOverrideEn;
  Field_t *mpRcpDataQueueId;

  Register_t *mpSpecial;
  Field_t *mpSpecialVlanTpidCustTpid0;
  Field_t *mpSpecialVlanTpidCustTpid1;

  Register_t *mpRoaEppCtrl;
  Field_t *mpRoaEppAddr;
  Field_t *mpRoaEppCnt;
  Register_t *mpRoaEppData;
  Field_t *mpRoaEppDataPushTunnel;
  Field_t *mpRoaEppDataTxPort;
};

typedef struct IoaNthw IoaNthw_t;
// typedef struct IoaNthw IoaNthw;

//
//
//
struct IoaNthw *IoaNthw_new(void);
void IoaNthw_delete(struct IoaNthw *p);
int IoaNthw_init(struct IoaNthw *p, Fpga_t *pFpga, int nInstance);
//
int IoaNthw_setup(struct IoaNthw *p, int nIdx, int nIdxCnt);
void ioaNthw_setDebugMode(struct IoaNthw *p, unsigned nDebugMode);
//

#if 0
  uint32_t GetVersionMajor(void) {return (mIoa.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mIoa.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mIoa.setDebugMode(Register::ON_WRITE); else mIoa.setDebugMode(Register::NO_DEBUG);}
#endif

// RCP
void IoaNthw_RcpSelect(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpCnt(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpTunnelPop(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanPop(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanPush(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanVid(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanDei(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanPcp(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpVlanTpidSel(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpQueueOverrideEn(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpQueueId(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RcpFlush(const struct IoaNthw *p);

// Vlan Tpid Special
void IoaNthw_SpecialVlanTpidCustTpid0(const struct IoaNthw *p, uint32_t val);
void IoaNthw_SpecialVlanTpidCustTpid1(const struct IoaNthw *p, uint32_t val);
void IoaNthw_SpecialVlanTpidFlush(const struct IoaNthw *p);

// EPP module
void IoaNthw_RoaEppSelect(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppCnt(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppPushTunnel(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppTxPort(const struct IoaNthw *p, uint32_t val);
void IoaNthw_RoaEppFlush(const struct IoaNthw *p);


#endif //__FLOW_NTHW_IOA_H__
