//
// flow_nthw_km.c
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "flow_nthw_km.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

#define CHECK_AND_SET_VALUE(a, val) { if (a) { Field_setVal32(a, val); } }


void kmNthw_setDebugMode(struct KmNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mKm, nDebugMode);
}

struct KmNthw *KmNthw_new(void)
{
  struct KmNthw *p = malloc(sizeof(struct KmNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void KmNthw_delete(struct KmNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int KmNthw_init(struct KmNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod =   Fpga_queryModule(pFpga, MOD_KM, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Km %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mKm = pMod;

#if defined(DEBUG) && (0)
  Module_setDebugMode(p->mKm, 0xFF); // DEBUG:
#endif

  // RCP
  p->mpRcpCtrl = Module_getRegister(p->mKm, KM_RCP_CTRL);
  p->mpRcpAddr = Register_getField(p->mpRcpCtrl, KM_RCP_CTRL_ADR);
  p->mpRcpCnt = Register_getField(p->mpRcpCtrl, KM_RCP_CTRL_CNT);
  p->mpRcpData = Module_getRegister(p->mKm, KM_RCP_DATA);
  p->mpRcpDataQw0Dyn = Register_getField(p->mpRcpData, KM_RCP_DATA_QW0_DYN);
  p->mpRcpDataQw0Ofs = Register_getField(p->mpRcpData, KM_RCP_DATA_QW0_OFS);
  p->mpRcpDataQw0SelA = Register_getField(p->mpRcpData, KM_RCP_DATA_QW0_SEL_A);
  p->mpRcpDataQw0SelB = Register_getField(p->mpRcpData, KM_RCP_DATA_QW0_SEL_B);
  p->mpRcpDataQw4Dyn = Register_getField(p->mpRcpData, KM_RCP_DATA_QW4_DYN);
  p->mpRcpDataQw4Ofs = Register_getField(p->mpRcpData, KM_RCP_DATA_QW4_OFS);
  p->mpRcpDataQw4SelA = Register_getField(p->mpRcpData, KM_RCP_DATA_QW4_SEL_A);
  p->mpRcpDataQw4SelB = Register_getField(p->mpRcpData, KM_RCP_DATA_QW4_SEL_B);

  p->mpRcpDataSw8Dyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW8_DYN);
  p->mpRcpDataDw8Dyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW8_DYN);

  p->mpRcpDataSwxOvsSb = Register_queryField(p->mpRcpData, KM_RCP_DATA_SWX_OVS_SB);
  p->mpRcpDataSwxCch = Register_queryField(p->mpRcpData, KM_RCP_DATA_SWX_CCH);
  p->mpRcpDataSwxSelA = Register_getField(p->mpRcpData, KM_RCP_DATA_SWX_SEL_A);
  p->mpRcpDataSwxSelB = Register_getField(p->mpRcpData, KM_RCP_DATA_SWX_SEL_B);
  p->mpRcpDataMaskA = Register_getField(p->mpRcpData, KM_RCP_DATA_MASK_A);
  p->mpRcpDataMaskB = Register_getField(p->mpRcpData, KM_RCP_DATA_MASK_B);
  p->mpRcpDataDual = Register_getField(p->mpRcpData, KM_RCP_DATA_DUAL);
  p->mpRcpDataPaired = Register_getField(p->mpRcpData, KM_RCP_DATA_PAIRED);
  p->mpRcpDataElA = Register_getField(p->mpRcpData, KM_RCP_DATA_EL_A);
  p->mpRcpDataElB = Register_getField(p->mpRcpData, KM_RCP_DATA_EL_B);
  p->mpRcpDataInfoA = Register_getField(p->mpRcpData, KM_RCP_DATA_INFO_A);
  p->mpRcpDataInfoB = Register_getField(p->mpRcpData, KM_RCP_DATA_INFO_B);
  p->mpRcpDataFtmA = Register_getField(p->mpRcpData, KM_RCP_DATA_FTM_A);
  p->mpRcpDataFtmB = Register_getField(p->mpRcpData, KM_RCP_DATA_FTM_B);
  p->mpRcpDataBankA = Register_getField(p->mpRcpData, KM_RCP_DATA_BANK_A);
  p->mpRcpDataBankB = Register_getField(p->mpRcpData, KM_RCP_DATA_BANK_B);
  p->mpRcpDataKlA = Register_getField(p->mpRcpData, KM_RCP_DATA_KL_A);
  p->mpRcpDataKlB = Register_getField(p->mpRcpData, KM_RCP_DATA_KL_B);
  p->mpRcpDataFlowSet = Register_queryField(p->mpRcpData, KM_RCP_DATA_FLOW_SET);
  p->mpRcpDataKeywayA = Register_queryField(p->mpRcpData, KM_RCP_DATA_KEYWAY_A);
  p->mpRcpDataKeywayB = Register_queryField(p->mpRcpData, KM_RCP_DATA_KEYWAY_B);
  p->mpRcpDataSynergyMode = Register_getField(p->mpRcpData, KM_RCP_DATA_SYNERGY_MODE);

  // CAM
  p->mpCamCtrl = Module_getRegister(p->mKm, KM_CAM_CTRL);
  p->mpCamAddr = Register_getField(p->mpCamCtrl, KM_CAM_CTRL_ADR);
  p->mpCamCnt = Register_getField(p->mpCamCtrl, KM_CAM_CTRL_CNT);
  p->mpCamData = Module_getRegister(p->mKm, KM_CAM_DATA);
  p->mpCamDataW0 = Register_getField(p->mpCamData, KM_CAM_DATA_W0);
  p->mpCamDataW1 = Register_getField(p->mpCamData, KM_CAM_DATA_W1);
  p->mpCamDataW2 = Register_getField(p->mpCamData, KM_CAM_DATA_W2);
  p->mpCamDataW3 = Register_getField(p->mpCamData, KM_CAM_DATA_W3);
  p->mpCamDataW4 = Register_getField(p->mpCamData, KM_CAM_DATA_W4);
  p->mpCamDataW5 = Register_getField(p->mpCamData, KM_CAM_DATA_W5);
  p->mpCamDataFt0 = Register_getField(p->mpCamData, KM_CAM_DATA_FT0);
  p->mpCamDataFt1 = Register_getField(p->mpCamData, KM_CAM_DATA_FT1);
  p->mpCamDataFt2 = Register_getField(p->mpCamData, KM_CAM_DATA_FT2);
  p->mpCamDataFt3 = Register_getField(p->mpCamData, KM_CAM_DATA_FT3);
  p->mpCamDataFt4 = Register_getField(p->mpCamData, KM_CAM_DATA_FT4);
  p->mpCamDataFt5 = Register_getField(p->mpCamData, KM_CAM_DATA_FT5);
  // TCAM
  p->mpTcamCtrl = Module_getRegister(p->mKm, KM_TCAM_CTRL);
  p->mpTcamAddr = Register_getField(p->mpTcamCtrl, KM_TCAM_CTRL_ADR);
  p->mpTcamCnt = Register_getField(p->mpTcamCtrl, KM_TCAM_CTRL_CNT);
  p->mpTcamData = Module_getRegister(p->mKm, KM_TCAM_DATA);
  p->mpTcamDataT = Register_getField(p->mpTcamData, KM_TCAM_DATA_T);
  // TCI
  p->mpTciCtrl = Module_getRegister(p->mKm, KM_TCI_CTRL);
  p->mpTciAddr = Register_getField(p->mpTciCtrl, KM_TCI_CTRL_ADR);
  p->mpTciCnt = Register_getField(p->mpTciCtrl, KM_TCI_CTRL_CNT);
  p->mpTciData= Module_getRegister(p->mKm, KM_TCI_DATA);
  p->mpTciDataColor = Register_getField(p->mpTciData, KM_TCI_DATA_COLOR);
  p->mpTciDataFt = Register_getField(p->mpTciData, KM_TCI_DATA_FT);
  // TCQ
  p->mpTcqCtrl= Module_getRegister(p->mKm, KM_TCQ_CTRL);
  p->mpTcqAddr = Register_getField(p->mpTcqCtrl, KM_TCQ_CTRL_ADR);
  p->mpTcqCnt = Register_getField(p->mpTcqCtrl, KM_TCQ_CTRL_CNT);
  p->mpTcqData= Module_getRegister(p->mKm, KM_TCQ_DATA);
  p->mpTcqDataBankMask = Register_queryField(p->mpTcqData, KM_TCQ_DATA_BANK_MASK);
  p->mpTcqDataQual = Register_getField(p->mpTcqData, KM_TCQ_DATA_QUAL);

  p->mpRcpDataDw0BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW0_B_DYN);
  p->mpRcpDataDw0BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW0_B_OFS);
  p->mpRcpDataDw2BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW2_B_DYN);
  p->mpRcpDataDw2BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW2_B_OFS);
  p->mpRcpDataSw4BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW4_B_DYN);
  p->mpRcpDataSw4BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW4_B_OFS);
  p->mpRcpDataSw5BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW5_B_DYN);
  p->mpRcpDataSw5BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW5_B_OFS);
  if (!p->mpRcpDataDw0BDyn) {
    // old field defines
    p->mpRcpDataDw0BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_QW0_B_DYN);
    p->mpRcpDataDw0BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_QW0_B_OFS);
    p->mpRcpDataDw2BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_QW4_B_DYN);
    p->mpRcpDataDw2BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_QW4_B_OFS);
    p->mpRcpDataSw4BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW8_B_DYN);
    p->mpRcpDataSw4BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW8_B_OFS);
    p->mpRcpDataSw5BDyn = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW9_B_DYN);
    p->mpRcpDataSw5BOfs = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW9_B_OFS);
  }

  // v0.6+
  if (p->mpRcpDataDw8Dyn) {
    p->mpRcpDataDw8Ofs   = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW8_OFS  );
    p->mpRcpDataDw8SelA  = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW8_SEL_A);
    p->mpRcpDataDw8SelB  = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW8_SEL_B);
    p->mpRcpDataDw10Dyn  = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW10_DYN  );
    p->mpRcpDataDw10Ofs  = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW10_OFS  );
    p->mpRcpDataDw10SelA = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW10_SEL_A);
    p->mpRcpDataDw10SelB = Register_queryField(p->mpRcpData, KM_RCP_DATA_DW10_SEL_B);
  } else
  if (p->mpRcpDataSw8Dyn) {
    p->mpRcpDataSw8Ofs  = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW8_OFS  );
    p->mpRcpDataSw8SelA = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW8_SEL_A);
    p->mpRcpDataSw8SelB = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW8_SEL_B);
    p->mpRcpDataSw9Dyn  = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW9_DYN  );
    p->mpRcpDataSw9Ofs  = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW9_OFS  );
    p->mpRcpDataSw9SelA = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW9_SEL_A);
    p->mpRcpDataSw9SelB = Register_queryField(p->mpRcpData, KM_RCP_DATA_SW9_SEL_B);
  }


  return 0;
}


// RCP
void KmNthw_RcpSelect(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpAddr, val); };

void KmNthw_RcpCnt(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpRcpCnt, val); };

void KmNthw_RcpQw0Dyn(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataQw0Dyn, val); };

void KmNthw_RcpQw0Ofs(const struct KmNthw *p, int32_t val) { Field_setVal32(p->mpRcpDataQw0Ofs, val); };

void KmNthw_RcpQw0SelA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataQw0SelA, val); };

void KmNthw_RcpQw0SelB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataQw0SelB, val); };

void KmNthw_RcpQw4Dyn(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataQw4Dyn, val); };

void KmNthw_RcpQw4Ofs(const struct KmNthw *p, int32_t val) { Field_setVal32(p->mpRcpDataQw4Ofs, val); };

void KmNthw_RcpQw4SelA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataQw4SelA, val); };

void KmNthw_RcpQw4SelB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataQw4SelB, val); };

void KmNthw_RcpDw8Dyn(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw8Dyn, val);

void KmNthw_RcpSw8Dyn(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw8Dyn, val);

void KmNthw_RcpSw8Ofs(const struct KmNthw *p, int32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw8Ofs, val);

void KmNthw_RcpSw8SelA(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw8SelA, val);

void KmNthw_RcpSw8SelB(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw8SelB, val);

void KmNthw_RcpSw9Dyn(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw9Dyn, val);

void KmNthw_RcpSw9Ofs(const struct KmNthw *p, int32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw9Ofs, val);

void KmNthw_RcpSw9SelA(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw9SelA, val);

void KmNthw_RcpSw9SelB(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSw9SelB, val);

void KmNthw_RcpSwxOvsSb(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSwxOvsSb, val);

void KmNthw_RcpSwxCch(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataSwxCch, val);

void KmNthw_RcpDw8Ofs(const struct KmNthw *p, int32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw8Ofs, val);

void KmNthw_RcpDw8SelA(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw8SelA, val);

void KmNthw_RcpDw8SelB(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw8SelB, val);

void KmNthw_RcpDw10Dyn(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw10Dyn, val);

void KmNthw_RcpDw10Ofs(const struct KmNthw *p, int32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw10Ofs, val);

void KmNthw_RcpDw10SelA(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw10SelA, val);

void KmNthw_RcpDw10SelB(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataDw10SelB, val);

void KmNthw_RcpSwxSelA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataSwxSelA, val); };

void KmNthw_RcpSwxSelB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataSwxSelB, val); };

void KmNthw_RcpMaskA(const struct KmNthw *p, const uint32_t *val) { Field_setVal(p->mpRcpDataMaskA, val, p->mpRcpDataMaskA->mnWords); };

void KmNthw_RcpMaskB(const struct KmNthw *p, const uint32_t *val) { Field_setVal(p->mpRcpDataMaskB, val, p->mpRcpDataMaskB->mnWords); };

void KmNthw_RcpMaskDA(const struct KmNthw *p, const uint32_t *val) { Field_setVal(p->mpRcpDataMaskA, val, p->mpRcpDataMaskA->mnWords); }; // for DW8/DW10 from v6+

void KmNthw_RcpDual(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataDual, val); };

void KmNthw_RcpPaired(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataPaired, val); };

void KmNthw_RcpElA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataElA, val); };

void KmNthw_RcpElB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataElB, val); };

void KmNthw_RcpInfoA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataInfoA, val); };

void KmNthw_RcpInfoB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataInfoB, val); };

void KmNthw_RcpFtmA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataFtmA, val); };

void KmNthw_RcpFtmB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataFtmB, val); };

void KmNthw_RcpBankA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataBankA, val); };

void KmNthw_RcpBankB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataBankB, val); };

void KmNthw_RcpKlA(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataKlA, val); };

void KmNthw_RcpKlB(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataKlB, val); };

void KmNthw_RcpFlowSet(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataFlowSet, val);

void KmNthw_RcpKeywayA(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataKeywayA, val);

void KmNthw_RcpKeywayB(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpRcpDataKeywayB, val);

void KmNthw_RcpSynergyMode(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataSynergyMode, val); };

void KmNthw_RcpDw0BDyn(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataDw0BDyn, val); };

void KmNthw_RcpDw0BOfs(const struct KmNthw *p, int32_t val) { Field_setVal32(p->mpRcpDataDw0BOfs, val); };

void KmNthw_RcpDw2BDyn(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataDw2BDyn, val); };

void KmNthw_RcpDw2BOfs(const struct KmNthw *p, int32_t val) { Field_setVal32(p->mpRcpDataDw2BOfs, val); };

void KmNthw_RcpSw4BDyn(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataSw4BDyn, val); };

void KmNthw_RcpSw4BOfs(const struct KmNthw *p, int32_t val) { Field_setVal32(p->mpRcpDataSw4BOfs, val); };

void KmNthw_RcpSw5BDyn(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpRcpDataSw5BDyn, val); };

void KmNthw_RcpSw5BOfs(const struct KmNthw *p, int32_t val) { Field_setVal32(p->mpRcpDataSw5BOfs, val); };

void KmNthw_RcpFlush(const struct KmNthw *p)          { Register_flush(p->mpRcpCtrl, 1); Register_flush(p->mpRcpData, 1); };

// CAM
void KmNthw_CamSelect(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpCamAddr, val); };

void KmNthw_CamCnt(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamCnt, val); };

void KmNthw_CamW0(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpCamDataW0, val); };

void KmNthw_CamW1(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpCamDataW1, val); };

void KmNthw_CamW2(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpCamDataW2, val); };

void KmNthw_CamW3(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpCamDataW3, val); };

void KmNthw_CamW4(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpCamDataW4, val); };

void KmNthw_CamW5(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpCamDataW5, val); };

void KmNthw_CamFt0(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamDataFt0, val); };

void KmNthw_CamFt1(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamDataFt1, val); };

void KmNthw_CamFt2(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamDataFt2, val); };

void KmNthw_CamFt3(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamDataFt3, val); };

void KmNthw_CamFt4(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamDataFt4, val); };

void KmNthw_CamFt5(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpCamDataFt5, val); };

void KmNthw_CamFlush(const struct KmNthw *p)          { Register_flush(p->mpCamCtrl, 1); Register_flush(p->mpCamData, 1); };

// TCAM
void KmNthw_TcamSelect(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpTcamAddr, val); };

void KmNthw_TcamCnt(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpTcamCnt, val); };

void KmNthw_TcamT(const struct KmNthw *p, uint32_t *val)     { Field_setVal(p->mpTcamDataT, val, 3); };

void KmNthw_TcamFlush(const struct KmNthw *p)          { Register_flush(p->mpTcamCtrl, 1); Register_flush(p->mpTcamData, 1); };

// TCI
void KmNthw_TciSelect(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpTciAddr, val); };

void KmNthw_TciCnt(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpTciCnt, val); };

void KmNthw_TciColor(const struct KmNthw *p, uint32_t val)  { Field_setVal32(p->mpTciDataColor, val); };

void KmNthw_TciFt(const struct KmNthw *p, uint32_t val)     { Field_setVal32(p->mpTciDataFt, val); };

void KmNthw_TciFlush(const struct KmNthw *p)          { Register_flush(p->mpTciCtrl, 1); Register_flush(p->mpTciData, 1); };

// TCQ
void KmNthw_TcqSelect(const struct KmNthw *p, uint32_t val) { Field_setVal32(p->mpTcqAddr, val); };

void KmNthw_TcqCnt(const struct KmNthw *p, uint32_t val)    { Field_setVal32(p->mpTcqCnt, val); };

void KmNthw_TcqBankMask(const struct KmNthw *p, uint32_t val) CHECK_AND_SET_VALUE(p->mpTcqDataBankMask, val);

void KmNthw_TcqQual(const struct KmNthw *p, uint32_t val)   { Field_setVal32(p->mpTcqDataQual, val); };

void KmNthw_TcqQual72(const struct KmNthw *p, uint32_t *val){ Field_setVal(p->mpTcqDataQual, val, 3); }; // to use in v4

void KmNthw_TcqFlush(const struct KmNthw *p)          { Register_flush(p->mpTcqCtrl, 1); Register_flush(p->mpTcqData, 1); };

//
// EOF
//
