#ifndef __FLOW_NTHW_QSL_H__
#define __FLOW_NTHW_QSL_H__

#include <stdint.h> // uint32_t
#include "nthw_fpga_model.h"

struct QslNthw {
  uint8_t mPhysicalAdapterNo;
  Fpga_t *mpFpga;

  Module_t *mQsl;

  Register_t *mpRcpCtrl;
  Field_t *mpRcpAddr;
  Field_t *mpRcpCnt;
  Register_t *mpRcpData;
  Field_t *mpRcpDataDiscard;
  Field_t *mpRcpDataDrop;
  Field_t *mpRcpDataTblLo;
  Field_t *mpRcpDataTblHi;
  Field_t *mpRcpDataTblIdx;
  Field_t *mpRcpDataTblMsk;
  Field_t *mpRcpDataCao;
  Field_t *mpRcpDataLr;
  Field_t *mpRcpDataTsa;
  Field_t *mpRcpDataVli;

  Register_t *mpLtxCtrl;
  Field_t *mpLtxAddr;
  Field_t *mpLtxCnt;
  Register_t *mpLtxData;
  Field_t *mpLtxDataLr;
  Field_t *mpLtxDataTxPort;
  Field_t *mpLtxDataTsa;

  Register_t *mpQstCtrl;
  Field_t *mpQstAddr;
  Field_t *mpQstCnt;
  Register_t *mpQstData;
  Field_t *mpQstDataQueue;
  Field_t *mpQstDataEn;
  Field_t *mpQstDataTxPort;
  Field_t *mpQstDataLre;
  Field_t *mpQstDataTci;
  Field_t *mpQstDataVen;

  Register_t *mpQenCtrl;
  Field_t *mpQenAddr;
  Field_t *mpQenCnt;
  Register_t *mpQenData;
  Field_t *mpQenDataEn;

  Register_t *mpUnmqCtrl;
  Field_t *mpUnmqAddr;
  Field_t *mpUnmqCnt;
  Register_t *mpUnmqData;
  Field_t *mpUnmqDataDestQueue;
  Field_t *mpUnmqDataEn;
};

typedef struct QslNthw QslNthw_t;
//typedef struct QslNthw QslNthw;

//
//
//
struct QslNthw *QslNthw_new(void);
void QslNthw_delete(struct QslNthw *p);
int QslNthw_init(struct QslNthw *p, Fpga_t *pFpga, int nInstance);
//
int QslNthw_setup(struct QslNthw *p, int nIdx, int nIdxCnt);
void qslNthw_setDebugMode(struct QslNthw *p, unsigned nDebugMode);
//

#if 0
uint32_t GetVersionMajor(void) {return (mQsl.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mQsl.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mQsl.setDebugMode(Register::ON_WRITE); else mQsl.setDebugMode(Register::NO_DEBUG);}
#endif

// RCP
void QslNthw_RcpSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpDiscard(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpDrop(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblLo(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblHi(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblIdx(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTblMsk(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpCao(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpLr(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpTsa(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpVli(const struct QslNthw *p, uint32_t val);
void QslNthw_RcpFlush(const struct QslNthw *p);

// LTX
void QslNthw_LtxSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxLr(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxTxPort(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxTsa(const struct QslNthw *p, uint32_t val);
void QslNthw_LtxFlush(const struct QslNthw *p);

// QST
void QslNthw_QstSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_QstCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_QstQueue(const struct QslNthw *p, uint32_t val);
void QslNthw_QstEn(const struct QslNthw *p, uint32_t val);
void QslNthw_QstTxPort(const struct QslNthw *p, uint32_t val);
void QslNthw_QstLre(const struct QslNthw *p, uint32_t val);
void QslNthw_QstTci(const struct QslNthw *p, uint32_t val);
void QslNthw_QstVen(const struct QslNthw *p, uint32_t val);
void QslNthw_QstFlush(const struct QslNthw *p);

// QEN
void QslNthw_QenSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_QenCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_QenEn(const struct QslNthw *p, uint32_t val);
void QslNthw_QenFlush(const struct QslNthw *p);

// UNMQ
void QslNthw_UnmqSelect(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqCnt(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqDestQueue(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqEn(const struct QslNthw *p, uint32_t val);
void QslNthw_UnmqFlush(const struct QslNthw *p);

#endif //__FLOW_NTHW_QSL_H__
