/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "flow_nthw_rmc.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

void rmcNthw_setDebugMode(struct RmcNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mRmc, nDebugMode);
}

struct RmcNthw *RmcNthw_new(void)
{
  struct RmcNthw *p = malloc(sizeof(struct RmcNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void RmcNthw_delete(struct RmcNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int RmcNthw_init(struct RmcNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_RMC, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: RMC %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mRmc = pMod;

  // CTRL
  p->mpCtrl = Module_getRegister(p->mRmc, RMC_CTRL);
  p->mpCtrlBlockStatt = Register_getField(p->mpCtrl, RMC_CTRL_BLOCK_STATT);
  p->mpCtrlBlockKeepA = Register_getField(p->mpCtrl, RMC_CTRL_BLOCK_KEEPA);
  p->mpCtrlBlockRppSlice = Register_queryField(p->mpCtrl, RMC_CTRL_BLOCK_RPP_SLICE);
  p->mpCtrlBlockMacPort = Register_getField(p->mpCtrl, RMC_CTRL_BLOCK_MAC_PORT);
  p->mpCtrlLagPhyOddEven= Register_getField(p->mpCtrl, RMC_CTRL_LAG_PHY_ODD_EVEN);
  return 0;
}

int RmcNthw_setup(struct RmcNthw *p, int nIdx, int nIdxCnt)
{
  (void)p;
  (void)nIdx;
  (void)nIdxCnt;

  return 0;
}

  // CTRL
void RmcNthw_CtrlBlockStatt(const struct RmcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtrlBlockStatt, val);
};

void RmcNthw_CtrlBlockKeepA(const struct RmcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtrlBlockKeepA, val);
};

void RmcNthw_CtrlBlockRppSlice(const struct RmcNthw *p, uint32_t val)
{
  if (p->mpCtrlBlockRppSlice) {
    Field_setVal32(p->mpCtrlBlockRppSlice, val);
  }
};

void RmcNthw_CtrlBlockMacPort(const struct RmcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtrlBlockMacPort, val);
};

void RmcNthw_CtrlLagPhyOddEven(const struct RmcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpCtrlLagPhyOddEven, val);
};

void RmcNthw_CtrlFlush(const struct RmcNthw *p)
{
  Register_flush(p->mpCtrl, 1);
};
