#ifndef __FLOW_NTHW_ROA_H__
#define __FLOW_NTHW_ROA_H__

#include <stdint.h> // uint32_t
#include "nthw_fpga_model.h"

struct RoaNthw;

typedef struct RoaNthw RoaNthw_t;
// typedef struct RoaNthw RoaNthw;

//
//
//
struct RoaNthw *RoaNthw_new(void);
void RoaNthw_delete(struct RoaNthw *p);
int RoaNthw_init(struct RoaNthw *p, Fpga_t *pFpga, int nInstance);
//
int RoaNthw_setup(struct RoaNthw *p, int nIdx, int nIdxCnt);
void roaNthw_setDebugMode(struct RoaNthw *p, unsigned nDebugMode);
//

#if 0
uint32_t GetVersionMajor(void) {return (mRoa.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mRoa.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mRoa.setDebugMode(Register_t::ON_WRITE); else mRoa.setDebugMode(Register_t::NO_DEBUG);}
#endif

// TUN HDR
void RoaNthw_TunHdrSelect(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunHdrCnt(const struct RoaNthw *p, uint32_t val);

void RoaNthw_TunHdrTunnelHdr(const struct RoaNthw *p, uint32_t *val);
void RoaNthw_TunHdrFlush(const struct RoaNthw *p);

// TUN CFG
void RoaNthw_TunCfgSelect(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgCnt(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTunLen(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTunType(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTunVlan(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIpType(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIpcsUpd(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIpcsPrecalc(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIptlUpd(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgIptlPrecalc(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgVxlanUdpLenUpd(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgTxLagIx(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgRecirculate(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgPushTunnel(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgRecircPort(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgRecircBypass(const struct RoaNthw *p, uint32_t val);
void RoaNthw_TunCfgFlush(const struct RoaNthw *p);

// ROA CONFIG
void RoaNthw_ConfigFwdRecirculate(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdNormalPcks(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdTxPort0(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdTxPort1(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdCellBuilderPcks(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFwdNonNormalPcks(const struct RoaNthw *p, uint32_t val);
void RoaNthw_ConfigFlush(const struct RoaNthw *p);

// LAG
void RoaNthw_LagCfgSelect(const struct RoaNthw *p, uint32_t val);
void RoaNthw_LagCfgCnt(const struct RoaNthw *p, uint32_t val);
void RoaNthw_LagCfgTxPhyPort(const struct RoaNthw *p, uint32_t val);
void RoaNthw_LagCfgFlush(const struct RoaNthw *p);

struct RoaNthw {
  uint8_t mPhysicalAdapterNo;
  Fpga_t *mpFpga;

  Module_t *mRoa;

  Register_t *mpTunHdrCtrl;
  Field_t *mpTunHdrAddr;
  Field_t *mpTunHdrCnt;
  Register_t *mpTunHdrData;
  Field_t *mpTunHdrDataTunnelHdr;

  Register_t *mpTunCfgCtrl;
  Field_t *mpTunCfgAddr;
  Field_t *mpTunCfgCnt;
  Register_t *mpTunCfgData;
  Field_t *mpTunCfgDataTunLen;
  Field_t *mpTunCfgDataTunType;
  Field_t *mpTunCfgDataTunVlan;
  Field_t *mpTunCfgDataIpType;
  Field_t *mpTunCfgDataIpcsUpd;
  Field_t *mpTunCfgDataIpcsPrecalc;
  Field_t *mpTunCfgDataIptlUpd;
  Field_t *mpTunCfgDataIptlPrecalc;
  Field_t *mpTunCfgDataVxlanUdpLenUpd;
  Field_t *mpTunCfgDataTxLagIx;
  Field_t *mpTunCfgDataRecirculate;
  Field_t *mpTunCfgDataPushTunnel;
  Field_t *mpTunCfgDataRecircPort;
  Field_t *mpTunCfgDataRecircBypass;

  Register_t *mpConfig;
  Field_t *mpConfigFwdRecirculate;
  Field_t *mpConfigFwdNormalPcks;
  Field_t *mpConfigFwdTxPort0;
  Field_t *mpConfigFwdTxPort1;
  Field_t *mpConfigFwdCellBuilderPcks;
  Field_t *mpConfigFwdNonNormalPcks;

  Register_t *mpLagCfgCtrl;
  Field_t *mpLagCfgAddr;
  Field_t *mpLagCfgCnt;
  Register_t *mpLagCfgData;
  Field_t *mpLagCfgDataTxPhyPort;
};

#endif //__FLOW_NTHW_ROA_H__
