/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
#include "nthw_drv.h"
#include "nthw_register.h"

#include "flow_nthw_rpp_lr.h"

#include <stdlib.h>
#include <string.h>

void rppLrNthw_setDebugMode(struct RppLrNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mRppLr, nDebugMode);
}

struct RppLrNthw *RppLrNthw_new(void)
{
  struct RppLrNthw *p = malloc(sizeof(struct RppLrNthw));
  if (p) {
    (void)memset(p, 0, sizeof(*p));
  }
  return p;
}

void RppLrNthw_delete(struct RppLrNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int RppLrNthw_init(struct RppLrNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_RPP_LR, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: RppLr %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mRppLr = Fpga_queryModule(pFpga, MOD_RPP_LR, nInstance);

  p->mpRcpCtrl = Module_getRegister(p->mRppLr, RPP_LR_RCP_CTRL);
  p->mpRcpAddr = Register_getField(p->mpRcpCtrl, RPP_LR_RCP_CTRL_ADR);
  p->mpRcpCnt = Register_getField(p->mpRcpCtrl, RPP_LR_RCP_CTRL_CNT);
  p->mpRcpData = Module_getRegister(p->mRppLr, RPP_LR_RCP_DATA);
  p->mpRcpDataExp = Register_getField(p->mpRcpData, RPP_LR_RCP_DATA_EXP);

  return 0;
}

void RppLrNthw_RcpSelect(const struct RppLrNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpAddr, val);
}

void RppLrNthw_RcpCnt(const struct RppLrNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCnt, val);
}

void RppLrNthw_RcpExp(const struct RppLrNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataExp, val);
}

void RppLrNthw_RcpFlush(const struct RppLrNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
}
