/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "flow_nthw_slc.h"

#include <stdlib.h> // malloc
#include <string.h> // memset

void slcNthw_setDebugMode(struct SlcNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mSlc, nDebugMode);
}

struct SlcNthw *SlcNthw_new(void)
{
  struct SlcNthw *p = malloc(sizeof(struct SlcNthw));
  if (p)
    (void)memset(p, 0, sizeof(*p));

  return p;
}

void SlcNthw_delete(struct SlcNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int SlcNthw_init(struct SlcNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_SLC, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Slc %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mSlc = Fpga_queryModule(pFpga, MOD_SLC, nInstance);


  // RCP
  p->mpRcpCtrl = Module_getRegister(p->mSlc, SLC_RCP_CTRL);
  p->mpRcpAddr = Register_getField(p->mpRcpCtrl, SLC_RCP_CTRL_ADR);
  p->mpRcpCnt = Register_getField(p->mpRcpCtrl, SLC_RCP_CTRL_CNT);
  p->mpRcpData = Module_getRegister(p->mSlc, SLC_RCP_DATA);
  p->mpRcpDataTailSlcEn = Register_getField(p->mpRcpData, SLC_RCP_DATA_TAIL_SLC_EN);
  p->mpRcpDataTailDyn = Register_getField(p->mpRcpData, SLC_RCP_DATA_TAIL_DYN);
  p->mpRcpDataTailOfs = Register_getField(p->mpRcpData, SLC_RCP_DATA_TAIL_OFS);
  p->mpRcpDataPcap = Register_getField(p->mpRcpData, SLC_RCP_DATA_PCAP);

  return 0;
}

// RCP
void SlcNthw_RcpSelect(const struct SlcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpAddr, val);
};

void SlcNthw_RcpCnt(const struct SlcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCnt, val);
};

void SlcNthw_RcpTailSlcEn(const struct SlcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTailSlcEn, val);
};

void SlcNthw_RcpTailDyn(const struct SlcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataTailDyn, val);
};

void SlcNthw_RcpTailOfs(const struct SlcNthw *p, int32_t val)
{
  Field_setVal32(p->mpRcpDataTailOfs, val);
};

void SlcNthw_RcpPcap(const struct SlcNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataPcap, val);
};

void SlcNthw_RcpFlush(const struct SlcNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
};
