/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
#include "nthw_drv.h"
#include "nthw_register.h"

#include "flow_nthw_tx_cpy.h"

#include <stdlib.h>
#include <string.h>

void txCpyNthw_setDebugMode(struct TxCpyNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mTxCpy, nDebugMode);
}

struct TxCpyNthw *TxCpyNthw_new(void)
{
  struct TxCpyNthw *p = malloc(sizeof(struct TxCpyNthw));
  if (p) {
    (void)memset(p, 0, sizeof(*p));
  }
  return p;
}

void TxCpyNthw_delete(struct TxCpyNthw *p)
{
  if (p) {
    free(p->mWriters);
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int TxCpyNthw_init(struct TxCpyNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_TX_CPY, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: TxCpy %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mTxCpy = Fpga_queryModule(pFpga, MOD_TX_CPY, nInstance);

  const int writersCnt = Fpga_getProductParam(p->mpFpga, NT_TX_CPY_WRITERS, 0);
  if (writersCnt < 1) {
    return -1;
  }

  p->mWritersCnt = (unsigned)writersCnt;
  p->mWriters = calloc(p->mWritersCnt, sizeof(struct TxCpyWriters_s));
  if (p->mWriters == NULL) {
    return -1;
  }

  const int variant = Fpga_getProductParam(p->mpFpga, NT_TX_CPY_VARIANT, 0);

  switch(p->mWritersCnt) {
  default:
  case 6:
    p->mWriters[5].mpWriterCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER5_CTRL);
    p->mWriters[5].mpWriterCtrlAddr = Register_getField(p->mWriters[5].mpWriterCtrl, CPY_WRITER5_CTRL_ADR);
    p->mWriters[5].mpWriterCtrlCnt = Register_getField(p->mWriters[5].mpWriterCtrl, CPY_WRITER5_CTRL_CNT);
    p->mWriters[5].mpWriterData = Module_getRegister(p->mTxCpy, CPY_WRITER5_DATA);
    p->mWriters[5].mpWriterDataReaderSelect = Register_getField(p->mWriters[5].mpWriterData, CPY_WRITER5_DATA_READER_SELECT);
    p->mWriters[5].mpWriterDataDyn = Register_getField(p->mWriters[5].mpWriterData, CPY_WRITER5_DATA_DYN);
    p->mWriters[5].mpWriterDataOfs = Register_getField(p->mWriters[5].mpWriterData, CPY_WRITER5_DATA_OFS);
    p->mWriters[5].mpWriterDataLen = Register_getField(p->mWriters[5].mpWriterData, CPY_WRITER5_DATA_LEN);
    if (variant != 0) {
      p->mWriters[5].mpWriterDataMaskPointer = Register_getField(p->mWriters[5].mpWriterData, CPY_WRITER5_DATA_MASK_POINTER);
      p->mWriters[5].mpWriterMaskCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER5_MASK_CTRL);
      p->mWriters[5].mpWriterMaskCtrlAddr = Register_getField(p->mWriters[5].mpWriterMaskCtrl, CPY_WRITER5_MASK_CTRL_ADR);
      p->mWriters[5].mpWriterMaskCtrlCnt = Register_getField(p->mWriters[5].mpWriterMaskCtrl, CPY_WRITER5_MASK_CTRL_CNT);
      p->mWriters[5].mpWriterMaskData = Module_getRegister(p->mTxCpy, CPY_WRITER5_MASK_DATA);
      p->mWriters[5].mpWriterMaskDataByteMask = Register_getField(p->mWriters[5].mpWriterMaskData, CPY_WRITER5_MASK_DATA_BYTE_MASK);
    }
    // Fallthrough
  case 5:
    p->mWriters[4].mpWriterCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER4_CTRL);
    p->mWriters[4].mpWriterCtrlAddr = Register_getField(p->mWriters[4].mpWriterCtrl, CPY_WRITER4_CTRL_ADR);
    p->mWriters[4].mpWriterCtrlCnt = Register_getField(p->mWriters[4].mpWriterCtrl, CPY_WRITER4_CTRL_CNT);
    p->mWriters[4].mpWriterData = Module_getRegister(p->mTxCpy, CPY_WRITER4_DATA);
    p->mWriters[4].mpWriterDataReaderSelect = Register_getField(p->mWriters[4].mpWriterData, CPY_WRITER4_DATA_READER_SELECT);
    p->mWriters[4].mpWriterDataDyn = Register_getField(p->mWriters[4].mpWriterData, CPY_WRITER4_DATA_DYN);
    p->mWriters[4].mpWriterDataOfs = Register_getField(p->mWriters[4].mpWriterData, CPY_WRITER4_DATA_OFS);
    p->mWriters[4].mpWriterDataLen = Register_getField(p->mWriters[4].mpWriterData, CPY_WRITER4_DATA_LEN);
    if (variant != 0) {
      p->mWriters[4].mpWriterDataMaskPointer = Register_getField(p->mWriters[4].mpWriterData, CPY_WRITER4_DATA_MASK_POINTER);
      p->mWriters[4].mpWriterMaskCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER4_MASK_CTRL);
      p->mWriters[4].mpWriterMaskCtrlAddr = Register_getField(p->mWriters[4].mpWriterMaskCtrl, CPY_WRITER4_MASK_CTRL_ADR);
      p->mWriters[4].mpWriterMaskCtrlCnt = Register_getField(p->mWriters[4].mpWriterMaskCtrl, CPY_WRITER4_MASK_CTRL_CNT);
      p->mWriters[4].mpWriterMaskData = Module_getRegister(p->mTxCpy, CPY_WRITER4_MASK_DATA);
      p->mWriters[4].mpWriterMaskDataByteMask = Register_getField(p->mWriters[4].mpWriterMaskData, CPY_WRITER4_MASK_DATA_BYTE_MASK);
    }
    // Fallthrough
  case 4:
    p->mWriters[3].mpWriterCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER3_CTRL);
    p->mWriters[3].mpWriterCtrlAddr = Register_getField(p->mWriters[3].mpWriterCtrl, CPY_WRITER3_CTRL_ADR);
    p->mWriters[3].mpWriterCtrlCnt = Register_getField(p->mWriters[3].mpWriterCtrl, CPY_WRITER3_CTRL_CNT);
    p->mWriters[3].mpWriterData = Module_getRegister(p->mTxCpy, CPY_WRITER3_DATA);
    p->mWriters[3].mpWriterDataReaderSelect = Register_getField(p->mWriters[3].mpWriterData, CPY_WRITER3_DATA_READER_SELECT);
    p->mWriters[3].mpWriterDataDyn = Register_getField(p->mWriters[3].mpWriterData, CPY_WRITER3_DATA_DYN);
    p->mWriters[3].mpWriterDataOfs = Register_getField(p->mWriters[3].mpWriterData, CPY_WRITER3_DATA_OFS);
    p->mWriters[3].mpWriterDataLen = Register_getField(p->mWriters[3].mpWriterData, CPY_WRITER3_DATA_LEN);
    if (variant != 0) {
      p->mWriters[3].mpWriterDataMaskPointer = Register_getField(p->mWriters[3].mpWriterData, CPY_WRITER3_DATA_MASK_POINTER);
      p->mWriters[3].mpWriterMaskCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER3_MASK_CTRL);
      p->mWriters[3].mpWriterMaskCtrlAddr = Register_getField(p->mWriters[3].mpWriterMaskCtrl, CPY_WRITER3_MASK_CTRL_ADR);
      p->mWriters[3].mpWriterMaskCtrlCnt = Register_getField(p->mWriters[3].mpWriterMaskCtrl, CPY_WRITER3_MASK_CTRL_CNT);
      p->mWriters[3].mpWriterMaskData = Module_getRegister(p->mTxCpy, CPY_WRITER3_MASK_DATA);
      p->mWriters[3].mpWriterMaskDataByteMask = Register_getField(p->mWriters[3].mpWriterMaskData, CPY_WRITER3_MASK_DATA_BYTE_MASK);
    }
    // Fallthrough
  case 3:
    p->mWriters[2].mpWriterCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER2_CTRL);
    p->mWriters[2].mpWriterCtrlAddr = Register_getField(p->mWriters[2].mpWriterCtrl, CPY_WRITER2_CTRL_ADR);
    p->mWriters[2].mpWriterCtrlCnt = Register_getField(p->mWriters[2].mpWriterCtrl, CPY_WRITER2_CTRL_CNT);
    p->mWriters[2].mpWriterData = Module_getRegister(p->mTxCpy, CPY_WRITER2_DATA);
    p->mWriters[2].mpWriterDataReaderSelect = Register_getField(p->mWriters[2].mpWriterData, CPY_WRITER2_DATA_READER_SELECT);
    p->mWriters[2].mpWriterDataDyn = Register_getField(p->mWriters[2].mpWriterData, CPY_WRITER2_DATA_DYN);
    p->mWriters[2].mpWriterDataOfs = Register_getField(p->mWriters[2].mpWriterData, CPY_WRITER2_DATA_OFS);
    p->mWriters[2].mpWriterDataLen = Register_getField(p->mWriters[2].mpWriterData, CPY_WRITER2_DATA_LEN);
    if (variant != 0) {
      p->mWriters[2].mpWriterDataMaskPointer = Register_getField(p->mWriters[2].mpWriterData, CPY_WRITER2_DATA_MASK_POINTER);
      p->mWriters[2].mpWriterMaskCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER2_MASK_CTRL);
      p->mWriters[2].mpWriterMaskCtrlAddr = Register_getField(p->mWriters[2].mpWriterMaskCtrl, CPY_WRITER2_MASK_CTRL_ADR);
      p->mWriters[2].mpWriterMaskCtrlCnt = Register_getField(p->mWriters[2].mpWriterMaskCtrl, CPY_WRITER2_MASK_CTRL_CNT);
      p->mWriters[2].mpWriterMaskData = Module_getRegister(p->mTxCpy, CPY_WRITER2_MASK_DATA);
      p->mWriters[2].mpWriterMaskDataByteMask = Register_getField(p->mWriters[2].mpWriterMaskData, CPY_WRITER2_MASK_DATA_BYTE_MASK);
    }
    // Fallthrough
  case 2:
    p->mWriters[1].mpWriterCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER1_CTRL);
    p->mWriters[1].mpWriterCtrlAddr = Register_getField(p->mWriters[1].mpWriterCtrl, CPY_WRITER1_CTRL_ADR);
    p->mWriters[1].mpWriterCtrlCnt = Register_getField(p->mWriters[1].mpWriterCtrl, CPY_WRITER1_CTRL_CNT);
    p->mWriters[1].mpWriterData = Module_getRegister(p->mTxCpy, CPY_WRITER1_DATA);
    p->mWriters[1].mpWriterDataReaderSelect = Register_getField(p->mWriters[1].mpWriterData, CPY_WRITER1_DATA_READER_SELECT);
    p->mWriters[1].mpWriterDataDyn = Register_getField(p->mWriters[1].mpWriterData, CPY_WRITER1_DATA_DYN);
    p->mWriters[1].mpWriterDataOfs = Register_getField(p->mWriters[1].mpWriterData, CPY_WRITER1_DATA_OFS);
    p->mWriters[1].mpWriterDataLen = Register_getField(p->mWriters[1].mpWriterData, CPY_WRITER1_DATA_LEN);
    if (variant != 0) {
      p->mWriters[1].mpWriterDataMaskPointer = Register_getField(p->mWriters[1].mpWriterData, CPY_WRITER1_DATA_MASK_POINTER);
      p->mWriters[1].mpWriterMaskCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER1_MASK_CTRL);
      p->mWriters[1].mpWriterMaskCtrlAddr = Register_getField(p->mWriters[1].mpWriterMaskCtrl, CPY_WRITER1_MASK_CTRL_ADR);
      p->mWriters[1].mpWriterMaskCtrlCnt = Register_getField(p->mWriters[1].mpWriterMaskCtrl, CPY_WRITER1_MASK_CTRL_CNT);
      p->mWriters[1].mpWriterMaskData = Module_getRegister(p->mTxCpy, CPY_WRITER1_MASK_DATA);
      p->mWriters[1].mpWriterMaskDataByteMask = Register_getField(p->mWriters[1].mpWriterMaskData, CPY_WRITER1_MASK_DATA_BYTE_MASK);
    }
    // Fallthrough
  case 1:
    p->mWriters[0].mpWriterCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER0_CTRL);
    p->mWriters[0].mpWriterCtrlAddr = Register_getField(p->mWriters[0].mpWriterCtrl, CPY_WRITER0_CTRL_ADR);
    p->mWriters[0].mpWriterCtrlCnt = Register_getField(p->mWriters[0].mpWriterCtrl, CPY_WRITER0_CTRL_CNT);
    p->mWriters[0].mpWriterData = Module_getRegister(p->mTxCpy, CPY_WRITER0_DATA);
    p->mWriters[0].mpWriterDataReaderSelect = Register_getField(p->mWriters[0].mpWriterData, CPY_WRITER0_DATA_READER_SELECT);
    p->mWriters[0].mpWriterDataDyn = Register_getField(p->mWriters[0].mpWriterData, CPY_WRITER0_DATA_DYN);
    p->mWriters[0].mpWriterDataOfs = Register_getField(p->mWriters[0].mpWriterData, CPY_WRITER0_DATA_OFS);
    p->mWriters[0].mpWriterDataLen = Register_getField(p->mWriters[0].mpWriterData, CPY_WRITER0_DATA_LEN);
    if (variant != 0) {
      p->mWriters[0].mpWriterDataMaskPointer = Register_getField(p->mWriters[0].mpWriterData, CPY_WRITER0_DATA_MASK_POINTER);
      p->mWriters[0].mpWriterMaskCtrl = Module_getRegister(p->mTxCpy, CPY_WRITER0_MASK_CTRL);
      p->mWriters[0].mpWriterMaskCtrlAddr = Register_getField(p->mWriters[0].mpWriterMaskCtrl, CPY_WRITER0_MASK_CTRL_ADR);
      p->mWriters[0].mpWriterMaskCtrlCnt = Register_getField(p->mWriters[0].mpWriterMaskCtrl, CPY_WRITER0_MASK_CTRL_CNT);
      p->mWriters[0].mpWriterMaskData = Module_getRegister(p->mTxCpy, CPY_WRITER0_MASK_DATA);
      p->mWriters[0].mpWriterMaskDataByteMask = Register_getField(p->mWriters[0].mpWriterMaskData, CPY_WRITER0_MASK_DATA_BYTE_MASK);
    }
    break;
  case 0:
    return -1;
  }

  return 0;
}

void TxCpyNthw_WriterSelect(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  Field_setVal32(p->mWriters[index].mpWriterCtrlAddr, val);
}

void TxCpyNthw_WriterCnt(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  Field_setVal32(p->mWriters[index].mpWriterCtrlCnt, val);
}

void TxCpyNthw_WriterReaderSelect(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  Field_setVal32(p->mWriters[index].mpWriterDataReaderSelect, val);
}

void TxCpyNthw_WriterDyn(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  Field_setVal32(p->mWriters[index].mpWriterDataDyn, val);
}

void TxCpyNthw_WriterOfs(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  Field_setVal32(p->mWriters[index].mpWriterDataOfs, val);
}

void TxCpyNthw_WriterLen(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  Field_setVal32(p->mWriters[index].mpWriterDataLen, val);
}

void TxCpyNthw_WriterMaskPointer(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  assert(p->mWriters[index].mpWriterDataMaskPointer);
  Field_setVal32(p->mWriters[index].mpWriterDataMaskPointer, val);
}

void TxCpyNthw_WriterFlush(const struct TxCpyNthw *p, unsigned index)
{
  assert(index < p->mWritersCnt);
  Register_flush(p->mWriters[index].mpWriterCtrl, 1);
  Register_flush(p->mWriters[index].mpWriterData, 1);
}

void TxCpyNthw_WriterMaskSelect(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  assert(p->mWriters[index].mpWriterMaskCtrlAddr);
  Field_setVal32(p->mWriters[index].mpWriterMaskCtrlAddr, val);
}

void TxCpyNthw_WriterMaskCnt(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  assert(p->mWriters[index].mpWriterMaskCtrlCnt);
  Field_setVal32(p->mWriters[index].mpWriterMaskCtrlCnt, val);
}

void TxCpyNthw_WriterMask(const struct TxCpyNthw *p, unsigned index, uint32_t val)
{
  assert(index < p->mWritersCnt);
  assert(p->mWriters[index].mpWriterMaskDataByteMask);
  Field_setVal32(p->mWriters[index].mpWriterMaskDataByteMask, val);
}

void TxCpyNthw_WriterMaskFlush(const struct TxCpyNthw *p, unsigned index)
{
  assert(index < p->mWritersCnt);
  assert(p->mWriters[index].mpWriterMaskCtrl);
  assert(p->mWriters[index].mpWriterMaskData);
  Register_flush(p->mWriters[index].mpWriterMaskCtrl, 1);
  Register_flush(p->mWriters[index].mpWriterMaskData, 1);
}
