/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//
#include "ntlog.h"
#include "nthw_drv.h"
#include "nthw_register.h"

#include "flow_nthw_tx_rpl.h"

#include <stdlib.h>
#include <string.h>

void txRplNthw_setDebugMode(struct TxRplNthw *p, unsigned nDebugMode)
{
  Module_setDebugMode(p->mTxRpl, nDebugMode);
}

struct TxRplNthw *TxRplNthw_new(void)
{
  struct TxRplNthw *p = malloc(sizeof(struct TxRplNthw));
  if (p) {
    (void)memset(p, 0, sizeof(*p));
  }
  return p;
}

void TxRplNthw_delete(struct TxRplNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int TxRplNthw_init(struct TxRplNthw *p, Fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->pFpgaInfo->mpAdapterIdStr;
  struct Module_s* pMod = Fpga_queryModule(pFpga, MOD_TX_RPL, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: TxRpl %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mTxRpl = Fpga_queryModule(pFpga, MOD_TX_RPL, nInstance);

  p->mpRcpCtrl = Module_getRegister(p->mTxRpl, RPL_RCP_CTRL);
  p->mpRcpCtrlAddr = Register_getField(p->mpRcpCtrl, RPL_RCP_CTRL_ADR);
  p->mpRcpCtrlCnt = Register_getField(p->mpRcpCtrl, RPL_RCP_CTRL_CNT);
  p->mpRcpData = Module_getRegister(p->mTxRpl, RPL_RCP_DATA);
  p->mpRcpDataDyn = Register_getField(p->mpRcpData, RPL_RCP_DATA_DYN);
  p->mpRcpDataOfs = Register_getField(p->mpRcpData, RPL_RCP_DATA_OFS);
  p->mpRcpDataLen = Register_getField(p->mpRcpData, RPL_RCP_DATA_LEN);
  p->mpRcpDataRplPtr = Register_getField(p->mpRcpData, RPL_RCP_DATA_RPL_PTR);
  p->mpRcpDataExtPrio = Register_getField(p->mpRcpData, RPL_RCP_DATA_EXT_PRIO);

  p->mpExtCtrl = Module_getRegister(p->mTxRpl, RPL_EXT_CTRL);
  p->mpExtCtrlAddr = Register_getField(p->mpExtCtrl, RPL_EXT_CTRL_ADR);
  p->mpExtCtrlCnt = Register_getField(p->mpExtCtrl, RPL_EXT_CTRL_CNT);
  p->mpExtData = Module_getRegister(p->mTxRpl, RPL_EXT_DATA);
  p->mpExtDataRplPtr = Register_getField(p->mpExtData, RPL_EXT_DATA_RPL_PTR);

  p->mpRplCtrl = Module_getRegister(p->mTxRpl, RPL_RPL_CTRL);
  p->mpRplCtrlAddr = Register_getField(p->mpRplCtrl, RPL_RPL_CTRL_ADR);
  p->mpRplCtrlCnt = Register_getField(p->mpRplCtrl, RPL_RPL_CTRL_CNT);
  p->mpRplData = Module_getRegister(p->mTxRpl, RPL_RPL_DATA);
  p->mpRplDataValue = Register_getField(p->mpRplData, RPL_RPL_DATA_VALUE);

  return 0;
}

void TxRplNthw_RcpSelect(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCtrlAddr, val);
}

void TxRplNthw_RcpCnt(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpCtrlCnt, val);
}

void TxRplNthw_RcpDyn(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataDyn, val);
}

void TxRplNthw_RcpOfs(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataOfs, val);
}

void TxRplNthw_RcpLen(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataLen, val);
}

void TxRplNthw_RcpRplPtr(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataRplPtr, val);
}

void TxRplNthw_RcpExtPrio(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRcpDataExtPrio, val);
}

void TxRplNthw_RcpFlush(const struct TxRplNthw *p)
{
  Register_flush(p->mpRcpCtrl, 1);
  Register_flush(p->mpRcpData, 1);
}

void TxRplNthw_ExtSelect(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpExtCtrlAddr, val);
}

void TxRplNthw_ExtCnt(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpExtCtrlCnt, val);
}

void TxRplNthw_ExtRplPtr(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpExtDataRplPtr, val);
}

void TxRplNthw_ExtFlush(const struct TxRplNthw *p)
{
  Register_flush(p->mpExtCtrl, 1);
  Register_flush(p->mpExtData, 1);
}

void TxRplNthw_RplSelect(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRplCtrlAddr, val);
}

void TxRplNthw_RplCnt(const struct TxRplNthw *p, uint32_t val)
{
  Field_setVal32(p->mpRplCtrlCnt, val);
}

void TxRplNthw_RplValue(const struct TxRplNthw *p, const uint32_t* val)
{
  Field_setVal(p->mpRplDataValue, val, 4);
}

void TxRplNthw_RplFlush(const struct TxRplNthw *p)
{
  Register_flush(p->mpRplCtrl, 1);
  Register_flush(p->mpRplData, 1);
}
