//
// nthw_fpga_model.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

#ifndef __NTHW_FPGA_MODEL_H__
#define __NTHW_FPGA_MODEL_H__

//
// Includes:
//
#include <stdbool.h>
#include <stdio.h>
#include "fpga_model.hpp"

//
//
//
#ifndef FPGAID_TO_PRODUCTCODE
  #define FPGAID_TO_PRODUCTTYPE(fpga_id)  ((uint16_t)((fpga_id)>>32) & 0xFF)
  #define FPGAID_TO_PRODUCTCODE(fpga_id)  ((uint16_t)((fpga_id)>>16) & 0xFFFF)
  #define FPGAID_TO_VERSIONCODE(fpga_id)  ((uint16_t)((fpga_id)>>8 & 0xFF))
  #define FPGAID_TO_REVISIONCODE(fpga_id) ((uint16_t)((fpga_id)>>0 & 0xFF))
#endif

#define VERSION_PACKED64(_major_, _minor_) ((((uint64_t)_major_ & 0xFFFFFFFF)<<32) | (_minor_ & 0xFFFFFFFF))

//
//
//
enum DebugMode
{
  NO_DEBUG,
  ON_READ,
  ON_WRITE
};

//
//
//
enum NthwBusType {
  NTHW_BUS_UNKNOWN,
  NTHW_BUS_BAR,
  NTHW_BUS_PCI,
  NTHW_BUS_NMB,
  NTHW_BUS_NDM,
  NTHW_BUS_RAB0,
  NTHW_BUS_RAB1,
  NTHW_BUS_RAB2
};


//
// Forward decls
//
struct Fpga_s;

struct Param_s;

struct Module_s;

struct Register_s;

struct Field_s;


//
//
//
struct FpgaMgr_s
{
  int mnFpgas;
  //struct Fpga_s** mpaFpgas;
  struct FpgaProdInit** mpaFpgaProdInit;
};

typedef struct FpgaMgr_s FpgaMgr_t;

//
//
//
struct Fpga_s
{
  struct FpgaInfo_s* pFpgaInfo;
  //
  int mItemId;
  int mProductId;
  int mFpgaVersion;
  int mFpgaRevision;
  int mFpgaPatchNo;
  int mFpgaBuildNo;
  uint32_t mFpgaBuildTime;
  //
  int mnParams;
  struct Param_s** mpaParams;
  //
  int mnModules;
  struct Module_s** mpaModules;
  //
  FpgaProdInit_t* mpInit;
  //
  int mDebugMode;
  //
};

typedef struct Fpga_s Fpga_t;
typedef struct Fpga_s Fpga;



//
//
//
struct Param_s {
  Fpga_t* mpOwner;
  //
  int paramId;
  int paramValue;
  //
  FpgaProdParam_t* mpInit;
};

typedef struct Param_s Param_t;
typedef struct Param_s Param;

//
//
//
struct Module_s
{
  Fpga_t* mpOwner;
  //
  int mModId;
  //
  int mInstance;
  //
  int mModDefId;
  int mMajorVersion;
  int mMinorVersion;
  //
  int mBus;
  uint32_t mAddrBase;
  //
  int mDebugMode;
  //
  int mnRegisters;
  struct Register_s** mpaRegisters;
  //
  FpgaModuleInit_t* mpInit;
};

typedef struct Module_s Module_t;
typedef struct Module_s Module;


//
//
//
struct Register_s
{
  Module_t* mpOwner;
  //
  uint32_t mId;
  //
  uint32_t mnBitWidth;
  uint32_t mnAddrRel;
  uint32_t mAddr;
  uint32_t mType;
  uint32_t mLen;
  //
  int mDebugMode;
  //
  int mnFields;
  struct Field_s** mpaFields;
  //
  uint32_t* mpShadow;
  bool* mpDirty;
  //
  FpgaRegisterInit_t* mpInit;
};

typedef struct Register_s Register_t;
typedef struct Register_s Register;

//
//
//
struct Field_s
{
  Register_t* mpOwner;
  //
  uint32_t mId;
  //
  uint32_t mnBitWidth;
  uint32_t mnBitPosLow;
  uint32_t mResetVal;
  uint32_t mFirstWord;
  uint32_t mFirstBit;
  uint32_t mFrontMask;
  uint32_t mBodyLength;
  uint32_t mnWords;
  uint32_t mTailMask;
  //
  int mDebugMode;
  //
  FpgaFieldInit_t* mpInit;
};

typedef struct Field_s Field_t;
typedef struct Field_s Field;


//
//
//
FpgaMgr_t* FpgaMgr_new(void);
void FpgaMgr_init(FpgaMgr_t* p);
void FpgaMgr_delete(FpgaMgr_t* p);
//Fpga_t* FpgaMgr_lookup(FpgaMgr_t* p, uint64_t nFpgaId);
Fpga_t* FpgaMgr_queryFpga(FpgaMgr_t* p, uint64_t nFpgaId, struct FpgaInfo_s* pFpgaInfo);
//
void FpgaMgr_log_dump(FpgaMgr_t* p);
void FpgaMgr_show(FpgaMgr_t* p, FILE* out, int detail_level);

//
//
//
Fpga_t* Fpga_new(void);
//void Fpga_init(Fpga_t* p, FpgaProdInit_t* fpgaProdInit);
void Fpga_delete(Fpga_t* p);
void Fpga_delete_all(Fpga_t* p);
void Fpga_init(Fpga_t* p, FpgaProdInit_t* fpgaProdInit, struct FpgaInfo_s* pFpgaInfo);
//
int Fpga_getProductParam(const Fpga_t* p, const int nParamId, const int defaultValue);
int Fpga_getProductId(const Fpga_t* p);
int Fpga_getFpgaVersion(const Fpga_t* p);
int Fpga_getFpgaRevision(const Fpga_t* p);
Module_t* Fpga_queryModule(const Fpga_t* p, int id, int instance);
FpgaModuleInit_t* Fpga_lookup_init(Fpga_t* p, int id, int instance);
bool Fpga_query(Fpga_t* p, int id, int instance);
void Fpga_setDebugMode(Fpga_t* p, int nDebugMode);
//
void Fpga_logInfo(const Fpga_t* p);
void Fpga_dump(const Fpga_t* p);
void Fpga_dumpParams(const Fpga_t* p);
void Fpga_dumpModules(const Fpga_t* p);

//
//
//
Param_t* Param_new(void);
void Param_delete(Param_t* p);
void Param_init(Param_t* p, Fpga_t* pFpga, FpgaProdParam_t* pInit);
//
void Param_dump(const Param_t* p);
//
//
//
Module_t* Module_new(void);
void Module_delete(Module_t* p);
void Module_init(Module_t* p, Fpga_t* pFpga, FpgaModuleInit_t* pInit);
void Module_init2(Module_t* p, Fpga_t* pFpga, int modId, int instance, int debugMode);
//
int Module_getMajorVersion(const Module_t* p);
int Module_getMinorVersion(const Module_t* p);
uint64_t Module_getVersionPacked64(const Module_t* p);
bool Module_isVersionNewer(const Module_t* p, int majorVersion, int minorVersion);
//
int Module_getBus(const Module_t* p);
Register_t* Module_getRegister(Module_t* p, uint32_t id);
Register_t* Module_queryRegister(Module_t* p, uint32_t id);
int Module_getDebugMode(const Module_t* p);
void Module_setDebugMode(Module_t* p, unsigned debugMode);
uint32_t Module_getAddrBase(const Module_t* p);
void Module_unsuppported(const Module_t* p);
//
void Module_dump(const Module_t* p);
void Module_dumpRegisters(const Module_t* p);
//
//
//
Register_t* Register_new(void);
void Register_delete(Register_t* p);
void Register_init(Register_t* p, Module_t* pModule, FpgaRegisterInit_t* pInit);
//
Field_t* Register_getField(const Register_t* p, uint32_t id);
Field_t* Register_queryField(const Register_t* p, uint32_t id);
//
uint32_t Register_getAddress(const Register_t* p);
uint32_t Register_getAddrRel(const Register_t* p);
int Register_getBitWidth(const Register_t* p);
int Register_getDebugMode(const Module_t* p);
void Register_setDebugMode(Register_t* p, unsigned debugMode);
//
void Register_getVal(const Register_t* p, uint32_t* pData, uint32_t len);
uint32_t Register_getVal32(const Register_t* p);
uint32_t Register_getValUpdated32(const Register_t* p);
//
void Register_setVal(Register_t* p, const uint32_t* pData, uint32_t len);
void Register_setValFlush(Register_t* p, const uint32_t* pData, uint32_t len);
//
void Register_makeDirty(Register_t* p);
void Register_update(const Register_t* p);
void Register_reset(const Register_t* p);
void Register_flush(const Register_t* p, uint32_t cnt);
void Register_clr(Register_t* p);
void Register_set(Register_t* p);
//
void Register_doReadTrigTs(const Register_t* p, uint64_t* tsc1, uint64_t* tsc2);
//
void Register_dump(const Register_t* p);
void Register_dumpFields(const Register_t* p);
//
//
//
Field_t* Field_new(void);
void Field_delete(Field_t* p);
void Field_init(Field_t* p, Register* pReg, const FpgaFieldInit_t* pInit);
//
int Field_getDebugMode(const Module_t* p);
void Field_setDebugMode(Field_t* p, unsigned nDebugMode);
int Field_getBitWidth(const Field_t* p);
int Field_getBitPosLow(const Field_t* p);
int Field_getBitPosHigh(const Field_t* p);
uint32_t Field_getMask(const Field_t* p);
void Field_reset(const Field_t* p);
uint32_t Field_getResetVal(const Field_t* p);
void Field_getVal(const Field_t* p, uint32_t* pData, uint32_t len);
void Field_setVal(const Field_t* p, const uint32_t* pData, uint32_t len);
void Field_setValFlush(const Field_t* p, const uint32_t* pData, uint32_t len);
uint32_t Field_getValMask(const Field_t* p);
uint32_t Field_getVal32(const Field_t* p);
uint32_t Field_getUpdated(const Field_t* p);
void Field_readTrigWithTsc(const Field_t* p, uint64_t* tsc1, uint64_t* tsc2);
void Field_updateRegister(const Field_t* p);
void Field_flushRegister(const Field_t* p);
void Field_setVal32(const Field_t* p, uint32_t val);
void Field_setValFlush32(const Field_t* p, uint32_t val);
void Field_clrAll(const Field_t* p);
void Field_clrFlush(const Field_t* p);
void Field_setAll(const Field_t* p);
void Field_setFlush(const Field_t* p);
//
int Field_waitClrAll32(const Field_t* p, int nPollIterations, int nPollInterval);
int Field_waitSetAll32(const Field_t* p, int nPollIterations, int nPollInterval);
//
int Field_waitClrAny32(const Field_t* p, int nPollIterations, int nPollInterval);
int Field_waitSetAny32(const Field_t* p, int nPollIterations, int nPollInterval);
//
int Field_waitValMask32(const Field_t* p, uint32_t nWaitCondValue, uint32_t nWaitCondMask, int nPollIterations, int nPollInterval);

//
void Field_dump(const Field_t* p);
void Field_dumpVal(const Field_t* p);
void Field_dump_init(const FpgaFieldInit_t* p);

//
// nthw helpers
//
Fpga_t* Nthw_getFpga(struct FpgaInfo_s* pFpgaInfo, uint64_t nFpgaIdent);
Module_t* Nthw_getModule(Fpga_t* pFpga, int nMod, int nInstance);
Register_t* Nthw_getRegister(Module_t* pMod, int nReg);
Field_t* Nthw_getField(Register_t* pReg, int nFld);


//
//
//
#endif // __NTHW_FPGA_MODEL_H__

//
// EOF
//
