//
// nthw_dbs.h
//


/*
 * %NT_SOFTWARE_LICENSE%
 */


#ifndef NTHW_DBS_HPP_
#define NTHW_DBS_HPP_

// Note: for now defined here. Ideally get from P200 file
#define NT_DBS_RX_QUEUES_MAX (128)
#define NT_DBS_TX_QUEUES_MAX (128)

//**************************************************************//
// Struct for implementation of memory bank shadows
//**************************************************************//

// DBS_RX_AM_DATA
struct NthwDbsRxAmData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t enable;
  uint32_t hostId;
  uint32_t packed;
  uint32_t int_enable;
};

// DBS_TX_AM_DATA
struct NthwDbsTxAmData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t enable;
  uint32_t hostId;
  uint32_t packed;
  uint32_t int_enable;
};

// DBS_RX_UW_DATA
struct NthwDbsRxUwData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t packed;
  uint32_t int_enable;
  uint32_t vec;
  uint32_t istk;
};

// DBS_TX_UW_DATA
struct NthwDbsTxUwData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t packed;
  uint32_t int_enable;
  uint32_t vec;
  uint32_t istk;
  uint32_t inOrder;
};

// DBS_RX_DR_DATA
struct NthwDbsRxDrData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t header;
  uint32_t packed;
};

// DBS_TX_DR_DATA
struct NthwDbsTxDrData_s
{
  uint64_t guestPhysicalAddress;
  uint32_t hostId;
  uint32_t queueSize;
  uint32_t header;
  uint32_t port;
  uint32_t packed;
};

// DBS_TX_QP_DATA
struct NthwDbsTxQpData_s
{
  uint32_t virtualPort;
};

struct NthwDbsTxQosData_s
{
  uint32_t enable;
  uint32_t ir;
  uint32_t bs;
};
//@G++


//
//
//
struct NthwDbs_s
{
  //
  Fpga_t* mpFpga;
  //
  Module_t* mpModDbs;
  //
  int mnInstance;
  //
  int mnParamDbsPresent;
  //
  Register_t* mpRegRxControl;
  Field_t* mpFldRxControlLastQueue;
  Field_t* mpFldRxControlAvailMonitorEnable;
  Field_t* mpFldRxControlAvailMonitorScanSpeed;
  Field_t* mpFldRxControlUsedWriteEnable;
  Field_t* mpFldRxControlUsedWriterUpdateSpeed;
  Field_t* mpFldRxControlRxQueuesEnable;
  //
  Register_t* mpRegTxControl;
  Field_t* mpFldTxControlLastQueue;
  Field_t* mpFldTxControlAvailMonitorEnable;
  Field_t* mpFldTxControlAvailMonitorScanSpeed;
  Field_t* mpFldTxControlUsedWriteEnable;
  Field_t* mpFldTxControlUsedWriterUpdateSpeed;
  Field_t* mpFldTxControlTxQueuesEnable;
  //
  Register_t* mpRegRxInit;
  Field_t* mpFldRxInitInit;
  Field_t* mpFldRxInitQueue;
  Field_t* mpFldRxInitBusy;

  Register_t* mpRegRxInitVal;
  Field_t* mpFldRxInitValIdx;
  Field_t* mpFldRxInitValPtr;

  Register_t* mpRegRxPtr;
  Field_t* mpFldRxPtrPtr;
  Field_t* mpFldRxPtrQueue;
  Field_t* mpFldRxPtrValid;


  Register_t* mpRegTxInit;
  Field_t* mpFldTxInitInit;
  Field_t* mpFldTxInitQueue;
  Field_t* mpFldTxInitBusy;

  Register_t* mpRegTxInitVal;
  Field_t* mpFldTxInitValIdx;
  Field_t* mpFldTxInitValPtr;

  Register_t* mpRegTxPtr;
  Field_t* mpFldTxPtrPtr;
  Field_t* mpFldTxPtrQueue;
  Field_t* mpFldTxPtrValid;


  Register_t* mpRegRxIdle;
  Field_t* mpFldRxIdleIdle;
  Field_t* mpFldRxIdleQueue;
  Field_t* mpFldRxIdleBusy;

  Register_t* mpRegTxIdle;
  Field_t* mpFldTxIdleIdle;
  Field_t* mpFldTxIdleQueue;
  Field_t* mpFldTxIdleBusy;

  Register_t* mpRegRxAvailMonitorControl;
  Field_t* mpFldRxAvailMonitorControlAdr;
  Field_t* mpFldRxAvailMonitorControlCnt;

  Register_t* mpRegRxAvailMonitorData;
  Field_t* mpFldRxAvailMonitorDataGuestPhysicalAddress;
  Field_t* mpFldRxAvailMonitorDataEnable;
  Field_t* mpFldRxAvailMonitorDataHostID;
  Field_t* mpFldRxAvailMonitorDataPacked;
  Field_t* mpFldRxAvailMonitorDataInt;

  Register_t* mpRegTxAvailMonitorControl;
  Field_t* mpFldTxAvailMonitorControlAdr;
  Field_t* mpFldTxAvailMonitorControlCnt;

  Register_t* mpRegTxAvailMonitorData;
  Field_t* mpFldTxAvailMonitorDataGuestPhysicalAddress;
  Field_t* mpFldTxAvailMonitorDataEnable;
  Field_t* mpFldTxAvailMonitorDataHostID;
  Field_t* mpFldTxAvailMonitorDataPacked;
  Field_t* mpFldTxAvailMonitorDataInt;

  Register_t* mpRegRxUsedWriterControl;
  Field_t* mpFldRxUsedWriterControlAdr;
  Field_t* mpFldRxUsedWriterControlCnt;

  Register_t* mpRegRxUsedWriterData;
  Field_t* mpFldRxUsedWriterDataGuestPhysicalAddress;
  Field_t* mpFldRxUsedWriterDataHostID;
  Field_t* mpFldRxUsedWriterDataQueueSize;
  Field_t* mpFldRxUsedWriterDataPacked;
  Field_t* mpFldRxUsedWriterDataInt;
  Field_t* mpFldRxUsedWriterDataVec;
  Field_t* mpFldRxUsedWriterDataIstk;

  Register_t* mpRegTxUsedWriterControl;
  Field_t* mpFldTxUsedWriterControlAdr;
  Field_t* mpFldTxUsedWriterControlCnt;

  Register_t* mpRegTxUsedWriterData;
  Field_t* mpFldTxUsedWriterDataGuestPhysicalAddress;
  Field_t* mpFldTxUsedWriterDataHostID;
  Field_t* mpFldTxUsedWriterDataQueueSize;
  Field_t* mpFldTxUsedWriterDataPacked;
  Field_t* mpFldTxUsedWriterDataInt;
  Field_t* mpFldTxUsedWriterDataVec;
  Field_t* mpFldTxUsedWriterDataIstk;
  Field_t* mpFldTxUsedWriterDataInOrder;


  Register_t* mpRegRxDescriptorReaderControl;
  Field_t* mpFldRxDescriptorReaderControlAdr;
  Field_t* mpFldRxDescriptorReaderControlCnt;

  Register_t* mpRegRxDescriptorReaderData;
  Field_t* mpFldRxDescriptorReaderDataGuestPhysicalAddress;
  Field_t* mpFldRxDescriptorReaderDataHostID;
  Field_t* mpFldRxDescriptorReaderDataQueueSize;
  Field_t* mpFldRxDescriptorReaderDataHeader;
  Field_t* mpFldRxDescriptorReaderDataPacked;

  Register_t* mpRegTxDescriptorReaderControl;
  Field_t* mpFldTxDescriptorReaderControlAdr;
  Field_t* mpFldTxDescriptorReaderControlCnt;

  Register_t* mpRegTxDescriptorReaderData;
  Field_t* mpFldTxDescriptorReaderDataGuestPhysicalAddress;
  Field_t* mpFldTxDescriptorReaderDataHostID;
  Field_t* mpFldTxDescriptorReaderDataQueueSize;
  Field_t* mpFldTxDescriptorReaderDataPort;
  Field_t* mpFldTxDescriptorReaderDataHeader;
  Field_t* mpFldTxDescriptorReaderDataPacked;

  Register_t* mpRegTxQueuePropertyControl;
  Field_t* mpFldTxQueuePropertyControlAdr;
  Field_t* mpFldTxQueuePropertyControlCnt;

  Register_t* mpRegTxQueuePropertyData;
  Field_t* mpFldTxQueuePropertyDataVPort;

  Register_t* mpRegTxQueueQosControl;
  Field_t* mpRegTxQueueQosControlAdr;
  Field_t* mpRegTxQueueQosControlCnt;

  Register_t* mpRegTxQueueQosData;
  Field_t* mpRegTxQueueQosDataEn;
  Field_t* mpRegTxQueueQosDataIR;
  Field_t* mpRegTxQueueQosDataBS;

  Register_t* mpRegTxQueueQosRate;
  Field_t* mpRegTxQueueQosRateMul;
  Field_t* mpRegTxQueueQosRateDiv;

  struct NthwDbsRxAmData_s mRxAmShadow[NT_DBS_RX_QUEUES_MAX];
  struct NthwDbsRxUwData_s mRxUwShadow[NT_DBS_RX_QUEUES_MAX];
  struct NthwDbsRxDrData_s mRxDrShadow[NT_DBS_RX_QUEUES_MAX];
  //
  struct NthwDbsTxAmData_s mTxAmShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxUwData_s mTxUwShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxDrData_s mTxDrShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxQpData_s mTxQpShadow[NT_DBS_TX_QUEUES_MAX];
  struct NthwDbsTxQosData_s mTxQosShadow[NT_DBS_TX_QUEUES_MAX];
  //
};

typedef struct NthwDbs_s NthwDbs_t;

//
//
//
NthwDbs_t* NthwDbs_new(void);
void NthwDbs_delete(NthwDbs_t* p);
int dbs_init(NthwDbs_t* p, Fpga_t* pFpga, int nInstance);
void dbs_reset(NthwDbs_t* p);

int dbs_reset_rx_control(NthwDbs_t* p);
int dbs_reset_tx_control(NthwDbs_t* p);
int set_rx_control(NthwDbs_t* p,
                         uint32_t lastQueue,
                         uint32_t availMonitorEnable,
                         uint32_t availMonitorSpeed,
                         uint32_t usedWriteEnable,
                         uint32_t usedWriteSpeed,
                         uint32_t rxQueueEnable);
int NthwDbs_getRxControl(NthwDbs_t* p,
                         uint32_t *lastQueue,
                         uint32_t *availMonitorEnable,
                         uint32_t *availMonitorSpeed,
                         uint32_t *usedWriteEnable,
                         uint32_t *usedWriteSpeed,
                         uint32_t *rxQueueEnable);
int set_tx_control(NthwDbs_t* p,
                         uint32_t lastQueue,
                         uint32_t availMonitorEnable,
                         uint32_t availMonitorSpeed,
                         uint32_t usedWriteEnable,
                         uint32_t usedWriteSpeed,
                         uint32_t txQueueEnable);
int NthwDbs_getTxControl(NthwDbs_t* p,
                         uint32_t *lastQueue,
                         uint32_t *availMonitorEnable,
                         uint32_t *availMonitorSpeed,
                         uint32_t *usedWriteEnable,
                         uint32_t *usedWriteSpeed,
                         uint32_t *txQueueEnable);
int set_rx_init(NthwDbs_t* p,
                      uint32_t startIdx,
                      uint32_t startPtr,
                      uint32_t init,
                      uint32_t queue);
int get_rx_init(NthwDbs_t* p,
                      uint32_t *init,
                      uint32_t *queue,
                      uint32_t *busy);
int set_tx_init(NthwDbs_t* p,
                      uint32_t startIdx,
                      uint32_t startPtr,
                      uint32_t init,
                      uint32_t queue);
int get_tx_init(NthwDbs_t* p,
                      uint32_t *init,
                      uint32_t *queue,
                      uint32_t *busy);
int set_rx_idle(NthwDbs_t* p,
                      uint32_t idle,
                      uint32_t queue);
int get_rx_idle(NthwDbs_t* p,
                      uint32_t *idle,
                      uint32_t *queue,
                      uint32_t *busy);
int set_tx_idle(NthwDbs_t* p,
                      uint32_t idle,
                      uint32_t queue);
int get_tx_idle(NthwDbs_t* p,
                      uint32_t *idle,
                      uint32_t *queue,
                      uint32_t *busy);
int set_rx_ptr_queue(NthwDbs_t* p, uint32_t queue);
int get_rx_ptr(NthwDbs_t* p,
                     uint32_t *ptr,
                     uint32_t *queue,
                     uint32_t *valid);
int set_tx_ptr_queue(NthwDbs_t* p, uint32_t queue);
int get_tx_ptr(NthwDbs_t* p,
                     uint32_t *ptr,
                     uint32_t *queue,
                     uint32_t *valid);
int set_rx_am_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t enable,
                        uint32_t hostId,
                        uint32_t packed,
                        uint32_t int_enable);
int set_tx_am_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t enable,
                        uint32_t hostId,
                        uint32_t packed,
                        uint32_t int_enable);
int set_rx_uw_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t packed,
                        uint32_t int_enable,
                        uint32_t vec,
                        uint32_t istk);
int set_tx_uw_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t packed,
                        uint32_t int_enable,
                        uint32_t vec,
                        uint32_t istk,
                        uint32_t inOrder);
int set_rx_dr_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t header,
                        uint32_t packed);
int set_tx_dr_data(NthwDbs_t* p,
                        uint32_t index,
                        uint64_t guestPhysicalAddress,
                        uint32_t hostId,
                        uint32_t queueSize,
                        uint32_t port,
                        uint32_t header,
                        uint32_t packed);
int NthwDbs_setTxQpData(NthwDbs_t* p,
                        uint32_t index,
                        uint32_t virtualPort);
int set_tx_qos_data(NthwDbs_t* p,
                         uint32_t index,
                         uint32_t enable,
                         uint32_t ir,
                         uint32_t bs);
int set_tx_qos_rate(NthwDbs_t* p,
                         uint32_t mul,
                         uint32_t div);
//
//
//
#endif // NTHW_DBS_H_

//
// EOF
//
