//
// nthw_epp.c
//

#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"
//
#include "nthw_epp.h"

#include <errno.h>  // ENOTSUP


//
//
//
NthwEpp_t* NthwEpp_new(void)
{
  NthwEpp_t* p = malloc(sizeof(NthwEpp_t));
  if (p) {
    memset(p, 0, sizeof(NthwEpp_t));
  }
  return p;
}

//
//
//
void NthwEpp_delete(NthwEpp_t *p)
{
  if (p) {
    memset(p, 0, sizeof(NthwEpp_t));
    free(p);
  }
}

//
//
//
int NthwEpp_present(Fpga_t *pFpga, int nInstance)
{
  return NthwEpp_init(NULL, pFpga, nInstance) == 0;
}

//
//
//
int NthwEpp_init(NthwEpp_t *p, Fpga_t *pFpga, int nInstance)
{
  struct Module_s *mod = Fpga_queryModule(pFpga, MOD_EPP, nInstance);

  if (p == NULL)
    return mod == NULL ? -1 : 0;

  if (mod == NULL) {
    NT_LOG(ERR, NTHW, "%s: EPP %d: no such instance\n", pFpga->pFpgaInfo->mpAdapterIdStr,
        nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mnInstance = nInstance;
  p->mpModEpp = mod;

#if defined(DEBUG) && (0)
  Module_setDebugMode(p->mpModEpp, 0x00); // DEBUG:
#endif

  p->mnEppCategories = Fpga_getProductParam(pFpga, NT_EPP_CATEGORIES, 0);

  p->mpRegReciepeMemoryControl = Module_getRegister(p->mpModEpp, EPP_RCP_CTRL);
  p->mpFldReciepeMemoryControlAdr = Register_getField(p->mpRegReciepeMemoryControl, EPP_RCP_CTRL_ADR);
  p->mpFldReciepeMemoryControlCnt = Register_getField(p->mpRegReciepeMemoryControl, EPP_RCP_CTRL_CNT);

  p->mpRegReciepeMemoryData = Module_getRegister(p->mpModEpp, EPP_RCP_DATA);
  p->mpFldReciepeMemoryDataTxMtuEppEnable  = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_TX_MTU_EPP_EN);
  p->mpFldReciepeMemoryDataQueueMtuEppEnable  = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_QUEUE_MTU_EPP_EN);
  p->mpFldReciepeMemoryDataSizeAdjustTxPort = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_SIZE_ADJUST_TXP);
  p->mpFldReciepeMemoryDataSizeAdjustVirtualPort = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_SIZE_ADJUST_VPORT);
  p->mpFldReciepeMemoryDataFixed18bL2Mtu = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_FIXED_18B_L2_MTU);
  p->mpFldReciepeMemoryDataTxpQosEppEnable  = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_TX_QOS_EPP_EN);
  p->mpFldReciepeMemoryDataQueueQosEppEnable  = Register_getField(p->mpRegReciepeMemoryData, EPP_RCP_DATA_QUEUE_QOS_EPP_EN);

  p->mpRegTxpPortMtuControl = Module_getRegister(p->mpModEpp, EPP_TXP_MTU_CTRL);
  p->mpFldTxpPortMtuControlAdr = Register_getField(p->mpRegTxpPortMtuControl, EPP_TXP_MTU_CTRL_ADR);
  p->mpFldTxpPortMtuControlCnt = Register_getField(p->mpRegTxpPortMtuControl, EPP_TXP_MTU_CTRL_CNT);

  p->mpRegTxpPortMtuData = Module_getRegister(p->mpModEpp, EPP_TXP_MTU_DATA);
  p->mpFldTxpPortMtuDataMaxMtu = Register_getField(p->mpRegTxpPortMtuData, EPP_TXP_MTU_DATA_MAX_MTU);

  p->mpRegQueueMtuControl = Module_getRegister(p->mpModEpp, EPP_QUEUE_MTU_CTRL);
  p->mpFldQueueMtuControlAdr = Register_getField(p->mpRegQueueMtuControl, EPP_QUEUE_MTU_CTRL_ADR);
  p->mpFldQueueMtuControlCnt = Register_getField(p->mpRegQueueMtuControl, EPP_QUEUE_MTU_CTRL_CNT);

  p->mpRegQueueMtuData = Module_getRegister(p->mpModEpp, EPP_QUEUE_MTU_DATA);
  p->mpFldQueueMtuDataMaxMtu = Register_getField(p->mpRegQueueMtuData, EPP_QUEUE_MTU_DATA_MAX_MTU);

  p->mpRegTxpQosControl = Module_getRegister(p->mpModEpp, EPP_TXP_QOS_CTRL);
  p->mpFldTxpQosControlAdr = Register_getField(p->mpRegTxpQosControl, EPP_TXP_QOS_CTRL_ADR);
  p->mpFldTxpQosControlCnt = Register_getField(p->mpRegTxpQosControl, EPP_TXP_QOS_CTRL_CNT);

  p->mpRegTxpQosData = Module_getRegister(p->mpModEpp, EPP_TXP_QOS_DATA);
  p->mpFldTxpQosDataEnable = Register_getField(p->mpRegTxpQosData, EPP_TXP_QOS_DATA_EN);
  p->mpFldTxpQosDataInformationRate = Register_getField(p->mpRegTxpQosData, EPP_TXP_QOS_DATA_IR);
  p->mpFldTxpQosDataInformationRateFractional = Register_getField(p->mpRegTxpQosData, EPP_TXP_QOS_DATA_IR_FRACTION);
  p->mpFldTxpQosDataBurstSize = Register_getField(p->mpRegTxpQosData, EPP_TXP_QOS_DATA_BS);

  p->mpRegVportQosControl = Module_getRegister(p->mpModEpp, EPP_VPORT_QOS_CTRL);
  p->mpFldVportQosControlAdr = Register_getField(p->mpRegVportQosControl, EPP_VPORT_QOS_CTRL_ADR);
  p->mpFldVportQosControlCnt = Register_getField(p->mpRegVportQosControl, EPP_VPORT_QOS_CTRL_CNT);

  p->mpRegVportQosData = Module_getRegister(p->mpModEpp, EPP_VPORT_QOS_DATA);
  p->mpFldVportQosDataEnable = Register_getField(p->mpRegVportQosData, EPP_VPORT_QOS_DATA_EN);
  p->mpFldVportQosDataInformationRate = Register_getField(p->mpRegVportQosData, EPP_VPORT_QOS_DATA_IR);
  p->mpFldVportQosDataInformationRateFractional = Register_getField(p->mpRegVportQosData, EPP_VPORT_QOS_DATA_IR_FRACTION);
  p->mpFldVportQosDataBurstSize = Register_getField(p->mpRegVportQosData, EPP_VPORT_QOS_DATA_BS);

  p->mpRegQueueVportControl = Module_getRegister(p->mpModEpp, EPP_QUEUE_VPORT_CTRL);
  p->mpFldQueueVportControlAdr = Register_getField(p->mpRegQueueVportControl, EPP_QUEUE_VPORT_CTRL_ADR);
  p->mpFldQueueVportControlCnt = Register_getField(p->mpRegQueueVportControl, EPP_QUEUE_VPORT_CTRL_CNT);

  p->mpRegQueueVportData = Module_getRegister(p->mpModEpp, EPP_QUEUE_VPORT_DATA);
  p->mpFldQueueVportDataVport = Register_getField(p->mpRegQueueVportData, EPP_QUEUE_VPORT_DATA_VPORT);

  return 0;
}

int NthwEpp_setup(NthwEpp_t *p)
{
  if(p == NULL)
    return 0;

  // set recieps for 2 first records
  Field_setVal32(p->mpFldReciepeMemoryControlCnt, 1);

  // Zero all categories
  for(int i = 0; i < p->mnEppCategories; ++i) {
    Field_setVal32(p->mpFldReciepeMemoryControlAdr, i);
    Register_flush(p->mpRegReciepeMemoryControl, 1);

    Field_setVal32(p->mpFldReciepeMemoryDataTxMtuEppEnable, 0);
    Field_setVal32(p->mpFldReciepeMemoryDataQueueMtuEppEnable, 0);
    Field_setVal32(p->mpFldReciepeMemoryDataSizeAdjustTxPort, 0);
    Field_setVal32(p->mpFldReciepeMemoryDataSizeAdjustVirtualPort, 0);
    Field_setVal32(p->mpFldReciepeMemoryDataFixed18bL2Mtu, 0);
    Field_setVal32(p->mpFldReciepeMemoryDataTxpQosEppEnable, 0);
    Field_setVal32(p->mpFldReciepeMemoryDataQueueQosEppEnable, 0);
    Register_flush(p->mpRegReciepeMemoryData, 1);
  }

  for(int i = 0; i < NRECIPE; ++i) {
    Field_setVal32(p->mpFldReciepeMemoryControlAdr, i);
    Register_flush(p->mpRegReciepeMemoryControl, 1);

    Field_setVal32(p->mpFldReciepeMemoryDataTxMtuEppEnable, 1);
    Field_setVal32(p->mpFldReciepeMemoryDataQueueMtuEppEnable, 1);
    Field_setVal32(p->mpFldReciepeMemoryDataSizeAdjustTxPort, rcpDataSizeAdjustTxp[i]);
    Field_setVal32(p->mpFldReciepeMemoryDataSizeAdjustVirtualPort, rcpDataSizeAdjustVport[i]);
    Field_setVal32(p->mpFldReciepeMemoryDataFixed18bL2Mtu, 1);
    Field_setVal32(p->mpFldReciepeMemoryDataTxpQosEppEnable, 1);
    Field_setVal32(p->mpFldReciepeMemoryDataQueueQosEppEnable, 1);
    Register_flush(p->mpRegReciepeMemoryData, 1);
  }
  // phy mtu setup
  Field_setVal32(p->mpFldTxpPortMtuControlCnt, 1);
  for(int i = 0; i < 2; ++i) {
    Field_setVal32(p->mpFldTxpPortMtuControlAdr, i);
    Register_flush(p->mpRegTxpPortMtuControl, 1);

    Field_setVal32(p->mpFldTxpPortMtuDataMaxMtu, MTUINITVAL);
    Register_flush(p->mpRegTxpPortMtuData, 1);
  }
  // phy QoS setup
  Field_setVal32(p->mpFldTxpQosControlCnt, 1);
  for(int i = 0; i < 2; ++i) {
    Field_setVal32(p->mpFldTxpQosControlAdr, i);
    Register_flush(p->mpRegTxpQosControl, 1);

    Field_setVal32(p->mpFldTxpQosDataEnable, 0);
    Register_flush(p->mpRegTxpQosData, 1);
  }

  // virt mtu setup
  Field_setVal32(p->mpFldQueueMtuControlCnt, 1);
  for(int i = 0; i < 128; ++i) {
    Field_setVal32(p->mpFldQueueMtuControlAdr, i);
    Register_flush(p->mpRegQueueMtuControl, 1);

    Field_setVal32(p->mpFldQueueMtuDataMaxMtu, MTUINITVAL);
    Register_flush(p->mpRegQueueMtuData, 1);
  }

  // virt QoS setup
  Field_setVal32(p->mpFldVportQosControlCnt, 1);
  for(int i = 0; i < 128; ++i) {
    Field_setVal32(p->mpFldVportQosControlAdr, i);
    Register_flush(p->mpRegVportQosControl, 1);

    Field_setVal32(p->mpFldVportQosDataEnable, 0);
    Register_flush(p->mpRegVportQosData, 1);
  }

  return 0;
}

//
// Set the MTU registers in context with the current setMTU request.
//
int NthwEpp_setMtu(NthwEpp_t *p, uint32_t port, uint32_t maxMtu, NtMetaPortType_t portType)
{
  if(p == NULL)
    return 0;

  if (portType == PORT_TYPE_PHYSICAL) {
    // Set the TXP Mtu control register
    Field_setVal32(p->mpFldTxpPortMtuControlAdr, port);
    Field_setVal32(p->mpFldTxpPortMtuControlCnt, 1);
    Register_flush(p->mpRegTxpPortMtuControl, 1);

    // Set the TXP Mtu control register
    Field_setVal32(p->mpFldTxpPortMtuDataMaxMtu, maxMtu);
    Register_flush(p->mpRegTxpPortMtuData, 1);
  } else if (portType == PORT_TYPE_VIRTUAL) {
    // Set the TXP Mtu control register
      Field_setVal32(p->mpFldQueueMtuControlAdr, port);
    Field_setVal32(p->mpFldQueueMtuControlCnt, 1);
    Register_flush(p->mpRegQueueMtuControl, 1);

    // Set the TXP Mtu control register
    Field_setVal32(p->mpFldQueueMtuDataMaxMtu, maxMtu);
    Register_flush(p->mpRegQueueMtuData, 1);
  } else {
    NT_LOG(DBG, NTHW, "NthwEpp::%s - portType unsupported", __func__);
    Register_reset(p->mpRegQueueMtuControl);
    Register_flush(p->mpRegQueueMtuControl, 1);
    Register_reset(p->mpRegQueueMtuData);
    Register_flush(p->mpRegQueueMtuData, 1);
    Register_reset(p->mpRegTxpPortMtuControl);
    Register_flush(p->mpRegTxpPortMtuControl, 1);
    Register_reset(p->mpRegTxpPortMtuData);
    Register_flush(p->mpRegTxpPortMtuData, 1);

    return -ENOTSUP;
  }

  return 0;
}

int NthwEpp_setTxpQos(NthwEpp_t *p, uint32_t port, uint32_t informationRate, uint32_t informationRateFractional, uint32_t burstSize)
{
  if(p == NULL)
    return 0;

  Field_setVal32(p->mpFldTxpQosControlAdr, port);
  Field_setVal32(p->mpFldTxpQosControlCnt, 1);
  Register_flush(p->mpRegTxpQosControl, 1);

  uint32_t enable = ((informationRate | informationRateFractional | burstSize) != 0);
  Field_setVal32(p->mpFldTxpQosDataEnable, enable);
  Field_setVal32(p->mpFldTxpQosDataInformationRate, informationRate);
  Field_setVal32(p->mpFldTxpQosDataInformationRateFractional, informationRateFractional);
  Field_setVal32(p->mpFldTxpQosDataBurstSize, burstSize);
  Register_flush(p->mpRegTxpQosData, 1);

    return 0;
}

int NthwEpp_setVportQos(NthwEpp_t *p, uint32_t port, uint32_t informationRate, uint32_t informationRateFractional, uint32_t burstSize)
{
  if(p == NULL)
    return 0;

  Field_setVal32(p->mpFldVportQosControlAdr, port);
  Field_setVal32(p->mpFldVportQosControlCnt, 1);
  Register_flush(p->mpRegVportQosControl, 1);

  uint32_t enable = ((informationRate | informationRateFractional | burstSize) != 0);
  Field_setVal32(p->mpFldVportQosDataEnable, enable);
  Field_setVal32(p->mpFldVportQosDataInformationRate, informationRate);
  Field_setVal32(p->mpFldVportQosDataInformationRateFractional, informationRateFractional);
  Field_setVal32(p->mpFldVportQosDataBurstSize, burstSize);
  Register_flush(p->mpRegVportQosData, 1);

  return 0;
}

int NthwEpp_setQueueToVport(NthwEpp_t *p, uint32_t qid, uint32_t vport)
{
  if(p == NULL)
    return 0;

  Field_setVal32(p->mpFldQueueVportControlAdr, qid);
  Field_setVal32(p->mpFldQueueVportControlCnt, 1);
  Register_flush(p->mpRegQueueVportControl, 1);

  Field_setVal32(p->mpFldQueueVportDataVport, vport);
  Register_flush(p->mpRegQueueVportData, 1);
  return 0;
}

//
// EOF
//
