//
// nthw_mcu.c
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

//
// Includes:
//

//
#include "ntlog.h"
//
#include "nthw_drv.h"
#include "nthw_register.h"

//
//
//
#include "nthw_mcu.h"

//
//
//
NthwMcu_t* NthwMcu_new(void)
{
  NthwMcu_t* p = malloc(sizeof(NthwMcu_t));
  if (p) {
    memset(p, 0, sizeof(NthwMcu_t));
  }
  return p;
}


//
//
//
void NthwMcu_delete(NthwMcu_t* p)
{
  if (p) {
    memset(p, 0, sizeof(NthwMcu_t));
    free(p);
  }
}

//
//
//
int NthwMcu_init(NthwMcu_t* p, Fpga_t* pFpga, int nInstance)
{
  struct Module_s* mod = Fpga_queryModule(pFpga, MOD_MCU, nInstance);

  if (p == NULL) {
    return mod == NULL ? -1 : 0;
  }

  if (mod == NULL) {
    NT_LOG(ERR, NTHW, "%s: MCU %d: no such instance\n", pFpga->pFpgaInfo->mpAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mnInstance = nInstance;
  p->mpModMcu = mod;

#if defined(DEBUG) && (1)
  Module_setDebugMode(p->mpModMcu, 0xff); // DEBUG:
#endif

  p->mnModuleMajorVersion = Module_getMajorVersion(p->mpModMcu);
  p->mnModuleMinorVersion = Module_getMinorVersion(p->mpModMcu);
  p->mnModuleVersionPacked64 = Module_getVersionPacked64(p->mpModMcu);

  //
  p->mb_McuPresent = Fpga_getProductParam(pFpga, NT_MCU_PRESENT, 0);
  p->mn_McuType = Fpga_getProductParam(pFpga, NT_MCU_TYPE, -1);
  p->mn_McuDramSize = Fpga_getProductParam(pFpga, NT_MCU_DRAM_SIZE, 0);

  //
  p->mpRegCsr = Module_getRegister(p->mpModMcu, MCU_CSR);
  p->mpFldCsrHalt = Register_getField(p->mpRegCsr, MCU_CSR_HALT);
  p->mpFldCsrPause = Register_getField(p->mpRegCsr, MCU_CSR_PAUSE);
  p->mpFldCsrReset = Register_getField(p->mpRegCsr, MCU_CSR_RESET_MCU);

  //
  p->mpRegDramCtrl = Module_getRegister(p->mpModMcu, MCU_DRAM_CTRL);
  p->mpFldDramCtrlAdr = Register_getField(p->mpRegDramCtrl, MCU_DRAM_CTRL_ADR);
  p->mpFldDramCtrlCnt = Register_getField(p->mpRegDramCtrl, MCU_DRAM_CTRL_CNT);

  //
  p->mpRegDramRdData = Module_getRegister(p->mpModMcu, MCU_DRAM_RD_DATA);
  p->mpFldDramRdDataData = Register_getField(p->mpRegDramRdData, MCU_DRAM_RD_DATA_DATA);

  //
  p->mpRegDramWrData = Module_getRegister(p->mpModMcu, MCU_DRAM_WR_DATA);
  p->mpFldDramWrDataData = Register_getField(p->mpRegDramWrData, MCU_DRAM_WR_DATA_DATA);

  //
  p->mpRegIramCtrl = Module_getRegister(p->mpModMcu, MCU_IRAM_CTRL);
  p->mpFldIramCtrlAdr = Register_getField(p->mpRegIramCtrl, MCU_IRAM_CTRL_ADR);
  p->mpFldIramCtrlCnt = Register_getField(p->mpRegIramCtrl, MCU_IRAM_CTRL_CNT);

  //
  p->mpRegIramData = Module_getRegister(p->mpModMcu, MCU_IRAM_DATA);
  p->mpFldIramDataData = Register_getField(p->mpRegIramData, MCU_IRAM_DATA_DATA);

  //
  return 0;
}

//
//
void NthwMcu_setHalt(NthwMcu_t* p, uint32_t value)
{
  Field_setValFlush32(p->mpFldCsrHalt, value);
}

//
//
//
uint32_t NthwMcu_getHalt(NthwMcu_t* p)
{
  return Field_getVal32(p->mpFldCsrHalt);
}

//
//
//
void NthwMcu_setPause(NthwMcu_t* p, uint32_t value)
{
  Field_setValFlush32(p->mpFldCsrPause, value);
}

//
//
//
uint32_t NthwMcu_getPause(NthwMcu_t* p)
{
  return Field_getVal32(p->mpFldCsrPause);
}

//
//
//
void NthwMcu_setReset(NthwMcu_t* p, uint32_t value)
{
  Field_setValFlush32(p->mpFldCsrReset, value);
}

//
//
//
uint32_t NthwMcu_getReset(NthwMcu_t* p)
{
  return Field_getVal32(p->mpFldCsrReset);
}

//
// NOTE: currently not available
//
#if (0)
int NthwMcu_IramRead(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt)
{
  uint32_t* pData32 = (uint32_t*) pData;
  uint32_t nData32;
  int i;

  Field_setVal32(p->mpFldIramCtrlAdr, nAddr);
  Field_setValFlush32(p->mpFldIramCtrlCnt, ((nDataCnt+1)>>2));

  for (i = 0; i < ((nDataCnt+1)>>2); i++) {
    pData32[i] = Field_getUpdated(p->mpFldIramDataData);
  }

  return -1;
}
#endif

//
//
//
int NthwMcu_IramWrite(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt)
{
  uint32_t* pData32 = (uint32_t*) pData;
  int i;

  Field_setVal32(p->mpFldIramCtrlAdr, nAddr);
  Field_setValFlush32(p->mpFldIramCtrlCnt, ((nDataCnt+1)>>2));

  for (i = 0; i < ((nDataCnt+1)>>2); i++) {
    Field_setValFlush32(p->mpFldIramDataData, pData32[i]);
  }

  return 0;
}

//
//
//
int NthwMcu_DramRead(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt)
{
  uint32_t* pData32 = (uint32_t*) pData;
  int i;

  Field_setVal32(p->mpFldDramCtrlAdr, nAddr);
  Field_setValFlush32(p->mpFldDramCtrlCnt, ((nDataCnt+1)>>2));

  for (i = 0; i < ((nDataCnt+1)>>2); i++) {
    pData32[i] = Field_getUpdated(p->mpFldDramRdDataData);
  }

  return 0;
}

//
//
//
int NthwMcu_DramWrite(NthwMcu_t *p, uint32_t nAddr, uint8_t* pData, int nDataCnt)
{
  uint32_t* pData32 = (uint32_t*) pData;
  int i;

  Field_setVal32(p->mpFldDramCtrlAdr, nAddr);
  Field_setValFlush32(p->mpFldDramCtrlCnt, ((nDataCnt+1)>>2));

  for (i = 0; i < ((nDataCnt+1)>>2); i++) {
    Field_setValFlush32(p->mpFldDramWrDataData, pData32[i]);
  }
  return 0;
}


//
// EOF
//
