//
// nthw_rab.h
//

/*
 * %NT_SOFTWARE_LICENSE%
 */

//
//
//

#ifndef __NTHW_RAC_H__
#define __NTHW_RAC_H__

//
//
//

//typedef uint8_t RabBusId_t; // currently used typedef from nthw/include/nthw_bus.h

#include "nt_util.h"
#include "nthw_bus.h"

//
//
//
struct NthwRac
{
  //
  Fpga* mpFpga;
  //
  Module* mpModRac;
  //
  pthread_mutex_t mMutex;
  //
  int mnParamRacRabInterfaces;
  int mnParamRacRabObUpdate;
  //
  Register* mpRegDummy0;
  Register* mpRegDummy1;
  Register* mpRegDummy2;
  //
  Register* mpRegRabInit;
  Field* mpFldRabInit;
  //
  int mnFldRabInitBw;
  uint32_t mnFldRabInitMask;
  //
  Register* mpRegDbgCtrl;
  Field* mpFldDbgCtrl;
  //
  Register* mpRegDbgData;
  Field* mpFldDbgData;
  //
  Register* mpRegRabIBData;
  Field* mpFldRabIBData;
  //
  Register* mpRegRabOBData;
  Field* mpFldRabOBData;
  //
  Register* mpRegRabBufFree;
  Field* mpFldRabBufFreeIBFree;
  Field* mpFldRabBufFreeIBOvf;
  Field* mpFldRabBufFreeOBFree;
  Field* mpFldRabBufFreeOBOvf;
  Field* mpFldRabBufFreeTimeout;
  //
  Register* mpRegRabBufUsed;
  Field* mpFldRabBufUsedIBUsed;
  Field* mpFldRabBufUsedOBUsed;
  Field* mpFldRabBufUsedFlush;
  //
  Register* mpRegRabDmaIbLo;
  Field* mpFldRabDmaIbLoPhyAddr;
  //
  Register* mpRegRabDmaIbHi;
  Field* mpFldRabDmaIbHiPhyAddr;
  //
  Register* mpRegRabDmaObHi;
  Field* mpFldRabDmaObHiPhyAddr;
  //
  Register* mpRegRabDmaObLo;
  Field* mpFldRabDmaObLoPhyAddr;
  //
  Register* mpRegRabDmaIbWr;
  Field* mpFldRabDmaIbWrPtr;
  //
  Register* mpRegRabDmaIbRd;
  Field* mpFldRabDmaIbRdPtr;
  //
  Register* mpRegRabDmaObWr;
  Field* mpFldRabDmaObWrPtr;
  //
  // NMB:
  //
  Register* mpRegRabNmbRd;
  Register* mpRegRabNmbData;
  Register* mpRegRabNmbWr;
  Register* mpRegRabNmbStatus;

  //
  //
  //
  uint32_t RAC_RAB_INIT_ADDR;
  uint32_t RAC_RAB_IB_DATA_ADDR;
  uint32_t RAC_RAB_OB_DATA_ADDR;
  uint32_t RAC_RAB_BUF_FREE_ADDR;
  uint32_t RAC_RAB_BUF_USED_ADDR;

  //
  //
  //
  uint32_t RAC_RAB_DMA_IB_LO_ADDR;
  uint32_t RAC_RAB_DMA_IB_HI_ADDR;
  uint32_t RAC_RAB_DMA_OB_LO_ADDR;
  uint32_t RAC_RAB_DMA_OB_HI_ADDR;
  uint32_t RAC_RAB_DMA_IB_RD_ADDR;
  uint32_t RAC_RAB_DMA_OB_WR_ADDR;
  uint32_t RAC_RAB_DMA_IB_WR_ADDR;

  //
  //
  //
  uint32_t RAC_RAB_BUF_FREE_IB_FREE_MASK;
  uint32_t RAC_RAB_BUF_FREE_OB_FREE_MASK;
  uint32_t RAC_RAB_BUF_USED_IB_USED_MASK;
  uint32_t RAC_RAB_BUF_USED_OB_USED_MASK;
  uint32_t RAC_RAB_BUF_USED_FLUSH_MASK;
  //
  uint32_t RAC_RAB_BUF_USED_OB_USED_LOW;
  //
  // NMB
  //
  uint32_t RAC_NMB_RD_ADR_ADDR;
  uint32_t RAC_NMB_DATA_ADDR;
  uint32_t RAC_NMB_WR_ADR_ADDR;
  uint32_t RAC_NMB_STATUS_ADDR;
  //
  //
  //
  bool mDmaActive;
  //
  struct nt_dma_s *mDma;
  //
  volatile uint32_t* mDmaInBuf;
  volatile uint32_t* mDmaOutBuf;
  //
  uint16_t mDmaOutPtrRd;
  uint16_t mDmaInPtrWr;
  uint32_t mInFree;


};

typedef struct NthwRac NthwRac_t;
typedef struct NthwRac NthwRac;

struct DmaBufPtr {
  uint32_t size;
  uint32_t index;
  volatile uint32_t *base;
};

//
//
//

NthwRac_t* NthwRac_new(void);
void NthwRac_delete(NthwRac_t* p);
int NthwRac_init(NthwRac_t *p, Fpga_t *pFpga, struct FpgaInfo_s* pFpgaInfo);
//
int NthwRac_get_rab_interface_count(const NthwRac_t *p);
//
int NthwRac_rab_init(NthwRac_t* p, uint32_t rabIntfMask);
//
int NthwRac_rab_setup(NthwRac_t* p);
//
int NthwRac_rab_reset(NthwRac_t* p);
//
//
int NthwRac_rab_write32(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, const uint32_t *pData);
int NthwRac_rab_write32_dma(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, const uint32_t *pData);
int NthwRac_rab_read32(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, uint32_t* pData);
int NthwRac_rab_read32_dma(NthwRac_t* p, uint32_t address, RabBusId_t busId, uint32_t wordCnt, struct DmaBufPtr* bufPtr);
//
int NthwRac_rab_flush(NthwRac_t* p);
//
int NthwRac_rab_dma_begin(NthwRac_t* p);
int NthwRac_rab_dma_commit(NthwRac_t* p);

void NthwRac_regRead32(const struct FpgaInfo_s *pFpgaInfo, uint32_t regAddr, uint32_t *pData);
void NthwRac_regWrite32(const struct FpgaInfo_s *pFpgaInfo, uint32_t regAddr, uint32_t pData);

#endif // __NTHW_RAC_H__

//
// EOF
//
