from trex_stl_lib.api import *
from scapy.layers.inet import Ether, IP, UDP
# scapy == v2.4.3
from scapy.contrib.gtp import GTP_U_Header, Raw
# scapy >= v2.4.4
# from scapy.contrib.gtp import GTP_U_Header, GTPPDUSessionContainer, Raw


trex_client = STLClient(server = '127.0.0.1')
trex_client.connect()

# # create a UL base packet with scapy >= v2.4.4
# uplink_pkt = Ether(src = "16:16:16:16:16:16", dst = "1a:1a:1a:1a:1a:1a") \
#           / IP(src = "20.0.0.0", dst = "21.0.0.0") \
#           / UDP(dport=2152) \
#           / GTP_U_Header(teid = 0xcafe101)
#           / GTPPDUSessionContainer(type = 1, QFI = 6) \
#           / IP(src = "20.0.0.0", dst = "21.0.0.0") \
#           / UDP(sport = 4096, dport = 4097) \
#           / Raw("NapaTech inline UPF demo test")


# create a UL base packet with scapy == v2.4.3
# Extension header (PDU Session container)
# Extension Header Length: 1
# PDU Session Container
# 0001 .... = PDU Type: UL PDU SESSION INFORMATION (1)
# .... 0000 = Spare: 0x0
# 00.. .... = Spare: 0x0
# ..00 0110 = QoS Flow Identifier (QFI): 6
# Next extension header type: No more extension headers (0x00)

GTP_ext_h = [
              0x01, # Extension Header Length:
              0x10, # PDU Type: UL PDU SESSION INFORMATION (1)
              0x06, # QoS Flow Identifier (QFI): 6
              0x00, # Next extension header type
            ]

# gtp_type it is GTP message type = 0xFF(255) T-PDU
# next_ex it is next extension header type = 0x85(133) - PDU session container
# PT it is GTP Protocol type = GTP 1
# E it is flag (is next extension header present) = 1 - ext header presents
ENCAP_H = Ether(src = "16:16:16:16:16:16", dst = "1a:1a:1a:1a:1a:1a") \
    / IP(src = "20.0.0.0", dst = "21.0.0.0") \
    / UDP(dport=2152) \
    / GTP_U_Header(PT=1, E=1, gtp_type=0xFF, teid=0xcafe101, next_ex=0x85) \
    / Raw(GTP_ext_h)

CLIENT_FLOW= IP(src = "20.0.0.0", dst = "21.0.0.0") \
            / UDP(sport = 4096, dport = 4097) \
            / Raw("NapaTech inline UPF demo test")

uplink_pkt = ENCAP_H / CLIENT_FLOW

# create a DL base packet
downlink_pkt = Ether(src = "16:16:16:16:16:16", dst = "1a:1a:1a:1a:1a:1a") \
            / CLIENT_FLOW

PACKET_SIZE = 512  # Bytes
N3_PORT = 0
N6_PORT = 1

TEST_DURATION = 100 # seconds

def pad(base_pkt):
  padding = (PACKET_SIZE - len(base_pkt)) * '.'
  return padding

try:

  # Inctruction to generate range of L3 flows for N3 UPF interafce
  UL_flows_ins_A = [
                        STLVmFlowVar ( "ip_dst",
                                          min_value="21.0.0.0",
                                          max_value="21.0.0.254",
                                          size=4,
                                          step=1,
                                          op="inc"
                                          # op="random"
                                          ),
                        # write ip to packet IP.dst
                        STLVmWrFlowVar (fv_name="ip_dst",
                                        pkt_offset= "IP.dst" ),

                        STLVmFlowVar ( "inner_ip_dst",
                                          min_value="21.0.0.0",
                                          max_value="21.0.0.254",
                                          size=4,
                                          step=1,
                                          op="inc"
                                          # op="random"
                                          ),
                        # write ip to packet inner IP.dst
                        # 16 - offset to inner IPv4 address in the IPv4 header
                        STLVmWrFlowVar (fv_name="inner_ip_dst",
                                        pkt_offset = len(ENCAP_H) + 16 ),

                        STLVmFixIpv4(offset = "IP"),
                        STLVmFixChecksumHw(l3_offset= len(ENCAP_H),
                                          l4_offset = len(ENCAP_H) + 20,
                                          l4_type=CTRexVmInsFixHwCs.L4_TYPE_UDP)
                     ]

  UL_flows_ins_B = [
                      STLVmFlowVar ( "ip_dst",
                                        min_value="21.0.0.255",
                                        max_value="21.0.254.255",
                                        size=4,
                                        step=1,
                                        op="inc"
                                        # op="random"
                                        ),
                      # write ip to packet IP.dst
                      STLVmWrFlowVar (fv_name="ip_dst",
                                      pkt_offset= "IP.dst" ),

                      STLVmFlowVar ( "inner_ip_dst",
                                        min_value="21.0.0.255",
                                        max_value="21.0.254.255",
                                        size=4,
                                        step=1,
                                        op="inc"
                                        # op="random"
                                        ),
                      # write ip to packet inner IP.dst
                      # 16 - offset to inner IPv4 address in the IPv4 header
                      STLVmWrFlowVar (fv_name="inner_ip_dst",
                                      pkt_offset = len(ENCAP_H) + 16 ),

                        STLVmFixIpv4(offset = "IP"),
                        STLVmFixChecksumHw(l3_offset= len(ENCAP_H),
                                          l4_offset = len(ENCAP_H) + 20,
                                          l4_type=CTRexVmInsFixHwCs.L4_TYPE_UDP)
                    ]

  UL_flows_ins_C = [
                      STLVmFlowVar ( "ip_dst",
                                        min_value="21.0.255.255",
                                        max_value="21.255.255.255",
                                        size=4,
                                        step=1,
                                        op="inc"
                                        # op="random"
                                        ),
                      # write ip to packet IP.dst
                      STLVmWrFlowVar (fv_name="ip_dst",
                                      pkt_offset= "IP.dst" ),

                      STLVmFlowVar ( "inner_ip_dst",
                                        min_value="21.0.255.255",
                                        max_value="21.255.255.255",
                                        size=4,
                                        step=1,
                                        op="inc"
                                        # op="random"
                                        ),
                      # write ip to packet inner IP.dst
                      # 16 - offset to inner IPv4 address in the IPv4 header
                      STLVmWrFlowVar (fv_name="inner_ip_dst",
                                      pkt_offset = len(ENCAP_H) + 16 ),

                      STLVmFixIpv4(offset = "IP"),
                      STLVmFixChecksumHw(l3_offset= len(ENCAP_H),
                                        l4_offset = len(ENCAP_H) + 20,
                                        l4_type=CTRexVmInsFixHwCs.L4_TYPE_UDP)
                    ]

  # Inctruction to generate range of L3 flows for N3 UPF interafce
  DL_flows_ins_A = [
                        STLVmFlowVar ( "ip_dst",
                                      min_value="21.0.0.0",
                                      max_value="21.0.0.255",
                                      size=4,
                                      step=1,
                                      op="inc"
                                      # op="random"
                                      ),
                      # write ip to packet IP.dst
                      STLVmWrFlowVar (fv_name="ip_dst",
                                      pkt_offset= "IP.dst" ),

                      STLVmFixIpv4(offset = "IP")
                        ]

  DL_flows_ins_B = [
                        STLVmFlowVar ( "ip_dst",
                                      min_value="21.0.0.255",
                                      max_value="21.0.254.255",
                                      size=4,
                                      step=1,
                                      op="inc"
                                      # op="random"
                                      ),
                      # write ip to packet IP.dst
                      STLVmWrFlowVar (fv_name="ip_dst",
                                      pkt_offset= "IP.dst" ),

                      STLVmFixIpv4(offset = "IP")
                        ]


  DL_flows_ins_C = [
                        STLVmFlowVar ( "ip_dst",
                                      min_value="21.0.255.255",
                                      max_value="21.255.255.255",
                                      size=4,
                                      step=1,
                                      op="inc"
                                      # op="random"
                                      ),
                      # write ip to packet IP.dst
                      STLVmWrFlowVar (fv_name="ip_dst",
                                      pkt_offset= "IP.dst" ),

                      STLVmFixIpv4(offset = "IP")
                        ]

  vm_ul_A = STLScVmRaw(UL_flows_ins_A)
  vm_ul_B = STLScVmRaw(UL_flows_ins_B)
  vm_ul_C = STLScVmRaw(UL_flows_ins_C)

  vm_dl_A = STLScVmRaw(DL_flows_ins_A)
  vm_dl_B = STLScVmRaw(DL_flows_ins_B)
  vm_dl_C = STLScVmRaw(DL_flows_ins_C)

  uplink_flow_A = STLPktBuilder(uplink_pkt/pad(uplink_pkt),vm=vm_ul_A)
  uplink_flow_B = STLPktBuilder(uplink_pkt/pad(uplink_pkt),vm=vm_ul_B)
  uplink_flow_C = STLPktBuilder(uplink_pkt/pad(uplink_pkt),vm=vm_ul_C)

  downlink_flow_A = STLPktBuilder(downlink_pkt/pad(downlink_pkt),vm=vm_dl_A)
  downlink_flow_B = STLPktBuilder(downlink_pkt/pad(downlink_pkt),vm=vm_dl_B)
  downlink_flow_C = STLPktBuilder(downlink_pkt/pad(downlink_pkt),vm=vm_dl_C)

  ul_stream_A = STLStream(packet=uplink_flow_A,
                        mode=STLTXCont(percentage = 33)) # 33% from full link
  ul_stream_B = STLStream(packet=uplink_flow_B,
                        mode=STLTXCont(percentage = 33))
  ul_stream_C = STLStream(packet=uplink_flow_C,
                        mode=STLTXCont(percentage = 33))

  dl_stream_A = STLStream(packet=downlink_flow_A,
                        mode=STLTXCont(percentage = 33)) # 33% from full link
  dl_stream_B = STLStream(packet=downlink_flow_B,
                        mode=STLTXCont(percentage = 33))
  dl_stream_C = STLStream(packet=downlink_flow_C,
                        mode=STLTXCont(percentage = 33))

  print("Test duration = ", str(TEST_DURATION) + " seconds")

  print("Press CTRL-C to stop...")

  trex_client.reset(ports = [N3_PORT, N6_PORT])

  trex_client.add_streams(ul_stream_A, ports = [N3_PORT])
  trex_client.add_streams(ul_stream_B, ports = [N3_PORT])
  trex_client.add_streams(ul_stream_C, ports = [N3_PORT])

  trex_client.add_streams(dl_stream_A, ports = [N6_PORT])
  trex_client.add_streams(dl_stream_B, ports = [N6_PORT])
  trex_client.add_streams(dl_stream_C, ports = [N6_PORT])

  trex_client.start(ports = [N3_PORT, N6_PORT],
                  duration=TEST_DURATION)

  trex_client.wait_on_traffic()
  # trex_client.show_stats_line()

except STLError as e:
  print(e)

finally:
    trex_client.disconnect()
