from trex_stl_lib.api import *
from scapy.layers.inet import Ether, IP, UDP
# scapy == v2.4.3
from scapy.contrib.gtp import GTP_U_Header, Raw
# scapy >= v2.4.4
# from scapy.contrib.gtp import GTP_U_Header, GTPPDUSessionContainer, Raw


trex_client = STLClient(server = '127.0.0.1')
trex_client.connect()

# # create a UL base packet with scapy >= v2.4.4
# uplink_pkt = Ether(src = "16:16:16:16:16:16", dst = "1a:1a:1a:1a:1a:1a") \
#           / IP(src = "20.0.0.0", dst = "21.0.0.0") \
#           / UDP(dport=2152) \
#           / GTP_U_Header(teid = 0xcafe101)
#           / GTPPDUSessionContainer(type = 1, QFI = 6) \
#           / IP(src = "20.0.0.0", dst = "21.0.0.0") \
#           / UDP(sport = 4096, dport = 4097) \
#           / Raw("NapaTech inline UPF demo test")


# create a UL base packet with scapy == v2.4.3
# Extension header (PDU Session container)
# Extension Header Length: 1
# PDU Session Container
# 0001 .... = PDU Type: UL PDU SESSION INFORMATION (1)
# .... 0000 = Spare: 0x0
# 00.. .... = Spare: 0x0
# ..00 0110 = QoS Flow Identifier (QFI): 6
# Next extension header type: No more extension headers (0x00)

GTP_ext_h = [
              0x01, # Extension Header Length:
              0x10, # PDU Type: UL PDU SESSION INFORMATION (1)
              0x06, # QoS Flow Identifier (QFI): 6
              0x00, # Next extension header type
            ]


N3_IF_ADDR = "10.10.10.3"
N3_IF_PEER_S = "10.10.10.2"
N3_IF_PEER_E = "10.10.10.63"

UE_ADDR_S = "192.168.0.2"
UE_ADDR_E = "192.168.0.255"
SERVER_UE = "1.1.1.2"

TEID_S = 1
TEID_E = 0xffffffff

N6_IF_ADDR = "20.20.0.0" # RSS queue id 0
# N6_IF_ADDR = "20.21.0.0" # RSS queue id 1
# N6_IF_ADDR = "20.22.0.0" # RSS queue id 2
# N6_IF_ADDR = "20.23.0.0" # RSS queue id 3

NAT_PORT_S = 1024
NAT_PORT_E = 65535

PACKET_SIZE = 512  # Bytes
N3_PORT = 0
N6_PORT = 1

TEST_DURATION = 100 # seconds


# gtp_type it is GTP message type = 0xFF(255) T-PDU
# next_ex it is next extension header type = 0x85(133) - PDU session container
# PT it is GTP Protocol type = GTP 1
# E it is flag (is next extension header present) = 1 - ext header presents
GTP_U =  GTP_U_Header(PT=1, E=1, gtp_type=0xFF, teid=0xcafe101, next_ex=0x85) \
        / Raw(GTP_ext_h)

OUTER_H = Ether(src = "16:16:16:16:16:16", dst = "1a:1a:1a:1a:1a:1a") \
    / IP(src = N3_IF_PEER_S, dst = N3_IF_ADDR) \
    / UDP(sport=2152, dport=2152) \
    / GTP_U


CLIENT_FLOW_UL= IP(src = UE_ADDR_S, dst = SERVER_UE) \
            / UDP(sport = 4096, dport = 4097) \
            / Raw("NapaTech inline UPF demo test")

SERVER_FLOW_DL= IP(src = SERVER_UE, dst = N6_IF_ADDR) \
            / UDP(sport = 4096, dport = 4097) \
            / Raw("NapaTech inline UPF demo test")



uplink_pkt = OUTER_H / CLIENT_FLOW_UL

# create a DL base packet
downlink_pkt = Ether(src = "18:18:18:18:18:18", dst = "17:17:17:17:17:17") \
            / SERVER_FLOW_DL

def pad(base_pkt):
  padding = (PACKET_SIZE - len(base_pkt)) * '.'
  return padding

try:

  # Inctruction to generate range of L3 flows for N3 UPF interafce
  UL_flows_ins_A = [
                        STLVmFlowVar ( "ip_src",
                                          min_value=N3_IF_PEER_S,
                                          max_value=N3_IF_PEER_E,
                                          size=4,
                                          step=1,
                                          op="inc"
                                          # op="random"
                                          ),
                        # write ip to packet IP.dst
                        STLVmWrFlowVar (fv_name="ip_src",
                                        pkt_offset= "IP.src" ),

                        STLVmFlowVar ( "teid",
                                          min_value=TEID_S,
                                          max_value=16384,
                                          size=4,
                                          step=1,
                                          op="inc"
                                          # op="random"
                                          ),
                        # write ip to packet inner gtp teid
                        # -12 - offset to teid from end of outer header
                        STLVmWrFlowVar (fv_name="teid",
                                        pkt_offset = len(OUTER_H) - 12 ),

                        STLVmFlowVar ( "inner_ip_src",
                                          min_value=UE_ADDR_S,
                                          max_value="192.168.255.255",
                                          size=4,
                                          step=1,
                                          op="inc"
                                          # op="random"
                                          ),
                        # write ip to packet inner IP.src
                        # 12 - offset to inner IPv4 address in the IPv4 header
                        STLVmWrFlowVar (fv_name="inner_ip_src",
                                        pkt_offset = len(OUTER_H) + 12 ),

                      STLVmFixIpv4(offset = "IP"),
                      STLVmFixChecksumHw(l3_offset= len(ENCAP_H),
                                        l4_offset = len(ENCAP_H) + 20,
                                        l4_type=CTRexVmInsFixHwCs.L4_TYPE_UDP)
                     ]

  # Inctruction to generate range of L4 flows for N6 UPF interafce
  DL_flows_ins_A = [
                        STLVmFlowVar ( "udp_dst",
                                      min_value=NAT_PORT_S,
                                      max_value=NAT_PORT_E,
                                      size=2,
                                      step=1,
                                      op="inc"
                                      # op="random"
                                      ),
                      # write ip to packet IP.dst
                      STLVmWrFlowVar (fv_name="udp_dst",
                                      pkt_offset = 34 ),

                      STLVmFixIpv4(offset = "IP"),
                        ]


  vm_ul_A = STLScVmRaw(UL_flows_ins_A)
  vm_dl_A = STLScVmRaw(DL_flows_ins_A)

  uplink_flow_A = STLPktBuilder(uplink_pkt/pad(uplink_pkt),vm=vm_ul_A)
  downlink_flow_A = STLPktBuilder(uplink_pkt/pad(uplink_pkt),vm=vm_dl_A)

  ul_stream_A = STLStream(packet=uplink_flow_A,
                        mode=STLTXCont(percentage = 1)) # 100% from full link

  dl_stream_A = STLStream(packet=downlink_flow_A,
                        mode=STLTXCont(percentage = 1)) # 100% from full link



  print("Test duration = ", str(TEST_DURATION) + " seconds")

  print("Press CTRL-C to stop...")

  trex_client.reset(ports = [N3_PORT])

  trex_client.add_streams(ul_stream_A, ports = [N3_PORT])
  # trex_client.add_streams(dl_stream_A, ports = [N6_PORT])

  trex_client.start(ports = [N3_PORT],
                  duration=TEST_DURATION)

  trex_client.wait_on_traffic()
  # trex_client.show_stats_line()

except STLError as e:
  print(e)

finally:
    trex_client.disconnect()
